@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 - 2005 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm, Török Edvin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -42,41 +44,29 @@
 
 #include "libclamav/clamav.h"
 
-#include "shared/options.h"
-#include "shared/cfgparser.h"
+#include "shared/optparser.h"
 #include "shared/output.h"
 #include "shared/network.h"
 
 #include "others.h"
 #include "server.h"
 #include "tcpserver.h"
-
+#include "misc.h"
 #ifndef	C_WINDOWS
 #define	closesocket(s)	close(s)
 #endif
 
-int tcpserver(const struct cfgstruct *copt)
+int tcpserver(const struct optstruct *opts)
 {
-	struct sockaddr_in server;
-	int sockfd, backlog;
-	const struct cfgstruct *taddr;
-	struct hostent he;
-	char *estr, buf[1024];
-	int true = 1;
-
-    memset((char *) &server, 0, sizeof(server));
-    server.sin_family = AF_INET;
-    server.sin_port = htons(cfgopt(copt, "TCPSocket")->numarg);
-
-    if((taddr = cfgopt(copt, "TCPAddr"))->enabled) {
-	if(r_gethostbyname(taddr->strarg, &he, buf, sizeof(buf)) == -1) {
-	    logg("!TCP: r_gethostbyname(%s) error: %s\n", taddr->strarg, strerror(errno));
-	    return -1;
-	}
-	server.sin_addr = *(struct in_addr *) he.h_addr_list[0];
-    } else
-	server.sin_addr.s_addr = INADDR_ANY;
+    struct sockaddr_in server;
+    int sockfd, backlog;
+    struct hostent he;
+    char *estr, buf[1024];
+    int true = 1;
 
+    if (cfg_tcpsock(opts, &server, INADDR_ANY) == -1) {
+	return -1;
+    }
 
     if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
 	estr = strerror(errno);
@@ -94,13 +84,14 @@ int tcpserver(const struct cfgstruct *copt)
 	closesocket(sockfd);
 	return -1;
     } else {
+	const struct optstruct *taddr = optget(opts, "TCPAddr");
 	if(taddr->enabled)
-	    logg("#TCP: Bound to address %s on port %u\n", taddr->strarg, cfgopt(copt, "TCPSocket")->numarg);
+	    logg("#TCP: Bound to address %s on port %u\n", taddr->strarg, optget(opts, "TCPSocket")->numarg);
 	else
-	    logg("#TCP: Bound to port %u\n", cfgopt(copt, "TCPSocket")->numarg);
+	    logg("#TCP: Bound to port %u\n", optget(opts, "TCPSocket")->numarg);
     }
 
-    backlog = cfgopt(copt, "MaxConnectionQueueLength")->numarg;
+    backlog = optget(opts, "MaxConnectionQueueLength")->numarg;
     logg("#TCP: Setting connection queue length to %d\n", backlog);
 
     if(listen(sockfd, backlog) == -1) {
