@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm, aCaB
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -19,10 +21,17 @@
 #ifndef __CLIENT_H
 #define __CLIENT_H
 
-#include "options.h"
+#include "shared/optparser.h"
 
-int client(const struct optstruct *opt, int *infected);
-int get_clamd_version(const struct optstruct *opt);
-int reload_clamd_database(const struct optstruct *opt);
+enum {
+    CONT,
+    MULTI,
+    STREAM,
+    FILDES
+};
+
+int client(const struct optstruct *opts, int *infected);
+int get_clamd_version(const struct optstruct *opts);
+int reload_clamd_database(const struct optstruct *opts);
 
 #endif
