@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 - 2006 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,14 +40,14 @@
 #include <sys/resource.h>
 #endif
 
-#include "clamscan_opt.h"
 #include "others.h"
 #include "global.h"
 #include "manager.h"
 
 #include "shared/misc.h"
 #include "shared/output.h"
-#include "shared/options.h"
+#include "shared/actions.h"
+#include "shared/optparser.h"
 
 #include "libclamav/str.h"
 #include "libclamav/clamav.h"
@@ -62,14 +64,14 @@ short recursion = 0, printinfected = 0, bell = 0;
 int main(int argc, char **argv)
 {
 	int ds, dms, ret;
-	double mb;
+	double mb, rmb;
 	struct timeval t1, t2;
 #ifndef C_WINDOWS
 	struct timezone tz;
 	sigset_t sigset;
 #endif
-	struct optstruct *opt;
-	const char *pt;
+	struct optstruct *opts;
+	const struct optstruct *opt;
 
 #if defined(C_WINDOWS) && defined(CL_THREAD_SAFE)
     if(!pthread_win32_process_attach_np()) {
@@ -84,25 +86,25 @@ int main(int argc, char **argv)
     sigprocmask(SIG_SETMASK, &sigset, NULL);
 #endif
 
-    opt = opt_parse(argc, argv, clamscan_shortopt, clamscan_longopt, NULL, clamscan_deprecated);
-    if(!opt) {
-	mprintf("!Can't parse the command line\n");
+
+    if((opts = optparse(NULL, argc, argv, 1, OPT_CLAMSCAN, 0, NULL)) == NULL) {
+	mprintf("!Can't parse command line options\n");
 	return 40;
     }
 
-    if(opt_check(opt, "verbose")) {
+    if(optget(opts, "verbose")->enabled) {
 	mprintf_verbose = 1;
 	logg_verbose = 1;
     }
 
-    if(opt_check(opt, "quiet"))
+    if(optget(opts, "quiet")->enabled)
 	mprintf_quiet = 1;
 
-    if(opt_check(opt, "stdout"))
+    if(optget(opts, "stdout")->enabled)
 	mprintf_stdout = 1;
 
 
-    if(opt_check(opt, "debug")) {
+    if(optget(opts, "debug")->enabled) {
 #if defined(C_LINUX)
 	    /* njh@bandsman.co.uk: create a dump if needed */
 	    struct rlimit rlim;
@@ -114,107 +116,42 @@ int main(int argc, char **argv)
 	cl_debug(); /* enable debug messages */
     }
 
-    if(opt_check(opt, "version")) {
-	print_version(opt_arg(opt, "database"));
-	opt_free(opt);
+    if(optget(opts, "version")->enabled) {
+	print_version(optget(opts, "database")->strarg);
+	optfree(opts);
 	return 0;
     }
 
-    if(opt_check(opt, "help")) {
-	opt_free(opt);
+    if(optget(opts, "help")->enabled) {
+	optfree(opts);
     	help();
 	return 0;
     }
 
-    if(opt_check(opt, "recursive"))
+    if(optget(opts, "recursive")->enabled)
 	recursion = 1;
 
-    if(opt_check(opt, "infected"))
+    if(optget(opts, "infected")->enabled)
 	printinfected = 1;
 
-    if(opt_check(opt, "bell"))
+    if(optget(opts, "bell")->enabled)
 	bell = 1;
 
-    if(opt_check(opt, "tempdir"))
-	cl_settempdir(opt_arg(opt, "tempdir"), 0);
-
-    if(opt_check(opt, "leave-temps"))
-	cl_settempdir(NULL, 1);
-
     /* initialize logger */
-    if(opt_check(opt, "log")) {
-	logg_file = opt_arg(opt, "log");
+    if((opt = optget(opts, "log"))->enabled) {
+	logg_file = opt->strarg;
 	if(logg("#\n-------------------------------------------------------------------------------\n\n")) {
 	    mprintf("!Problem with internal logger.\n");
-	    opt_free(opt);
+	    optfree(opts);
 	    return 62;
 	}
     } else 
 	logg_file = NULL;
 
-
-    /* validate some numerical options */
-
-    if(opt_check(opt, "max-scansize")) {
-	pt = opt_arg(opt, "max-scansize");
-	if(!strchr(pt, 'M') && !strchr(pt, 'm')) {
-	    if(!cli_isnumber(pt)) {
-		logg("!--max-scansize requires a natural number\n");
-		opt_free(opt);
-		return 40;
-	    }
-	}
-    }
-
-    if(opt_check(opt, "max-filesize")) {
-	pt = opt_arg(opt, "max-filesize");
-	if(!strchr(pt, 'M') && !strchr(pt, 'm')) {
-	    if(!cli_isnumber(pt)) {
-		logg("!--max-filesize requires a natural number\n");
-		opt_free(opt);
-		return 40;
-	    }
-	}
-    }
-
-    if(opt_check(opt, "max-files")) {
-	if(!cli_isnumber(opt_arg(opt, "max-files"))) {
-	    logg("!--max-files requires a natural number\n");
-	    opt_free(opt);
-	    return 40;
-	}
-    }
-
-    if(opt_check(opt, "max-recursion")) {
-	if(!cli_isnumber(opt_arg(opt, "max-recursion"))) {
-	    logg("!--max-recursion requires a natural number\n");
-	    opt_free(opt);
-	    return 40;
-	}
-    }
-
-    if(opt_check(opt, "max-mail-recursion")) {
-	if(!cli_isnumber(opt_arg(opt, "max-mail-recursion"))) {
-	    logg("!--max-mail-recursion requires a natural number\n");
-	    opt_free(opt);
-	    return 40;
-	}
-    }
-
-    if(opt_check(opt, "max-dir-recursion")) {
-	if(!cli_isnumber(opt_arg(opt, "max-dir-recursion"))) {
-	    logg("!--max-dir-recursion requires a natural number\n");
-	    opt_free(opt);
-	    return 40;
-	}
-    }
-
-    if(opt_check(opt, "max-ratio")) {
-	if(!cli_isnumber(opt_arg(opt, "max-ratio"))) {
-	    logg("!--max-ratio requires a natural number\n");
-	    opt_free(opt);
-	    return 40;
-	}
+    if(actsetup(opts)) {
+	optfree(opts);
+	logg_close();
+	exit(2);
     }
 
     memset(&info, 0, sizeof(struct s_info));
@@ -232,9 +169,9 @@ int main(int argc, char **argv)
     gettimeofday(&t1, &tz);
 #endif
 
-    ret = scanmanager(opt);
+    ret = scanmanager(opts);
 
-    if(!opt_check(opt, "disable-summary") && !opt_check(opt, "no-summary")) {
+    if(!optget(opts, "no-summary")->enabled) {
 #ifdef C_WINDOWS
 	gettimeofday(&t2, NULL);
 #else
@@ -250,18 +187,20 @@ int main(int argc, char **argv)
 	logg("Scanned directories: %u\n", info.dirs);
 	logg("Scanned files: %u\n", info.files);
 	logg("Infected files: %u\n", info.ifiles);
-	if(info.notremoved) {
-	    logg("Not removed: %u\n", info.notremoved);
+	if(notremoved) {
+	    logg("Not removed: %u\n", notremoved);
 	}
-	if(info.notmoved) {
-	    logg("Not %s: %u\n", opt_check(opt, "copy") ? "moved" : "copied", info.notmoved);
+	if(notmoved) {
+	    logg("Not %s: %u\n", optget(opts, "copy")->enabled ? "moved" : "copied", notmoved);
 	}
 	mb = info.blocks * (CL_COUNT_PRECISION / 1024) / 1024.0;
 	logg("Data scanned: %2.2lf MB\n", mb);
+	rmb = info.rblocks * (CL_COUNT_PRECISION / 1024) / 1024.0;
+	logg("Data read: %2.2lf MB (ratio %.2f:1)\n", rmb, info.rblocks ? (double)info.blocks/(double)info.rblocks : 0);
 	logg("Time: %u.%3.3u sec (%u m %u s)\n", ds, dms/1000, ds/60, ds%60);
     }
 
-    opt_free(opt);
+    optfree(opts);
 
 #if defined(C_WINDOWS) && defined(CL_THREAD_SAFE)
     if(!pthread_win32_process_detach_np()) {
@@ -280,7 +219,8 @@ void help(void)
 
     mprintf("\n");
     mprintf("                       Clam AntiVirus Scanner %s\n", get_version());
-    mprintf("      (C) 2002 - 2007 ClamAV Team - http://www.clamav.net/team\n\n");
+    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
+    printf("           (C) 2007-2009 Sourcefire, Inc.\n\n");
 
     mprintf("    --help                -h             Print this help screen\n");
     mprintf("    --version             -V             Print version number\n");
@@ -291,15 +231,14 @@ void help(void)
     mprintf("    --no-summary                         Disable summary at end of scanning\n");
     mprintf("    --infected            -i             Only print infected files\n");
     mprintf("    --bell                               Sound bell on virus detection\n");
-
     mprintf("\n");
     mprintf("    --tempdir=DIRECTORY                  Create temporary files in DIRECTORY\n");
-    mprintf("    --leave-temps                        Do not remove temporary files\n");
+    mprintf("    --leave-temps[=yes/no(*)]            Do not remove temporary files\n");
     mprintf("    --database=FILE/DIR   -d FILE/DIR    Load virus database from FILE or load\n");
-    mprintf("                                         all .cvd and .db[2] files from DIR\n");
+    mprintf("                                         all supported db files from DIR\n");
     mprintf("    --log=FILE            -l FILE        Save scan report to FILE\n");
-    mprintf("    --recursive           -r             Scan subdirectories recursively\n");
-    mprintf("    --remove                             Remove infected files. Be careful!\n");
+    mprintf("    --recursive[=yes/no(*)]  -r          Scan subdirectories recursively\n");
+    mprintf("    --remove[=yes/no(*)]                 Remove infected files. Be careful!\n");
     mprintf("    --move=DIRECTORY                     Move infected files into DIRECTORY\n");
     mprintf("    --copy=DIRECTORY                     Copy infected files into DIRECTORY\n");
 #ifdef HAVE_REGEX_H
@@ -314,37 +253,38 @@ void help(void)
     mprintf("    --include-dir=PATT                   Only scan directories containing PATT\n");
 #endif
     mprintf("\n");
-    mprintf("    --detect-pua                         Detect Possibly Unwanted Applications\n");
+    mprintf("    --detect-pua[=yes/no(*)]             Detect Possibly Unwanted Applications\n");
     mprintf("    --exclude-pua=CAT                    Skip PUA sigs of category CAT\n");
     mprintf("    --include-pua=CAT                    Load PUA sigs of category CAT\n");
-    mprintf("    --detect-structured                  Detect structured data (SSN, Credit Card)\n");
+    mprintf("    --detect-structured[=yes/no(*)]      Detect structured data (SSN, Credit Card)\n");
     mprintf("    --structured-ssn-format=X            SSN format (0=normal,1=stripped,2=both)\n");
     mprintf("    --structured-ssn-count=N             Min SSN count to generate a detect\n");
     mprintf("    --structured-cc-count=N              Min CC count to generate a detect\n");
-    mprintf("    --no-mail                            Disable mail file support\n");
-    mprintf("    --no-phishing-sigs                   Disable signature-based phishing detection\n");
-    mprintf("    --no-phishing-scan-urls              Disable url-based phishing detection\n");
-    mprintf("    --heuristic-scan-precedence          Stop scanning as soon as a heuristic match is found\n");
-    mprintf("    --phishing-ssl                       Always block SSL mismatches in URLs (phishing module)\n");
-    mprintf("    --phishing-cloak                     Always block cloaked URLs (phishing module)\n");
-    mprintf("    --no-algorithmic                     Disable algorithmic detection\n");
-    mprintf("    --no-pe                              Disable PE analysis\n");
-    mprintf("    --no-elf                             Disable ELF support\n");
-    mprintf("    --no-ole2                            Disable OLE2 support\n");
-    mprintf("    --no-pdf                             Disable PDF support\n");
-    mprintf("    --no-html                            Disable HTML support\n");
-    mprintf("    --no-archive                         Disable archive support\n");
-    mprintf("    --detect-broken                      Try to detect broken executable files\n");
-    mprintf("    --block-encrypted                    Block encrypted archives\n");
-    mprintf("    --mail-follow-urls                   Download and scan URLs\n");
+    mprintf("    --scan-mail[=yes(*)/no]              Scan mail files\n");
+    mprintf("    --phishing-sigs[=yes(*)/no]          Signature-based phishing detection\n");
+    mprintf("    --phishing-scan-urls[=yes(*)/no]     URL-based phishing detection\n");
+    mprintf("    --heuristic-scan-precedence[=yes/no(*)] Stop scanning as soon as a heuristic match is found\n");
+    mprintf("    --phishing-ssl[=yes/no(*)]           Always block SSL mismatches in URLs (phishing module)\n");
+    mprintf("    --phishing-cloak[=yes/no(*)]         Always block cloaked URLs (phishing module)\n");
+    mprintf("    --algorithmic-detection[=yes(*)/no]  Algorithmic detection\n");
+    mprintf("    --scan-pe[=yes(*)/no]                Scan PE files\n");
+    mprintf("    --scan-elf[=yes(*)/no]               Scan ELF files\n");
+    mprintf("    --scan-ole2[=yes(*)/no]              Scan OLE2 containers\n");
+    mprintf("    --scan-pdf[=yes(*)/no]               Scan PDF files\n");
+    mprintf("    --scan-html[=yes(*)/no]              Scan HTML files\n");
+    mprintf("    --scan-archive[=yes(*)/no]           Scan archive files (supported by libclamav)\n");
+    mprintf("    --detect-broken[=yes/no(*)]          Try to detect broken executable files\n");
+    mprintf("    --block-encrypted[=yes/no(*)]        Block encrypted archives\n");
+    mprintf("    --mail-follow-urls[=yes/no(*)]       Download and scan URLs\n");
     mprintf("\n");
     mprintf("    --max-filesize=#n                    Files larger than this will be skipped and assumed clean\n");
-    mprintf("    --max-scansize=#n                    The maximum amount of data to scan for each container file (*)\n");
-    mprintf("    --max-files=#n                       The maximum number of files to scan for each container file (*)\n");
-    mprintf("    --max-recursion=#n                   Maximum archive recursion level for container file (*)\n");
+    mprintf("    --max-scansize=#n                    The maximum amount of data to scan for each container file (**)\n");
+    mprintf("    --max-files=#n                       The maximum number of files to scan for each container file (**)\n");
+    mprintf("    --max-recursion=#n                   Maximum archive recursion level for container file (**)\n");
     mprintf("    --max-dir-recursion=#n               Maximum directory recursion level\n");
 
     mprintf("\n");
-    mprintf("(*) Certain files (e.g. documents, archives, etc.) may in turn contain other\n");
-    mprintf("    files inside. The above options ensure safe processing of this kind of data.\n\n");
+    mprintf("(*) Default scan settings\n");
+    mprintf("(**) Certain files (e.g. documents, archives, etc.) may in turn contain other\n");
+    mprintf("   files inside. The above options ensure safe processing of this kind of data.\n\n");
 }
