@@ -26,6 +26,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+
+#include <assert.h>
 #ifdef	HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -38,9 +40,9 @@
 #include "cltypes.h"
 #include "str.h"
 #include "readdb.h"
+#include "default.h"
 
-uint8_t cli_ac_mindepth = AC_DEFAULT_MIN_DEPTH;
-uint8_t cli_ac_maxdepth = AC_DEFAULT_MAX_DEPTH;
+#include "mpool.h"
 
 int cli_ac_addpatt(struct cli_matcher *root, struct cli_ac_patt *pattern)
 {
@@ -59,14 +61,16 @@ int cli_ac_addpatt(struct cli_matcher *root, struct cli_ac_patt *pattern)
 	}
     }
 
-    if(len < root->ac_mindepth)
-	return CL_EPATSHORT;
+    if(len < root->ac_mindepth) {
+	/* cli_errmsg("cli_ac_addpatt: Signature for %s is too short\n", pattern->virname); */
+	return CL_EMALFDB;
+    }
 
     pt = root->ac_root;
 
     for(i = 0; i < len; i++) {
 	if(!pt->trans) {
-	    pt->trans = (struct cli_ac_node **) cli_calloc(256, sizeof(struct cli_ac_node *));
+	    pt->trans = (struct cli_ac_node **) mpool_calloc(root->mempool, 256, sizeof(struct cli_ac_node *));
 	    if(!pt->trans) {
 		cli_errmsg("cli_ac_addpatt: Can't allocate memory for pt->trans\n");
 		return CL_EMEM;
@@ -76,45 +80,42 @@ int cli_ac_addpatt(struct cli_matcher *root, struct cli_ac_patt *pattern)
 	next = pt->trans[(unsigned char) (pattern->pattern[i] & 0xff)]; 
 
 	if(!next) {
-	    next = (struct cli_ac_node *) cli_calloc(1, sizeof(struct cli_ac_node));
+	    next = (struct cli_ac_node *) mpool_calloc(root->mempool, 1, sizeof(struct cli_ac_node));
 	    if(!next) {
 		cli_errmsg("cli_ac_addpatt: Can't allocate memory for AC node\n");
 		return CL_EMEM;
 	    }
 
 	    if(i != len - 1) {
-		next->trans = (struct cli_ac_node **) cli_calloc(256, sizeof(struct cli_ac_node *));
+		next->trans = (struct cli_ac_node **) mpool_calloc(root->mempool, 256, sizeof(struct cli_ac_node *));
 		if(!next->trans) {
 		    cli_errmsg("cli_ac_addpatt: Can't allocate memory for next->trans\n");
-		    free(next);
+		    mpool_free(root->mempool, next);
 		    return CL_EMEM;
 		}
-	    } else {
-		next->leaf = 1;
 	    }
 
 	    root->ac_nodes++;
-	    newtable = cli_realloc(root->ac_nodetable, root->ac_nodes * sizeof(struct cli_ac_node *));
+	    newtable = mpool_realloc(root->mempool, root->ac_nodetable, root->ac_nodes * sizeof(struct cli_ac_node *));
 	    if(!newtable) {
 		root->ac_nodes--;
 		cli_errmsg("cli_ac_addpatt: Can't realloc ac_nodetable\n");
 		if(next->trans)
-		    free(next->trans);
-		free(next);
+		    mpool_free(root->mempool, next->trans);
+		mpool_free(root->mempool, next);
 		return CL_EMEM;
 	    }
 	    root->ac_nodetable = (struct cli_ac_node **) newtable;
 	    root->ac_nodetable[root->ac_nodes - 1] = next;
 
 	    pt->trans[(unsigned char) (pattern->pattern[i] & 0xff)] = next;
-	    pt->leaf = 0;
 	}
 
 	pt = next;
     }
 
     root->ac_patterns++;
-    newtable = cli_realloc(root->ac_pattable, root->ac_patterns * sizeof(struct cli_ac_patt *));
+    newtable = mpool_realloc(root->mempool, root->ac_pattable, root->ac_patterns * sizeof(struct cli_ac_patt *));
     if(!newtable) {
 	root->ac_patterns--;
 	cli_errmsg("cli_ac_addpatt: Can't realloc ac_pattable\n");
@@ -123,7 +124,6 @@ int cli_ac_addpatt(struct cli_matcher *root, struct cli_ac_patt *pattern)
     root->ac_pattable = (struct cli_ac_patt **) newtable;
     root->ac_pattable[root->ac_patterns - 1] = pattern;
 
-    pt->final = 1;
     pattern->depth = i;
 
     ph = pt->list;
@@ -249,14 +249,19 @@ static int ac_maketrans(struct cli_matcher *root)
     }
 
     while((node = bfs_dequeue(&bfs, &bfs_last))) {
-	if(node->leaf)
+	if(IS_LEAF(node)) {
+	    struct cli_ac_node *failtarget = node->fail;
+	    while(IS_LEAF(failtarget))
+		failtarget = failtarget->fail;
+	    node->fail = failtarget;
 	    continue;
+	}
 
 	for(i = 0; i < 256; i++) {
 	    child = node->trans[i];
 	    if(child) {
 		fail = node->fail;
-		while(fail->leaf || !fail->trans[i])
+		while(IS_LEAF(fail) || !fail->trans[i])
 		    fail = fail->fail;
 
 		child->fail = fail->trans[i];
@@ -271,9 +276,31 @@ static int ac_maketrans(struct cli_matcher *root)
 		    child->list = child->fail->list;
 		}
 
-		if(child->list)
-		    child->final = 1;
+		if((ret = bfs_enqueue(&bfs, &bfs_last, child)) != 0)
+		    return ret;
+	    }
+	}
+    }
 
+    bfs = bfs_last = NULL;
+    for(i = 0; i < 256; i++) {
+	node = ac_root->trans[i];
+	if(node != ac_root) {
+	    if((ret = bfs_enqueue(&bfs, &bfs_last, node)))
+		return ret;
+	}
+    }
+    while((node = bfs_dequeue(&bfs, &bfs_last))) {
+	if(IS_LEAF(node))
+	    continue;
+	for(i = 0; i < 256; i++) {
+	    child = node->trans[i];
+	    if(!child) {
+		struct cli_ac_node *failtarget = node->fail;
+		while(IS_LEAF(failtarget) || !failtarget->trans[i])
+		    failtarget = failtarget->fail;
+		node->trans[i] = failtarget->trans[i];
+	    } else {
 		if((ret = bfs_enqueue(&bfs, &bfs_last, child)) != 0)
 		    return ret;
 	    }
@@ -298,17 +325,20 @@ int cli_ac_buildtrie(struct cli_matcher *root)
 
 int cli_ac_init(struct cli_matcher *root, uint8_t mindepth, uint8_t maxdepth)
 {
+#ifdef USE_MPOOL
+    assert(root->mempool && "mempool must be initialized");
+#endif
 
-    root->ac_root = (struct cli_ac_node *) cli_calloc(1, sizeof(struct cli_ac_node));
+    root->ac_root = (struct cli_ac_node *) mpool_calloc(root->mempool, 1, sizeof(struct cli_ac_node));
     if(!root->ac_root) {
 	cli_errmsg("cli_ac_init: Can't allocate memory for ac_root\n");
 	return CL_EMEM;
     }
 
-    root->ac_root->trans = (struct cli_ac_node **) cli_calloc(256, sizeof(struct cli_ac_node *));
+    root->ac_root->trans = (struct cli_ac_node **) mpool_calloc(root->mempool, 256, sizeof(struct cli_ac_node *));
     if(!root->ac_root->trans) {
 	cli_errmsg("cli_ac_init: Can't allocate memory for ac_root->trans\n");
-	free(root->ac_root);
+	mpool_free(root->mempool, root->ac_root);
 	return CL_EMEM;
     }
 
@@ -318,7 +348,13 @@ int cli_ac_init(struct cli_matcher *root, uint8_t mindepth, uint8_t maxdepth)
     return CL_SUCCESS;
 }
 
+#ifdef USE_MPOOL
+#define mpool_ac_free_alt(a, b) ac_free_alt(a, b)
+static void ac_free_alt(mpool_t *mempool, struct cli_ac_patt *p)
+#else
+#define mpool_ac_free_alt(a, b) ac_free_alt(b)
 static void ac_free_alt(struct cli_ac_patt *p)
+#endif
 {
 	uint16_t i;
 	struct cli_ac_alt *a1, *a2;
@@ -333,11 +369,11 @@ static void ac_free_alt(struct cli_ac_patt *p)
 	    a2 = a1;
 	    a1 = a1->next;
 	    if(a2->str)
-		free(a2->str);
-	    free(a2);
+		mpool_free(mempool, a2->str);
+	    mpool_free(mempool, a2);
 	}
     }
-    free(p->alttable);
+    mpool_free(mempool, p->alttable);
 }
 
 void cli_ac_free(struct cli_matcher *root)
@@ -348,29 +384,28 @@ void cli_ac_free(struct cli_matcher *root)
 
     for(i = 0; i < root->ac_patterns; i++) {
 	patt = root->ac_pattable[i];
-	patt->prefix ? free(patt->prefix) : free(patt->pattern);
-	free(patt->virname);
+	mpool_free(root->mempool, patt->prefix ? patt->prefix : patt->pattern);
+	mpool_free(root->mempool, patt->virname);
 	if(patt->offset)
-	    free(patt->offset);
+	    mpool_free(root->mempool, patt->offset);
 	if(patt->alt)
-	    ac_free_alt(patt);
-	free(patt);
+	    mpool_ac_free_alt(root->mempool, patt);
+	mpool_free(root->mempool, patt);
     }
     if(root->ac_pattable)
-	free(root->ac_pattable);
+	mpool_free(root->mempool, root->ac_pattable);
 
     for(i = 0; i < root->ac_nodes; i++) {
-	if(!root->ac_nodetable[i]->leaf)
-	    free(root->ac_nodetable[i]->trans);
-	free(root->ac_nodetable[i]);
+	if(!IS_LEAF(root->ac_nodetable[i]))
+	    mpool_free(root->mempool, root->ac_nodetable[i]->trans);
+	mpool_free(root->mempool, root->ac_nodetable[i]);
     }
 
     if(root->ac_nodetable)
-	free(root->ac_nodetable);
-
+	mpool_free(root->mempool, root->ac_nodetable);
     if(root->ac_root) {
-	free(root->ac_root->trans);
-	free(root->ac_root);
+	mpool_free(root->mempool, root->ac_root->trans);
+	mpool_free(root->mempool, root->ac_root);
     }
 }
 
@@ -464,7 +499,7 @@ int cli_ac_chklsig(const char *expr, const char *end, uint32_t *lsigcnt, unsigne
 	    val = lsigcnt[id];
 
 	if(mod) {
-	    pt = strchr(expr, mod) + modoff;
+	    pt = expr + modoff + 1;
 	    ret = sscanf(pt, "%u", &modval1);
 	    if(!ret || ret == EOF) {
 		cli_errmsg("chklexpr: Syntax error: Missing number after '%c'\n", mod);
@@ -806,7 +841,7 @@ inline static int ac_addtype(struct cli_matched_type **list, cli_file_t type, of
 
 
     if(type == CL_TYPE_ZIPSFX) {
-	if(*list && ctx && ctx->limits && ctx->limits->maxfiles && (*list)->cnt > ctx->limits->maxfiles)
+	if(*list && ctx && ctx->engine->maxfiles && (*list)->cnt > ctx->engine->maxfiles)
 	    return CL_SUCCESS;
     } else if(*list && (*list)->cnt >= MAX_EMBEDDED_OBJ)
 	return CL_SUCCESS;
@@ -858,12 +893,12 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
     for(i = 0; i < length; i++)  {
 
-	while(current->leaf || !current->trans[buffer[i]])
+	if(IS_LEAF(current))
 	    current = current->fail;
 
 	current = current->trans[buffer[i]];
 
-	if(current->final) {
+	if(IS_FINAL(current)) {
 	    patt = current->list;
 	    while(patt) {
 		bp = i + 1 - patt->depth;
@@ -879,7 +914,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 			realoff = offset + bp - pt->prefix_length;
 
 			if(pt->offset && (!pt->sigid || pt->partno == 1)) {
-			    if((fd == -1 && !ftype) || !cli_validatesig(ftype, pt->offset, realoff, &info, fd, pt->virname)) {
+			    if(!cli_validatesig(ftype, pt->offset, realoff, &info, fd, pt->virname)) {
 				pt = pt->next_same;
 				continue;
 			    }
@@ -901,7 +936,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 				    return CL_EMEM;
 				}
 
-				mdata->offmatrix[pt->sigid - 1][0] = cli_malloc(pt->parts * (AC_DEFAULT_TRACKLEN + 1) * sizeof(int32_t));
+				mdata->offmatrix[pt->sigid - 1][0] = cli_malloc(pt->parts * (CLI_DEFAULT_AC_TRACKLEN + 1) * sizeof(int32_t));
 				if(!mdata->offmatrix[pt->sigid - 1][0]) {
 				    cli_errmsg("cli_ac_scanbuff: Can't allocate memory for mdata->offmatrix[%u][0]\n", pt->sigid - 1);
 				    free(mdata->offmatrix[pt->sigid - 1]);
@@ -910,10 +945,10 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 					free(info.exeinfo.section);
 				    return CL_EMEM;
 				}
-				memset(mdata->offmatrix[pt->sigid - 1][0], -1, pt->parts * (AC_DEFAULT_TRACKLEN + 1) * sizeof(int32_t));
+				memset(mdata->offmatrix[pt->sigid - 1][0], -1, pt->parts * (CLI_DEFAULT_AC_TRACKLEN + 1) * sizeof(int32_t));
 				mdata->offmatrix[pt->sigid - 1][0][0] = 0;
 				for(j = 1; j < pt->parts; j++) {
-				    mdata->offmatrix[pt->sigid - 1][j] = mdata->offmatrix[pt->sigid - 1][0] + j * (AC_DEFAULT_TRACKLEN + 1);
+				    mdata->offmatrix[pt->sigid - 1][j] = mdata->offmatrix[pt->sigid - 1][0] + j * (CLI_DEFAULT_AC_TRACKLEN + 1);
 				    mdata->offmatrix[pt->sigid - 1][j][0] = 0;
 				}
 			    }
@@ -921,7 +956,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
 			    if(pt->partno != 1) {
 				found = 0;
-				for(j = 1; j <= AC_DEFAULT_TRACKLEN && offmatrix[pt->partno - 2][j] != -1; j++) {
+				for(j = 1; j <= CLI_DEFAULT_AC_TRACKLEN && offmatrix[pt->partno - 2][j] != -1; j++) {
 				    found = 1;
 				    if(pt->maxdist)
 					if(realoff - offmatrix[pt->partno - 2][j] > pt->maxdist)
@@ -937,7 +972,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 			    }
 
 			    if(pt->partno == 1 || (found && (pt->partno != pt->parts))) {
-				offmatrix[pt->partno - 1][0] %= AC_DEFAULT_TRACKLEN;
+				offmatrix[pt->partno - 1][0] %= CLI_DEFAULT_AC_TRACKLEN;
 				offmatrix[pt->partno - 1][0]++;
 				offmatrix[pt->partno - 1][offmatrix[pt->partno - 1][0]] = offset + matchend;
 
@@ -958,7 +993,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 					type = pt->type;
 					if(ftoffset && (!*ftoffset || (*ftoffset)->cnt < MAX_EMBEDDED_OBJ || type == CL_TYPE_ZIPSFX) && ((ftype == CL_TYPE_MSEXE && type >= CL_TYPE_SFX) || ((ftype == CL_TYPE_MSEXE || ftype == CL_TYPE_ZIP || ftype == CL_TYPE_MSOLE2) && type == CL_TYPE_MSEXE)))  {
 					    /* FIXME: we don't know which offset of the first part is the correct one */
-					    for(j = 1; j <= AC_DEFAULT_TRACKLEN && offmatrix[0][j] != -1; j++) {
+					    for(j = 1; j <= CLI_DEFAULT_AC_TRACKLEN && offmatrix[0][j] != -1; j++) {
 						if(ac_addtype(ftoffset, type, offmatrix[pt->parts - 1][j], ctx)) {
 						    if(info.exeinfo.section)
 							free(info.exeinfo.section);
@@ -967,7 +1002,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 					    }
 					}
 
-					memset(offmatrix[0], -1, pt->parts * (AC_DEFAULT_TRACKLEN + 1) * sizeof(int32_t));
+					memset(offmatrix[0], -1, pt->parts * (CLI_DEFAULT_AC_TRACKLEN + 1) * sizeof(int32_t));
 					for(j = 0; j < pt->parts; j++)
 					    offmatrix[j][0] = 0;
 				    }
@@ -1091,10 +1126,12 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	return CL_ENULLARG;
     }
 
-    if(strlen(hexsig) / 2 < root->ac_mindepth)
-	return CL_EPATSHORT;
+    if(strlen(hexsig) / 2 < root->ac_mindepth) {
+	cli_errmsg("cli_ac_addsig: Signature for %s is too short\n", virname);
+	return CL_EMALFDB;
+    }
 
-    if((new = (struct cli_ac_patt *) cli_calloc(1, sizeof(struct cli_ac_patt))) == NULL)
+    if((new = (struct cli_ac_patt *) mpool_calloc(root->mempool, 1, sizeof(struct cli_ac_patt))) == NULL)
 	return CL_EMEM;
 
     new->rtype = rtype;
@@ -1114,7 +1151,7 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 
     if(strchr(hexsig, '[')) {
 	if(!(hexcpy = cli_strdup(hexsig))) {
-	    free(new);
+	    mpool_free(root->mempool, new);
 	    return CL_EMEM;
 	}
 
@@ -1179,14 +1216,14 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 
 	if(error) {
 	    free(hexcpy);
-	    free(new);
+	    mpool_free(root->mempool, new);
 	    return error;
 	}
 
 	hex = cli_strdup(hex);
 	free(hexcpy);
 	if(!hex) {
-	    free(new);
+	    mpool_free(root->mempool, new);
 	    return CL_EMEM;
 	}
     }
@@ -1197,13 +1234,13 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	if(hex) {
 	    hexcpy = hex;
 	} else if(!(hexcpy = cli_strdup(hexsig))) {
-	    free(new);
+	    mpool_free(root->mempool, new);
 	    return CL_EMEM;
 	}
 
 	if(!(hexnew = (char *) cli_calloc(strlen(hexsig) + 1, 1))) {
-	    free(hexcpy);
 	    free(new);
+	    free(hexcpy);
 	    return CL_EMEM;
 	}
 
@@ -1225,7 +1262,7 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	    }
 	    *start++ = 0;
 
-	    newalt = (struct cli_ac_alt *) cli_calloc(1, sizeof(struct cli_ac_alt));
+	    newalt = (struct cli_ac_alt *) mpool_calloc(root->mempool, 1, sizeof(struct cli_ac_alt));
 	    if(!newalt) {
 		cli_errmsg("cli_ac_addsig: Can't allocate newalt\n");
 		error = CL_EMEM;
@@ -1233,10 +1270,11 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	    }
 
 	    new->alt++;
-	    newtable = (struct cli_ac_alt **) cli_realloc(new->alttable, new->alt * sizeof(struct cli_ac_alt *));
+
+	    newtable = (struct cli_ac_alt **) mpool_realloc(root->mempool, new->alttable, new->alt * sizeof(struct cli_ac_alt *));
 	    if(!newtable) {
 		new->alt--;
-		free(newalt);
+		mpool_free(root->mempool, newalt);
 		cli_errmsg("cli_ac_addsig: Can't realloc new->alttable\n");
 		error = CL_EMEM;
 		break;
@@ -1256,7 +1294,7 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 
 	    if(3 * newalt->num - 1 == (uint16_t) strlen(pt)) {
 		newalt->chmode = 1;
-		newalt->str = (unsigned char *) cli_malloc(newalt->num);
+		newalt->str = (unsigned char *) mpool_malloc(root->mempool, newalt->num);
 		if(!newalt->str) {
 		    cli_errmsg("cli_ac_addsig: Can't allocate newalt->str\n");
 		    error = CL_EMEM;
@@ -1270,7 +1308,7 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 		    break;
 		}
 
-		if(!(c = cli_hex2str(h))) {
+		if(!(c = cli_mpool_hex2str(root->mempool, h))) {
 		    free(h);
 		    error = CL_EMALFDB;
 		    break;
@@ -1278,14 +1316,14 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 
 		if(newalt->chmode) {
 		    newalt->str[i] = *c;
-		    free(c);
+		    mpool_free(root->mempool, c);
 		} else {
 		    if(i) {
 			altpt = newalt;
 			while(altpt->next)
 			    altpt = altpt->next;
 
-			altpt->next = (struct cli_ac_alt *) cli_calloc(1, sizeof(struct cli_ac_alt));
+			altpt->next = (struct cli_ac_alt *) mpool_calloc(root->mempool, 1, sizeof(struct cli_ac_alt));
 			if(!altpt->next) {
 			    cli_errmsg("cli_ac_addsig: Can't allocate altpt->next\n");
 			    error = CL_EMEM;
@@ -1318,20 +1356,22 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	if(error) {
 	    if(new->alt) {
 		free(hex);
-		ac_free_alt(new);
+		mpool_ac_free_alt(root->mempool, new);
 	    }
-	    free(new);
+	    mpool_free(root->mempool, new);
 	    return error;
 	}
     }
 
-    if((new->pattern = cli_hex2ui(hex ? hex : hexsig)) == NULL) {
+    new->pattern = cli_mpool_hex2ui(root->mempool, hex ? hex : hexsig);
+    if(new->pattern == NULL) {
 	if(new->alt)
-	    ac_free_alt(new);
+	    mpool_ac_free_alt(root->mempool, new);
+	mpool_free(root->mempool, new);
 	free(hex);
-	free(new);
 	return CL_EMALFDB;
     }
+
     new->length = strlen(hex ? hex : hexsig) / 2;
     free(hex);
 
@@ -1375,9 +1415,9 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 
 	if(plen < root->ac_mindepth) {
 	    cli_errmsg("cli_ac_addsig: Can't find a static subpattern of length %u\n", root->ac_mindepth);
-	    ac_free_alt(new);
-	    free(new->pattern);
-	    free(new);
+	    mpool_ac_free_alt(root->mempool, new);
+	    mpool_free(root->mempool, new->pattern);
+	    mpool_free(root->mempool, new);
 	    return CL_EMALFDB;
 	}
 
@@ -1394,11 +1434,11 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
     if(new->length > root->maxpatlen)
 	root->maxpatlen = new->length;
 
-    new->virname = cli_virname((char *) virname, options & CL_DB_OFFICIAL, 0);
+    new->virname = cli_mpool_virname(root->mempool, (char *) virname, options & CL_DB_OFFICIAL);
     if(!new->virname) {
-	new->prefix ? free(new->prefix) : free(new->pattern);
-	ac_free_alt(new);
-	free(new);
+	mpool_free(root->mempool, new->prefix ? new->prefix : new->pattern);
+	mpool_ac_free_alt(root->mempool, new);
+	mpool_free(root->mempool, new);
 	return CL_EMEM;
     }
 
@@ -1406,31 +1446,25 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	root->ac_lsigtable[new->lsigid[1]]->virname = new->virname;
 
     if(offset) {
-	new->offset = cli_strdup(offset);
+	new->offset = cli_mpool_strdup(root->mempool, offset);
 	if(!new->offset) {
-	    new->prefix ? free(new->prefix) : free(new->pattern);
-	    ac_free_alt(new);
-	    free(new->virname);
-	    free(new);
+	    mpool_free(root->mempool, new->prefix ? new->prefix : new->pattern);
+	    mpool_ac_free_alt(root->mempool, new);
+	    mpool_free(root->mempool, new->virname);
+	    mpool_free(root->mempool, new);
 	    return CL_EMEM;
 	}
     }
 
     if((ret = cli_ac_addpatt(root, new))) {
-	new->prefix ? free(new->prefix) : free(new->pattern);
-	free(new->virname);
-	ac_free_alt(new);
+	mpool_free(root->mempool, new->prefix ? new->prefix : new->pattern);
+	mpool_free(root->mempool, new->virname);
+	mpool_ac_free_alt(root->mempool, new);
 	if(new->offset)
-	    free(new->offset);
-	free(new);
+	    mpool_free(root->mempool, new->offset);
+	mpool_free(root->mempool, new);
 	return ret;
     }
 
     return CL_SUCCESS;
 }
-
-void cli_ac_setdepth(uint8_t mindepth, uint8_t maxdepth)
-{
-    cli_ac_mindepth = mindepth;
-    cli_ac_maxdepth = maxdepth;
-}
