@@ -23,7 +23,7 @@
 #endif
 
 #include <stdio.h>
-
+#include <assert.h>
 #include "clamav.h"
 #include "memory.h"
 #include "others.h"
@@ -32,6 +32,8 @@
 #include "matcher-bm.h"
 #include "filetypes.h"
 
+#include "mpool.h"
+
 #define BM_MIN_LENGTH	3
 #define BM_BLOCK_SIZE	3
 #define HASH(a,b,c) (211 * a + 37 * b + c)
@@ -44,8 +46,8 @@ int cli_bm_addpatt(struct cli_matcher *root, struct cli_bm_patt *pattern)
 
 
     if(pattern->length < BM_MIN_LENGTH) {
-	cli_errmsg("Signature for %s is too short\n", pattern->virname);
-	return CL_EPATSHORT;
+	cli_errmsg("cli_bm_addpatt: Signature for %s is too short\n", pattern->virname);
+	return CL_EMALFDB;
     }
 
 #if BM_MIN_LENGTH == BM_BLOCK_SIZE
@@ -96,13 +98,14 @@ int cli_bm_addpatt(struct cli_matcher *root, struct cli_bm_patt *pattern)
 int cli_bm_init(struct cli_matcher *root)
 {
 	uint16_t i, size = HASH(255, 255, 255) + 1;
-
-
-    if(!(root->bm_shift = (uint8_t *) cli_calloc(size, sizeof(uint8_t))))
+#ifdef USE_MPOOL
+    assert (root->mempool && "mempool must be initialized");
+#endif
+    if(!(root->bm_shift = (uint8_t *) mpool_calloc(root->mempool, size, sizeof(uint8_t))))
 	return CL_EMEM;
 
-    if(!(root->bm_suffix = (struct cli_bm_patt **) cli_calloc(size, sizeof(struct cli_bm_patt *)))) {
-	free(root->bm_shift);
+    if(!(root->bm_suffix = (struct cli_bm_patt **) mpool_calloc(root->mempool, size, sizeof(struct cli_bm_patt *)))) {
+	mpool_free(root->mempool, root->bm_shift);
 	return CL_EMEM;
     }
 
@@ -119,7 +122,7 @@ void cli_bm_free(struct cli_matcher *root)
 
 
     if(root->bm_shift)
-	free(root->bm_shift);
+	mpool_free(root->mempool, root->bm_shift);
 
     if(root->bm_suffix) {
 	for(i = 0; i < size; i++) {
@@ -128,17 +131,17 @@ void cli_bm_free(struct cli_matcher *root)
 		prev = patt;
 		patt = patt->next;
 		if(prev->prefix)
-		    free(prev->prefix);
+		    mpool_free(root->mempool, prev->prefix);
 		else
-		    free(prev->pattern);
+		    mpool_free(root->mempool, prev->pattern);
 		if(prev->virname)
-		    free(prev->virname);
+		    mpool_free(root->mempool, prev->virname);
 		if(prev->offset)
-		    free(prev->offset);
-		free(prev);
+		    mpool_free(root->mempool, prev->offset);
+		mpool_free(root->mempool, prev);
 	    }
 	}
-	free(root->bm_suffix);
+	mpool_free(root->mempool, root->bm_suffix);
     }
 }
 
@@ -211,9 +214,9 @@ int cli_bm_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
 		if(found && p->length + p->prefix_length == j) {
 
-		    if(p->target || p->offset) {
+		    if(p->offset) {
 			off = offset + i - p->prefix_length - BM_MIN_LENGTH + BM_BLOCK_SIZE;
-			if((fd == -1 && !ftype) || !cli_validatesig(ftype, p->offset, off, &info, fd, p->virname)) {
+			if(!cli_validatesig(ftype, p->offset, off, &info, fd, p->virname)) {
 			    p = p->next;
 			    continue;
 			}
