@@ -34,6 +34,8 @@
 #include "matcher-bm.h"
 #include "hashtab.h"
 
+#include "mpool.h"
+
 #define CLI_MATCH_WILDCARD	0xff00
 #define CLI_MATCH_CHAR		0x0000
 #define CLI_MATCH_IGNORE	0x0100
@@ -41,6 +43,31 @@
 #define CLI_MATCH_NIBBLE_HIGH	0x0300
 #define CLI_MATCH_NIBBLE_LOW	0x0400
 
+struct cli_lsig_tdb {
+#define CLI_TDB_UINT	0
+#define CLI_TDB_RANGE	1
+#define CLI_TDB_STR	2
+#define CLI_TDB_RANGE2	3
+    uint32_t *val, *range;
+    char *str;
+    uint32_t cnt[3];
+
+    const uint32_t *target;
+    const uint32_t *engine, *nos, *ep;
+    const uint32_t *sectoff, *sectrva, *sectvsz, *sectraw, *sectrsz,
+		   *secturva, *sectuvsz, *secturaw, *sectursz;
+#ifdef USE_MPOOL
+    mpool_t *mempool;
+#endif
+};
+
+struct cli_ac_lsig {
+    uint32_t id;
+    char *logic;
+    const char *virname;
+    struct cli_lsig_tdb tdb;
+};
+
 struct cli_matcher {
     /* Extended Boyer-Moore */
     uint8_t *bm_shift;
@@ -50,13 +77,17 @@ struct cli_matcher {
     uint32_t bm_patterns;
 
     /* Extended Aho-Corasick */
-    uint32_t ac_partsigs, ac_nodes, ac_patterns;
+    uint32_t ac_partsigs, ac_nodes, ac_patterns, ac_lsigs;
+    struct cli_ac_lsig **ac_lsigtable;
     struct cli_ac_node *ac_root, **ac_nodetable;
     struct cli_ac_patt **ac_pattable;
     uint8_t ac_mindepth, ac_maxdepth;
 
     uint16_t maxpatlen;
     uint8_t ac_only;
+#ifdef USE_MPOOL
+    mpool_t *mempool;
+#endif
 };
 
 struct cli_meta_node {
@@ -73,7 +104,7 @@ struct cli_mtarget {
     uint8_t ac_only;
 };
 
-#define CLI_MTARGETS 8
+#define CLI_MTARGETS 9
 static const struct cli_mtarget cli_mtargets[CLI_MTARGETS] =  {
     { 0,		    "GENERIC",	    0,	0   },
     { CL_TYPE_MSEXE,	    "PE",	    1,	0   },
@@ -82,7 +113,8 @@ static const struct cli_mtarget cli_mtargets[CLI_MTARGETS] =  {
     { CL_TYPE_MAIL,	    "MAIL",	    4,	1   },
     { CL_TYPE_GRAPHICS,	    "GRAPHICS",	    5,	1   },
     { CL_TYPE_ELF,	    "ELF",	    6,	1   },
-    { CL_TYPE_TEXT_ASCII,   "ASCII",	    7,	1   }
+    { CL_TYPE_TEXT_ASCII,   "ASCII",	    7,	1   },
+    { CL_TYPE_PE_DISASM,    "DISASM",	    8,	1   }
 };
 
 struct cli_target_info {
@@ -91,7 +123,7 @@ struct cli_target_info {
     int8_t status; /* 0 == not initialised, 1 == initialised OK, -1 == error */
 };
 
-int cli_scanbuff(const unsigned char *buffer, uint32_t length, cli_ctx *ctx, cli_file_t ftype);
+int cli_scanbuff(const unsigned char *buffer, uint32_t length, uint32_t offset, cli_ctx *ctx, cli_file_t ftype, struct cli_ac_data **acdata);
 
 int cli_scandesc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode);
 
