@@ -18,6 +18,8 @@
  *  MA 02110-1301, USA.
  */
 
+#include "matcher.h"
+
 #ifndef __OTHERS_H_LC
 #define __OTHERS_H_LC
 
@@ -31,8 +33,21 @@
 
 #include "clamav.h"
 #include "dconf.h"
+#include "libclamunrar_iface/unrar_iface.h"
 
-extern uint8_t cli_debug_flag, cli_leavetemps_flag;
+/*
+ * CL_FLEVEL is the signature f-level specific to the current code and
+ *	     should never be modified
+ * CL_FLEVEL_DCONF is used in the dconf module and can be bumped by
+ * distribution packagers provided they fix *all* security issues found
+ * in the old versions of ClamAV. Updating CL_FLEVEL_DCONF will result
+ * in re-enabling affected modules.
+ */
+
+#define CL_FLEVEL 42
+#define CL_FLEVEL_DCONF	CL_FLEVEL
+
+extern uint8_t cli_debug_flag;
 
 /*
  * CLI_ISCONTAINED(buf1, size1, buf2, size2) checks if buf2 is contained
@@ -44,12 +59,12 @@ extern uint8_t cli_debug_flag, cli_leavetemps_flag;
  * The macro can be used to protect against wraps.
  */
 #define CLI_ISCONTAINED(bb, bb_size, sb, sb_size)	\
-    (bb_size > 0 && sb_size > 0 && sb_size <= bb_size	\
-     && sb >= bb && sb + sb_size <= bb + bb_size && sb + sb_size > bb)
+  ((bb_size) > 0 && (sb_size) > 0 && (size_t)(sb_size) <= (size_t)(bb_size) \
+   && (sb) >= (bb) && ((sb) + (sb_size)) <= ((bb) + (bb_size)) && ((sb) + (sb_size)) > (bb) && (sb) < ((bb) + (bb_size)))
 
 #define CLI_ISCONTAINED2(bb, bb_size, sb, sb_size)	\
-    (bb_size > 0 && sb_size >= 0 && sb_size <= bb_size	\
-     && sb >= bb && sb + sb_size <= bb + bb_size && sb + sb_size >= bb)
+  ((bb_size) > 0 && (sb_size) >= 0 && (size_t)(sb_size) <= (size_t)(bb_size) \
+   && (sb) >= (bb) && ((sb) + (sb_size)) <= ((bb) + (bb_size)) && ((sb) + (sb_size)) >= (bb) && (sb) < ((bb) + (bb_size)))
 
 #define CLI_MAX_ALLOCATION 184549376
 
@@ -81,7 +96,6 @@ typedef struct {
     unsigned long int *scanned;
     const struct cli_matcher *root;
     const struct cl_engine *engine;
-    const struct cl_limits *limits;
     unsigned long scansize;
     unsigned int options;
     unsigned int recursion;
@@ -90,6 +104,99 @@ typedef struct {
     struct cli_dconf *dconf;
 } cli_ctx;
 
+struct cl_engine {
+    uint32_t refcount; /* reference counter */
+    uint32_t sdb;
+    uint32_t dboptions;
+    uint32_t dbversion[2];
+    uint32_t ac_only;
+    uint32_t ac_mindepth;
+    uint32_t ac_maxdepth;
+    char *tmpdir;
+    uint32_t keeptmp;
+
+    /* Limits */
+    uint64_t maxscansize;  /* during the scanning of archives this size
+				     * will never be exceeded
+				     */
+    uint64_t maxfilesize;  /* compressed files will only be decompressed
+				     * and scanned up to this size
+				     */
+    uint32_t maxreclevel;	    /* maximum recursion level for archives */
+    uint32_t maxfiles;	    /* maximum number of files to be scanned
+				     * within a single archive
+				     */
+    /* This is for structured data detection.  You can set the minimum
+     * number of occurences of an CC# or SSN before the system will
+     * generate a notification.
+     */
+    uint32_t min_cc_count;
+    uint32_t min_ssn_count;
+
+    /* Roots table */
+    struct cli_matcher **root;
+
+    /* B-M matcher for standard MD5 sigs */
+    struct cli_matcher *md5_hdb;
+
+    /* B-M matcher for MD5 sigs for PE sections */
+    struct cli_matcher *md5_mdb;
+
+    /* B-M matcher for whitelist db */
+    struct cli_matcher *md5_fp;
+
+    /* Zip metadata */
+    struct cli_meta_node *zip_mlist;
+
+    /* RAR metadata */
+    struct cli_meta_node *rar_mlist;
+
+    /* Phishing .pdb and .wdb databases*/
+    struct regex_matcher *whitelist_matcher;
+    struct regex_matcher *domainlist_matcher;
+    struct phishcheck *phishcheck;
+
+    /* Dynamic configuration */
+    struct cli_dconf *dconf;
+
+    /* Filetype definitions */
+    struct cli_ftype *ftypes;
+
+    /* Ignored signatures */
+    struct cli_ignored *ignored;
+
+    /* PUA categories (to be included or excluded) */
+    char *pua_cats;
+
+    /* Used for memory pools */
+    mpool_t *mempool;
+};
+
+struct cl_settings {
+    /* don't store dboptions here; it needs to be provided to cl_load() and
+     * can be optionally obtained with cl_engine_get() or from the original
+     * settings stored by the application
+     */
+    uint32_t ac_only;
+    uint32_t ac_mindepth;
+    uint32_t ac_maxdepth;
+    char *tmpdir;
+    uint32_t keeptmp;
+    uint64_t maxscansize;
+    uint64_t maxfilesize;
+    uint32_t maxreclevel;
+    uint32_t maxfiles;
+    uint32_t min_cc_count;
+    uint32_t min_ssn_count;
+    char *pua_cats;
+};
+
+extern int (*cli_unrar_open)(int fd, const char *dirname, unrar_state_t *state);
+extern int (*cli_unrar_extract_next_prepare)(unrar_state_t *state, const char *dirname);
+extern int (*cli_unrar_extract_next)(unrar_state_t *state, const char *dirname);
+extern void (*cli_unrar_close)(unrar_state_t *state);
+extern int have_rar;
+
 #define SCAN_ARCHIVE	    (ctx->options & CL_SCAN_ARCHIVE)
 #define SCAN_MAIL	    (ctx->options & CL_SCAN_MAIL)
 #define SCAN_OLE2	    (ctx->options & CL_SCAN_OLE2)
@@ -276,4 +383,59 @@ int cli_checklimits(const char *, cli_ctx *, unsigned long, unsigned long, unsig
 int cli_updatelimits(cli_ctx *, unsigned long);
 unsigned long cli_getsizelimit(cli_ctx *, unsigned long);
 int cli_matchregex(const char *str, const char *regex);
+
+/* symlink behaviour */
+#define CLI_FTW_FOLLOW_FILE_SYMLINK 0x01
+#define CLI_FTW_FOLLOW_DIR_SYMLINK  0x02
+
+/* if the callback needs the stat */
+#define CLI_FTW_NEED_STAT	    0x04
+
+/* remove leading/trailing slashes */
+#define CLI_FTW_TRIM_SLASHES	    0x08
+#define CLI_FTW_STD (CLI_FTW_NEED_STAT | CLI_FTW_TRIM_SLASHES)
+
+enum cli_ftw_reason {
+    visit_file,
+    visit_directory_toplev, /* this is a directory at toplevel of recursion */
+    error_mem, /* recommended to return CL_EMEM */
+    /* recommended to return CL_SUCCESS below */
+    error_stat,
+    warning_skipped_link,
+    warning_skipped_special,
+    warning_skipped_dir
+};
+
+/* wrap void*, so that we don't mix it with some other pointer */
+struct cli_ftw_cbdata {
+    void *data;
+};
+
+/* 
+ * return CL_BREAK to break out without an error, CL_SUCCESS to continue,
+ * or any CL_E* to break out due to error.
+ * The callback is responsible for freeing filename when it is done using it.
+ * Note that callback decides if directory traversal should continue 
+ * after an error, we call the callback with reason == error,
+ * and if it returns CL_BREAK we break.
+ */
+typedef int (*cli_ftw_cb)(struct stat *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
+
+/*
+ * returns 
+ *  CL_SUCCESS if it traversed all files and subdirs
+ *  CL_BREAK if traversal has stopped at some point
+ *  CL_E* if error encountered during traversal and we had to break out
+ * This is regardless of virus found/not, that is the callback's job to store.
+ * Note that the callback may dispatch async the scan, so that when cli_ftw
+ * returns we don't know the infected/notinfected status of the directory yet!
+ * Due to this if the callback scans synchronously it should store the infected
+ * status in its cbdata.
+ * This works for both files and directories. It stats the path to determine
+ * which one it is.
+ * If it is a file, it simply calls the callback once, otherwise recurses.
+ */
+int cli_ftw(char *base, int flags, int maxdepth, cli_ftw_cb callback, struct cli_ftw_cbdata *data);
+
+const char *cli_strerror(int errnum, char* buf, size_t len);
 #endif
