@@ -46,9 +46,7 @@ static	char	const	rcsid[] = "$Id: pdf.c,v 1.61 2007/02/12 20:46:09 njh Exp $";
 #include <sys/mman.h>
 #endif
 
-#ifdef HAVE_ZLIB_H
 #include <zlib.h>
-#endif
 
 #ifdef	C_WINDOWS
 #include <io.h>
@@ -108,7 +106,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 	p = buf = mmap(NULL, size, PROT_READ, MAP_PRIVATE, desc, offset);
 	if(buf == MAP_FAILED) {
 		cli_errmsg("cli_pdf: mmap() failed\n");
-		return CL_EMEM;
+		return CL_EMAP;
 	}
 
 	cli_dbgmsg("cli_pdf: scanning %lu bytes\n", (unsigned long)size);
@@ -399,7 +397,8 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 		snprintf(fullname, sizeof(fullname), "%s/pdf%02u", dir, files);
 		fout = open(fullname, O_RDWR|O_CREAT|O_EXCL|O_TRUNC|O_BINARY, 0600);
 		if(fout < 0) {
-			cli_errmsg("cli_pdf: can't create temporary file %s: %s\n", fullname, strerror(errno));
+			char err[128];
+			cli_errmsg("cli_pdf: can't create temporary file %s: %s\n", fullname, cli_strerror(errno, err, sizeof(err)));
 			rc = CL_ETMPFILE;
 			break;
 		}
@@ -417,7 +416,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 			close(fout);
 			cli_dbgmsg("cli_pdf: Empty stream\n");
 			if (cli_unlink(fullname)) {
-				rc = CL_EIO;
+				rc = CL_EUNLINK;
 				break;
 			}
 			continue;
@@ -448,7 +447,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 			if(ret != CL_CLEAN) {
 				close(fout);
 				if (cli_unlink(fullname)) {
-					rc = CL_EIO;
+					rc = CL_EUNLINK;
 					break;
 				}
 				continue;
@@ -459,7 +458,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 			if(tmpbuf == NULL) {
 				close(fout);
 				if (cli_unlink(fullname)) {
-					rc = CL_EIO;
+					rc = CL_EUNLINK;
 					break;
 				}
 				continue;
@@ -471,7 +470,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 				free(tmpbuf);
 				close(fout);
 				if (cli_unlink(fullname)) {
-					rc = CL_EIO;
+					rc = CL_EUNLINK;
 					break;
 				}
 				continue;
@@ -486,7 +485,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 					free(tmpbuf);
 					close(fout);
 					if (cli_unlink(fullname)) {
-						rc = CL_EIO;
+						rc = CL_EUNLINK;
 						break;
 					}
 					continue;
@@ -500,7 +499,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 				if(is_flatedecode)
 					rc = try_flatedecode((unsigned char *)tmpbuf, real_streamlen, real_streamlen, fout, ctx);
 				else
-				  rc = (unsigned long)cli_writen(fout, (const char *)streamstart, real_streamlen)==real_streamlen ? CL_CLEAN : CL_EIO;
+				  rc = (unsigned long)cli_writen(fout, (const char *)streamstart, real_streamlen)==real_streamlen ? CL_CLEAN : CL_EWRITE;
 			}
 			free(tmpbuf);
 		} else if(is_flatedecode) {
@@ -510,7 +509,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 			cli_dbgmsg("cli_pdf: writing %lu bytes from the stream\n",
 				(unsigned long)real_streamlen);
 			if((rc = cli_checklimits("cli_pdf", ctx, real_streamlen, 0, 0))==CL_CLEAN)
-				rc = (unsigned long)cli_writen(fout, (const char *)streamstart, real_streamlen) == real_streamlen ? CL_CLEAN : CL_EIO;
+				rc = (unsigned long)cli_writen(fout, (const char *)streamstart, real_streamlen) == real_streamlen ? CL_CLEAN : CL_EWRITE;
 		}
 
 		if (rc == CL_CLEAN) {
@@ -532,7 +531,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 					ctx->scannedfiles++;
 					close(fout);
 					if (cli_unlink(fullname)) {
-						rc = CL_EIO;
+						rc = CL_EUNLINK;
 						break;
 					}
 					continue;
@@ -544,8 +543,8 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 			rc = cli_magic_scandesc(fout, ctx);
 		}
 		close(fout);
-		if(!cli_leavetemps_flag)
-			if (cli_unlink(fullname)) rc = CL_EIO;
+		if(!ctx->engine->keeptmp)
+			if (cli_unlink(fullname)) rc = CL_EUNLINK;
 		if(rc != CL_CLEAN) break;
 	}
 
@@ -558,7 +557,7 @@ cli_pdf(const char *dir, int desc, cli_ctx *ctx, off_t offset)
 }
 
 /*
- * flate inflation - returns clamAV status, e.g CL_SUCCESS, CL_EZIP
+ * flate inflation
  */
 static int
 try_flatedecode(unsigned char *buf, off_t real_len, off_t calculated_len, int fout, cli_ctx *ctx)
@@ -658,7 +657,7 @@ flatedecode(unsigned char *buf, off_t len, int fout, cli_ctx *ctx)
 					if ((written=cli_writen(fout, output, sizeof(output)))!=sizeof(output)) {
 						cli_errmsg("cli_pdf: failed to write output file\n");
 						inflateEnd(&stream);
-						return CL_EIO;
+						return CL_EWRITE;
 					}
 					nbytes += written;
 
@@ -690,14 +689,14 @@ flatedecode(unsigned char *buf, off_t len, int fout, cli_ctx *ctx)
 		if(cli_writen(fout, output, sizeof(output) - stream.avail_out) < 0) {
 			cli_errmsg("cli_pdf: failed to write output file\n");
 			inflateEnd(&stream);
-			return CL_EIO;
+			return CL_EWRITE;
 		}
 	}
 			
 #ifdef	SAVE_TMP
 	if (cli_unlink(tmpfilename)) {
 		inflateEnd(&stream);
-		return CL_EIO;
+		return CL_EUNLINK;
 	}
 #endif
 	inflateEnd(&stream);
