@@ -21,7 +21,7 @@
 #if HAVE_CONFIG_H
 #include "clamav-config.h"
 #endif
-
+#define _XOPEN_SOURCE 500
 #include <stdio.h>
 #if HAVE_STRING_H
 #include <string.h>
@@ -56,6 +56,7 @@
 #include "matcher.h"
 #include "matcher-bm.h"
 #include "disasm.h"
+#include "special.h"
 
 #ifndef	O_BINARY
 #define	O_BINARY	0
@@ -91,7 +92,7 @@ if(cli_checklimits(NAME, ctx, (CHK), 0, 0)!=CL_CLEAN) {	\
 }
 
 #define CLI_UNPTEMP(NAME,FREEME) \
-if(!(tempfile = cli_gentemp(NULL))) { \
+if(!(tempfile = cli_gentemp(ctx->engine->tmpdir))) { \
     cli_multifree FREEME; \
     return CL_EMEM; \
 } \
@@ -99,13 +100,13 @@ if((ndesc = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) { \
     cli_dbgmsg(NAME": Can't create file %s\n", tempfile); \
     free(tempfile); \
     cli_multifree FREEME; \
-    return CL_EIO; \
+    return CL_ECREAT; \
 }
 
-#define CLI_TMPUNLK() if(!cli_leavetemps_flag) { \
+#define CLI_TMPUNLK() if(!ctx->engine->keeptmp) { \
     if (cli_unlink(tempfile)) { \
 	free(tempfile); \
-	return CL_EIO; \
+	return CL_EUNLINK; \
     } \
 }
 
@@ -117,7 +118,7 @@ if((ndesc = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) { \
 	    free(exe_sections); \
 	    free(tempfile); \
 	    FREESEC; \
-	    return CL_EIO; \
+	    return CL_EUNLINK; \
 	} \
 	free(tempfile); \
 	FREESEC; \
@@ -132,7 +133,7 @@ if((ndesc = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) { \
 	if (cli_unlink(tempfile)) { \
 	    free(exe_sections); \
 	    free(tempfile); \
-	    return CL_EIO; \
+	    return CL_EUNLINK; \
 	} \
 	cli_dbgmsg("PESpin: Size exceeded\n"); \
 	free(tempfile); \
@@ -141,7 +142,7 @@ if((ndesc = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) { \
 #define CLI_UNPRESULTS_(NAME,FSGSTUFF,EXPR,GOOD,FREEME) \
     switch(EXPR) { \
     case GOOD: /* Unpacked and rebuilt */ \
-	if(cli_leavetemps_flag) \
+	if(ctx->engine->keeptmp) \
 	    cli_dbgmsg(NAME": Unpacked and rebuilt executable saved in %s\n", tempfile); \
 	else \
 	    cli_dbgmsg(NAME": Unpacked and rebuilt executable\n"); \
@@ -169,16 +170,16 @@ FSGSTUFF; \
 	    free(exe_sections); \
 	    free(tempfile); \
 	    cli_multifree FREEME; \
-	    return CL_EIO; \
+	    return CL_EUNLINK; \
 	} \
 	cli_multifree FREEME; \
         free(tempfile); \
     }
 
 
-#define CLI_UNPRESULTS(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,NULL,EXPR,GOOD,FREEME)
+#define CLI_UNPRESULTS(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,(void)0,EXPR,GOOD,FREEME)
 #define CLI_UNPRESULTSFSG1(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,FSGCASE(NAME,free(sections)),EXPR,GOOD,FREEME)
-#define CLI_UNPRESULTSFSG2(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,FSGCASE(NAME,NULL),EXPR,GOOD,FREEME)
+#define CLI_UNPRESULTSFSG2(NAME,EXPR,GOOD,FREEME) CLI_UNPRESULTS_(NAME,FSGCASE(NAME,(void)0),EXPR,GOOD,FREEME)
 
 struct offset_list {
     uint32_t offset;
@@ -316,6 +317,102 @@ static unsigned int cli_md5sect(int fd, struct cli_exe_section *s, unsigned char
     return 1;
 }
 
+static void cli_parseres_special(uint32_t base, uint32_t rva, int srcfd, struct cli_exe_section *exe_sections, uint16_t nsections, size_t fsize, uint32_t hdr_size, unsigned int level, uint32_t type, unsigned int *maxres, struct swizz_stats *stats) {
+    unsigned int err = 0, i;
+    uint8_t resdir[16];
+    uint8_t *entry, *oentry;
+    uint16_t named, unnamed;
+    uint32_t rawaddr = cli_rawaddr(rva, exe_sections, nsections, &err, fsize, hdr_size);
+    uint32_t entries;
+
+    if(level>2 || !*maxres) return;
+    *maxres-=1;
+    if(err || (pread(srcfd,resdir, sizeof(resdir), rawaddr) != sizeof(resdir)))
+	    return;
+    named = (uint16_t)cli_readint16(resdir+12);
+    unnamed = (uint16_t)cli_readint16(resdir+14);
+
+    entries = /*named+*/unnamed;
+    if (!entries)
+	    return;
+    oentry = entry = cli_malloc(entries*8);
+    rawaddr += named*8; /* skip named */
+    /* this is just used in a heuristic detection, so don't give error on failure */
+    if (!entry) {
+	    cli_dbgmsg("cli_parseres_special: failed to allocate memory for resource directory:%lu\n", (unsigned long)entries);
+	    return;
+    }
+    if (pread(srcfd, entry, entries*8, rawaddr+16) != entries*8) {
+	    cli_dbgmsg("cli_parseres_special: failed to read resource directory at:%lu\n", (unsigned long)rawaddr+16);
+	    free(oentry);
+	    return;
+    }
+    /*for (i=0; i<named; i++) {
+	uint32_t id, offs;
+	id = cli_readint32(entry);
+	offs = cli_readint32(entry+4);
+	if(offs>>31)
+	    cli_parseres( base, base + (offs&0x7fffffff), srcfd, exe_sections, nsections, fsize, hdr_size, level+1, type, maxres, stats);
+	entry+=8;
+    }*/
+    for (i=0; i<unnamed; i++, entry += 8) {
+	uint32_t id, offs;
+	id = cli_readint32(entry)&0x7fffffff;
+	if(level==0) {
+		type = 0;
+		switch(id) {
+			case 4: /* menu */
+			case 5: /* dialog */
+			case 6: /* string */
+			case 11:/* msgtable */
+				type = id;
+				break;
+			case 16:
+				type = id;
+				/* 14: version */
+				stats->has_version = 1;
+				break;
+			case 24: /* manifest */
+				stats->has_manifest = 1;
+				break;
+			/* otherwise keep it 0, we don't want it */
+		}
+	}
+	if (!type) {
+		/* if we are not interested in this type, skip */
+		continue;
+	}
+	offs = cli_readint32(entry+4);
+	if(offs>>31)
+		cli_parseres_special(base, base + (offs&0x7fffffff), srcfd, exe_sections, nsections, fsize, hdr_size, level+1, type, maxres, stats);
+	else {
+			offs = cli_readint32(entry+4);
+			rawaddr = cli_rawaddr(base + offs, exe_sections, nsections, &err, fsize, hdr_size);
+			if (!err && pread(srcfd, resdir, sizeof(resdir), rawaddr) == sizeof(resdir)) {
+				uint32_t isz = cli_readint32(resdir+4);
+				char *str;
+				rawaddr = cli_rawaddr(cli_readint32(resdir), exe_sections, nsections, &err, fsize, hdr_size);
+				if (err || !isz || isz >= fsize || rawaddr+isz >= fsize) {
+					cli_dbgmsg("cli_parseres_special: invalid resource table entry: %lu + %lu\n", 
+							(unsigned long)rawaddr, 
+							(unsigned long)isz);
+					continue;
+				}
+				str = cli_malloc(isz);
+				if (!str) {
+					cli_dbgmsg("cli_parseres_special: failed to allocate string mem: %lu\n", (unsigned long)isz);
+					continue;
+				}
+				if(pread(srcfd, str, isz, rawaddr) == isz) {
+					cli_detect_swizz_str(str, isz, stats, type);
+				}
+				free (str);
+			}
+	}
+    }
+    free (oentry);
+}
+
 int cli_scanpe(int desc, cli_ctx *ctx)
 {
 	uint16_t e_magic; /* DOS signature ("MZ") */
@@ -344,6 +441,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	struct cli_exe_section *exe_sections;
 	struct cli_matcher *md5_sect;
 	char timestr[32];
+	struct pe_image_data_dir *dirs;
 
 
     if(!ctx) {
@@ -597,6 +695,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	cli_dbgmsg("SizeOfImage: 0x%x\n", EC32(optional_hdr32.SizeOfImage));
 	cli_dbgmsg("SizeOfHeaders: 0x%x\n", hdr_size);
 	cli_dbgmsg("NumberOfRvaAndSizes: %d\n", EC32(optional_hdr32.NumberOfRvaAndSizes));
+	dirs = optional_hdr32.DataDirectory;
 
     } else { /* PE+ */
         /* read the remaining part of the header */
@@ -628,6 +727,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	cli_dbgmsg("SizeOfImage: 0x%x\n", EC32(optional_hdr64.SizeOfImage));
 	cli_dbgmsg("SizeOfHeaders: 0x%x\n", hdr_size);
 	cli_dbgmsg("NumberOfRvaAndSizes: %d\n", EC32(optional_hdr64.NumberOfRvaAndSizes));
+	dirs = optional_hdr64.DataDirectory;
     }
 
 
@@ -697,7 +797,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 
     if(fstat(desc, &sb) == -1) {
 	cli_dbgmsg("fstat failed\n");
-	return CL_EIO;
+	return CL_ESTAT;
     }
 
     fsize = sb.st_size;
@@ -791,7 +891,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 
 	cli_dbgmsg("------------------------------------\n");
 
-	if (DETECT_BROKEN && (exe_sections[i].urva % valign)) { /* Bad virtual alignment */
+	if (DETECT_BROKEN && (!valign || (exe_sections[i].urva % valign))) { /* Bad virtual alignment */
 	    cli_dbgmsg("VirtualAddress is misaligned\n");
 	    if(ctx->virname)
 	        *ctx->virname = "Broken.Executable";
@@ -905,7 +1005,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
     epsize = cli_readn(desc, epbuff, 4096);
 
     CLI_UNPTEMP("DISASM",(exe_sections,0));
-    disasmbuf(epbuff, epsize, ndesc);
+    disasmbuf((unsigned char*)epbuff, epsize, ndesc);
     lseek(ndesc, 0, SEEK_SET);
     ret = cli_scandesc(ndesc, ctx, CL_TYPE_PE_DISASM, 1, NULL, AC_SCAN_VIR);
     close(ndesc);
@@ -1068,7 +1168,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	}
 	if(cli_readn(desc, code, exe_sections[0].rsz)!=exe_sections[0].rsz) {
 	    free(exe_sections);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 	for(i=0; i<exe_sections[0].rsz - 5; i++) {
 	    if((uint8_t)(code[i]-0xe8) > 1) continue;
@@ -1113,6 +1213,28 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	break;
     }
 
+    /* Trojan.Swizzor.Gen */
+    if (SCAN_ALGO && (DCONF & PE_CONF_SWIZZOR) && nsections > 1 && fsize > 64*1024 && fsize < 4*1024*1024) {
+	    if(dirs[2].Size) {
+		    struct swizz_stats *stats = cli_calloc(1, sizeof(*stats));
+		    unsigned int m = 1000;
+		    int ret = CL_CLEAN;
+
+		    if (!stats)
+			    ret = CL_EMEM;
+		    else {
+			    cli_parseres_special(EC32(dirs[2].VirtualAddress), EC32(dirs[2].VirtualAddress), desc, exe_sections, nsections, fsize, hdr_size, 0, 0, &m, stats);
+			    if ((ret = cli_detect_swizz(stats)) == CL_VIRUS) {
+				    *ctx->virname = "Trojan.Swizzor.Gen";
+			    }
+			    free(stats);
+		    }
+		    if (ret != CL_CLEAN) {
+			    free(exe_sections);
+			    return ret;
+		    }
+	    }
+    }
 
     /* UPX, FSG, MEW support */
 
@@ -1142,7 +1264,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    if(lseek(desc, fileoffset, SEEK_SET) == -1) {
 	        cli_dbgmsg("MEW: lseek() failed\n");
 		free(exe_sections);
-		return CL_EIO;
+		return CL_ESEEK;
 	    }
 
 	    if((bytes = read(desc, buff, 0xb0)) != 0xb0) {
@@ -1161,7 +1283,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 
 	    if(!cli_seeksect(desc, &exe_sections[i + 1])) {
 		free(exe_sections);
-		return CL_EIO;
+		return CL_ESEEK;
 	    }
 	    ssize = exe_sections[i + 1].vsz;
 	    dsize = exe_sections[i].vsz;
@@ -1187,7 +1309,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	      cli_dbgmsg("MEW: Can't read %d bytes [read: %lu]\n", exe_sections[i + 1].rsz, (unsigned long)bytes);
 		free(exe_sections);
 		free(src);
-		return CL_EIO;
+		return CL_EREAD;
 	    }
 	    cli_dbgmsg("MEW: %u (%08x) bytes read\n", (unsigned int)bytes, (unsigned int)bytes);
 
@@ -1351,7 +1473,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("Can't read raw data of section %d\n", i + 1);
 	    free(exe_sections);
 	    free(src);
-	    return CL_EIO;
+	    return CL_ESEEK;
 	}
 
 	dest = src + newedx - exe_sections[i + 1].rva;
@@ -1452,7 +1574,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("Can't read %d bytes from padding area\n", gp); 
 	    free(exe_sections);
 	    free(support);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	/* newebx = cli_readint32(support) - EC32(optional_hdr32.ImageBase);  Unused */
@@ -1517,7 +1639,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    free(exe_sections);
 	    free(sections);
 	    free(src);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	if((dest = (char *) cli_calloc(dsize, sizeof(char))) == NULL) {
@@ -1590,7 +1712,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("Can't read %d bytes from padding area\n", gp); 
 	    free(exe_sections);
 	    free(support);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	/* Counting original sections */
@@ -1638,7 +1760,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    free(exe_sections);
 	    free(sections);
 	    free(src);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	if((dest = (char *) cli_calloc(dsize, sizeof(char))) == NULL) {
@@ -1690,7 +1812,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    free(exe_sections);
 	    free(src);
 	    free(dest);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	/* try to detect UPX code */
@@ -1782,13 +1904,13 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    free(tempfile);
 	    free(dest);
 	    close(ndesc);
-	    return CL_EIO;
+	    return CL_EWRITE;
 	}
 
 	free(dest);
 	lseek(ndesc, 0, SEEK_SET);
 
-	if(cli_leavetemps_flag)
+	if(ctx->engine->keeptmp)
 	    cli_dbgmsg("UPX/FSG: Decompressed data saved in %s\n", tempfile);
 
 	cli_dbgmsg("***** Scanning decompressed file *****\n");
@@ -1874,7 +1996,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("PESpin: Can't read %lu bytes\n", (unsigned long)fsize);
 	    free(spinned);
 	    free(exe_sections);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	CLI_UNPTEMP("PESpin",(spinned,exe_sections,0));
@@ -1900,7 +2022,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("yC: Can't read %lu bytes\n", (unsigned long)fsize);
 	    free(spinned);
 	    free(exe_sections);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	CLI_UNPTEMP("yC",(spinned,exe_sections,0));
@@ -1920,7 +2042,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	for(i=0 ; ; i++) {
 	    if(exe_sections[i].raw<head) 
 	        head=exe_sections[i].raw;
-	    if(i==nsections-1) break;
+	    if(i+1==nsections) break;
 	    if(ssize<exe_sections[i].rva+exe_sections[i].vsz)
 		ssize=exe_sections[i].rva+exe_sections[i].vsz;
 	}
@@ -1937,7 +2059,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("WWPack: Can't read %d bytes from headers\n", head);
 	    free(src);
 	    free(exe_sections);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
         for(i = 0 ; i < (unsigned int)nsections-1; i++) {
 	    if(!exe_sections[i].rsz) continue;
@@ -1945,7 +2067,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
             if(!CLI_ISCONTAINED(src, ssize, src+exe_sections[i].rva, exe_sections[i].rsz)) break;
             if(cli_readn(desc, src+exe_sections[i].rva, exe_sections[i].rsz)!=exe_sections[i].rsz) break;
         }
-        if(i!=nsections-1) {
+        if(i+1!=nsections) {
             cli_dbgmsg("WWpack: Probably hacked/damaged file.\n");
             free(src);
             break;
@@ -1960,7 +2082,7 @@ int cli_scanpe(int desc, cli_ctx *ctx)
 	    free(src);
 	    free(packer);
 	    free(exe_sections);
-	    return CL_EIO;
+	    return CL_EREAD;
 	}
 
 	CLI_UNPTEMP("WWPack",(src,packer,exe_sections,0));
