@@ -24,10 +24,6 @@
 #include "clamav-config.h"
 #endif
 
-#ifndef CL_DEBUG
-#define NDEBUG
-#endif
-
 #ifdef CL_THREAD_SAFE
 #ifndef _REENTRANT
 #define _REENTRANT
@@ -49,9 +45,11 @@
 #include "iana_tld.h"
 #include "iana_cctld.h"
 #include "scanners.h"
-#include "md5.h"
+#include "sha256.h"
 #include <assert.h>
 
+#include "mpool.h"
+
 #define DOMAIN_REAL 1
 #define DOMAIN_DISPLAY 0
 
@@ -63,6 +61,7 @@
 #define DOMAIN_LISTED		 8
 #define PHISHY_CLOAKED_NULL	16
 
+
 /*
 * Phishing design documentation,
 (initially written at http://wiki.clamav.net/index.php/phishing_design as discussed with aCaB)
@@ -147,9 +146,9 @@ static const char src_text[] = "src";
 static const char href_text[] = "href";
 static const char mailto[] = "mailto:";
 static const char mailto_proto[] = "mailto://";
-static const char https[]="https://";
-static const char http[]="http://";
-static const char ftp[] = "ftp://";
+static const char https[]="https:";
+static const char http[]="http:";
+static const char ftp[] = "ftp:";
 
 static const size_t href_text_len = sizeof(href_text);
 static const size_t src_text_len = sizeof(src_text);
@@ -342,7 +341,7 @@ static int build_regex(regex_t* preg,const char* regex,int nosub)
 }
 
 /* allocates memory */
-static int get_host(const struct phishcheck* s,const char* URL,int isReal,int* phishy,const char **hstart, const char **hend)
+static int get_host(const char* URL,int isReal,int* phishy,const char **hstart, const char **hend)
 {
 	int rc,ismailto = 0;
 	const char* start;
@@ -427,7 +426,7 @@ rfind(char *start, char c, size_t len)
 	return (p < start) ? NULL : p;
 }
 
-static void get_domain(const struct phishcheck* pchk,struct string* dest,struct string* host)
+static void get_domain(struct string* dest,struct string* host)
 {
 	char* domain;
 	char* tld = strrchr(host->data,'.');
@@ -484,13 +483,11 @@ static int isSSL(const char* URL)
 static void
 str_hex_to_char(char **begin, const char **end)
 {
+	char *firsthex, *sbegin_;
 	char *sbegin = *begin;
 	const char *str_end = *end;
 
-	if(str_end <= sbegin)
-		return;
-
-	if(strlen(sbegin) <= 2)
+	if(str_end <= &sbegin[1])
 		return;
 
 	/* convert leading %xx*/
@@ -499,16 +496,25 @@ str_hex_to_char(char **begin, const char **end)
 		sbegin += 2;
 	}
 	*begin = sbegin++;
-	while(sbegin+3 < str_end) {
-		while(sbegin+3<str_end && sbegin[0]=='%') {
-			const char* src = sbegin+3;
+	do {
+	    sbegin_ = sbegin;
+	    firsthex = NULL;
+	    while(sbegin+3 <= str_end) {
+		if (sbegin+3<=str_end && sbegin[0]=='%') {
+		    const char* src = sbegin+3;
+		    if (isxdigit(sbegin[1]) && isxdigit(sbegin[2])) {
 			*sbegin = hex2int((unsigned char*)sbegin+1);
+			if (*sbegin == '%' && !firsthex)
+			    firsthex = sbegin;
 			/* move string */
 			memmove(sbegin+1,src,str_end-src+1);
 			str_end -= 2;
+		    }
 		}
 		sbegin++;
-	}
+	    }
+	    sbegin = sbegin_;
+	} while (firsthex);
 	*end = str_end;
 }
 
@@ -724,7 +730,7 @@ cleanupURL(struct string *URL,struct string *pre_URL, int isReal)
 }
 
 /* -------end runtime disable---------*/
-int phishingScan(const char* dir,cli_ctx* ctx,tag_arguments_t* hrefs)
+int phishingScan(cli_ctx* ctx,tag_arguments_t* hrefs)
 {
 	/* TODO: get_host and then apply regex, etc. */
 	int i;
@@ -768,8 +774,7 @@ int phishingScan(const char* dir,cli_ctx* ctx,tag_arguments_t* hrefs)
 	fclose(f);
 	return 0;
 #endif
-	for(i=0;i<hrefs->count;i++)
-		if(hrefs->contents[i]) {
+	for(i=0;i<hrefs->count;i++) {
 			struct url_check urls;
 			enum phish_status rc;
 			urls.flags	 = strncmp((char*)hrefs->tag[i],href_text,href_text_len)? (CL_PHISH_ALL_CHECKS&~CHECK_SSL): CL_PHISH_ALL_CHECKS;
@@ -787,7 +792,7 @@ int phishingScan(const char* dir,cli_ctx* ctx,tag_arguments_t* hrefs)
 				urls.always_check_flags |= CHECK_CLOAKING;
 			}
 			string_init_c(&urls.realLink,(char*)hrefs->value[i]);
-			string_init_c(&urls.displayLink, hrefs->contents[i]);
+			string_init_c(&urls.displayLink, (char*)hrefs->contents[i]);
 			string_init_c(&urls.pre_fixup.pre_displayLink, NULL);
 
 			urls.realLink.refcount=-1;
@@ -821,20 +826,21 @@ int phishingScan(const char* dir,cli_ctx* ctx,tag_arguments_t* hrefs)
 					*ctx->virname="Phishing.Heuristics.Email.Cloaked.Username";/*http://banksite@fake.example.com*/
 					break;
 				case CL_PHISH_HASH0:
+					*ctx->virname="Safebrowsing.Suspected-malware_safebrowsing.clamav.net";
+					break;
 				case CL_PHISH_HASH1:
-				case CL_PHISH_HASH2:
 					*ctx->virname="Phishing.URL.Blacklisted";
 					break;
+				case CL_PHISH_HASH2:
+					*ctx->virname="Safebrowsing.Suspected-phishing_safebrowsing.clamav.net";
+					break;
 				case CL_PHISH_NOMATCH:
 				default:
 					*ctx->virname="Phishing.Heuristics.Email.SpoofedDomain";
 					break;
 			}
 			return cli_found_possibly_unwanted(ctx);
-		}
-		else
-			if(strcmp((char*)hrefs->tag[i],"href"))
-					cli_dbgmsg("Phishcheck: href with no contents?\n");
+	}
 	return CL_CLEAN;
 }
 
@@ -857,7 +863,7 @@ int phishing_init(struct cl_engine* engine)
 {
 	struct phishcheck* pchk;
 	if(!engine->phishcheck) {
-		pchk = engine->phishcheck = cli_malloc(sizeof(struct phishcheck));
+		pchk = engine->phishcheck = mpool_malloc(engine->mempool, sizeof(struct phishcheck));
 		if(!pchk)
 			return CL_EMEM;
 		pchk->is_disabled=1;
@@ -875,7 +881,7 @@ int phishing_init(struct cl_engine* engine)
 	cli_dbgmsg("Initializing phishcheck module\n");
 
 	if(build_regex(&pchk->preg_numeric,numeric_url_regex,1)) {
-		free(pchk);
+		mpool_free(engine->mempool, pchk);
 		engine->phishcheck = NULL;
 		return CL_EFORMAT;
 	}
@@ -895,7 +901,7 @@ void phishing_done(struct cl_engine* engine)
 	domainlist_done(engine);
 	if(pchk) {
 		cli_dbgmsg("Freeing phishcheck struct\n");
-		free(pchk);
+		mpool_free(engine->mempool, pchk);
 	}
 	cli_dbgmsg("Phishcheck cleaned up\n");
 }
@@ -963,8 +969,8 @@ static const uint8_t URI_xpalpha_nodot[256] = {
 
 static inline int validate_uri_xalphas_nodot(const char *start, const char *end)
 {
-	const unsigned char *p = start;
-	for(p=start;p < (const unsigned char*)end; p++) {
+	const unsigned char *p;
+	for(p=(const unsigned char*)start;p < (const unsigned char*)end; p++) {
 		if(!URI_xalpha_nodot[*p])
 			return 0;
 	}
@@ -973,8 +979,8 @@ static inline int validate_uri_xalphas_nodot(const char *start, const char *end)
 
 static inline int validate_uri_xpalphas_nodot(const char *start, const char *end)
 {
-	const unsigned char *p = start;
-	for(p=start;p < (const unsigned char*)end; p++) {
+	const unsigned char *p;
+	for(p=(const unsigned char*)start;p < (const unsigned char*)end; p++) {
 		if(!URI_xpalpha_nodot[*p])
 			return 0;
 	}
@@ -985,7 +991,7 @@ static inline int validate_uri_xpalphas_nodot(const char *start, const char *end
 
 static inline int validate_uri_ialpha(const char *start, const char *end)
 {
-	const unsigned char *p = start;
+	const unsigned char *p = (const unsigned char*) start;
 	if(start >= end || !URI_alpha[*p])
 		return 0;
 	return validate_uri_xalphas_nodot(start + 1, end);
@@ -994,10 +1000,10 @@ static inline int validate_uri_ialpha(const char *start, const char *end)
 /*
  * Only those URLs are identified as URLs for which phishing detection can be performed.
  */
-static int isURL(const struct phishcheck* pchk,const char* URL, int accept_anyproto)
+static int isURL(char* URL, int accept_anyproto)
 {
-	size_t len;
-	const char *start = NULL, *p, *q, *end;
+	char *last_tld_end = NULL, *q;
+	const char *start = NULL, *p, *end;
 	if(!URL)
 		return 0;
 
@@ -1005,33 +1011,34 @@ static int isURL(const struct phishcheck* pchk,const char* URL, int accept_anypr
 	switch (URL[0]) {
 		case 'h':
 			if (strncmp(URL, https, https_len) == 0)
-				start = URL + https_len;
+				start = URL + https_len - 1;
 			else if (strncmp(URL, http, http_len) == 0)
-				start = URL + http_len;
+				start = URL + http_len - 1;
 			break;
 		case 'f':
 		       if (strncmp(URL, ftp, ftp_len) == 0)
-			       start = URL + ftp_len;
+			       start = URL + ftp_len - 1;
 		       break;
 		case 'm':
 		       if (strncmp(URL, mailto_proto, mailto_proto_len) == 0)
-			       start = URL + mailto_proto_len;
+			       start = URL + mailto_proto_len - 1;
 		       break;
 	}
-	if(start) {
-		if(start[0] == '\0')
-			return 0;/* empty URL */
+	if(start && start[1] == '/' && start[2] == '/') {
 		/* has a valid protocol, it is a URL */
 		return 1;
 	}
-	start = accept_anyproto ?  strchr(URL, ':') : NULL;
+	start = accept_anyproto ?  strchr(URL, ':') : start;
 	if(start) {
 		/* validate URI scheme */
 		if(validate_uri_ialpha(URL, start)) {
-			if(start[1] == '/' && start[2] == '/')
-				start += 3; /* skip :// */
-			else
+			/* skip :// */
+			if (start[1] == '/') {
+			    start += 2;
+			    if (*start == '/')
 				start++;
+			} else
+			    start++;
 		}
 		else
 			start = URL; /* scheme invalid */
@@ -1048,6 +1055,8 @@ static int isURL(const struct phishcheck* pchk,const char* URL, int accept_anypr
 		if(q) {
 			if(!validate_uri_xpalphas_nodot(p, q))
 				return 0;
+			if (accept_anyproto && in_tld_set(p, q-p))
+			    last_tld_end = q;
 			p = q+1;
 		}
 	} while(q);
@@ -1056,7 +1065,16 @@ static int isURL(const struct phishcheck* pchk,const char* URL, int accept_anypr
 	if (end < p)
 		end = p;
 	while (*end == ' ' && end > p) --end;
-	return !!in_tld_set(p, end - p);
+
+	if (in_tld_set(p, end - p))
+	    return 1;
+	if (!accept_anyproto)
+	    return 0;
+	if (last_tld_end) {
+	    *last_tld_end = '\0';
+	    return 1;
+	}
+	return 0;
 }
 
 /*
@@ -1091,13 +1109,13 @@ static enum phish_status cleanupURLs(struct url_check* urls)
 	return CL_PHISH_NODECISION;
 }
 
-static int url_get_host(const struct phishcheck* pchk, struct url_check* url,struct url_check* host_url,int isReal,int* phishy)
+static int url_get_host(struct url_check* url,struct url_check* host_url,int isReal,int* phishy)
 {
 	const char *start, *end;
 	struct string* host = isReal ? &host_url->realLink : &host_url->displayLink;
 	const char* URL = isReal ? url->realLink.data : url->displayLink.data;
 	int rc;
-	if ((rc = get_host(pchk, URL, isReal, phishy, &start, &end))) {
+	if ((rc = get_host(URL, isReal, phishy, &start, &end))) {
 		return rc;
 	}
 	if(!start || !end) {
@@ -1125,14 +1143,15 @@ static int url_get_host(const struct phishcheck* pchk, struct url_check* url,str
 	if(!isReal) {
 		url->pre_fixup.host_start = start - URL;
 		url->pre_fixup.host_end = end - URL;
+		url->pre_fixup.pre_displayLink.data[url->pre_fixup.host_end] = '\0';
 	}
 	return CL_PHISH_NODECISION;
 }
 
-static void url_get_domain(const struct phishcheck* pchk, struct url_check* url,struct url_check* domains)
+static void url_get_domain(struct url_check* url,struct url_check* domains)
 {
-	get_domain(pchk, &domains->realLink, &url->realLink);
-	get_domain(pchk, &domains->displayLink, &url->displayLink);
+	get_domain(&domains->realLink, &url->realLink);
+	get_domain(&domains->displayLink, &url->displayLink);
 	domains->flags = url->flags;
 }
 
@@ -1151,7 +1170,7 @@ static int whitelist_check(const struct cl_engine* engine,struct url_check* urls
 	return whitelist_match(engine,urls->realLink.data,urls->displayLink.data,hostOnly);
 }
 
-static int hash_match(const struct regex_matcher *rlist, const char *host, size_t hlen, const char *path, size_t plen)
+static int hash_match(const struct regex_matcher *rlist, const char *host, size_t hlen, const char *path, size_t plen, int *prefix_matched)
 {
 	const char *virname;
 #if 0
@@ -1161,79 +1180,180 @@ static int hash_match(const struct regex_matcher *rlist, const char *host, size_
 	s[hlen+plen] = '\0';
 	cli_dbgmsg("hash lookup for: %s\n",s);
 #endif
-	if(rlist->md5_hashes.bm_patterns) {
-		unsigned char md5_dig[16];
-		cli_md5_ctx md5;
+	if(rlist->sha256_hashes.bm_patterns) {
+	    const char hexchars[] = "0123456789ABCDEF";
+	    unsigned char h[65];
+	    unsigned char sha256_dig[32];
+	    unsigned i;
+	    SHA256_CTX sha256;
 
-		cli_md5_init(&md5);
-		cli_md5_update(&md5, host, hlen);
-		cli_md5_update(&md5, path, plen);
-		cli_md5_final(md5_dig, &md5);
-		if(SO_search(&rlist->md5_filter, md5_dig, 16) != -1 &&
-				cli_bm_scanbuff(md5_dig, 16, &virname, &rlist->md5_hashes,0,0,-1) == CL_VIRUS) {
-			switch(*virname) {
-				case '1':
-					return CL_PHISH_HASH1;
-				case '2':
-					return CL_PHISH_HASH2;
-				default:
-					return CL_PHISH_HASH0;
-			}
+	    sha256_init(&sha256);
+	    sha256_update(&sha256, host, hlen);
+	    sha256_update(&sha256, path, plen);
+	    sha256_final(&sha256, sha256_dig);
+	    for(i=0;i<32;i++) {
+		h[2*i] = hexchars[sha256_dig[i]>>4];
+		h[2*i+1] = hexchars[sha256_dig[i]&0xf];
+	    }
+	    h[64]='\0';
+	    cli_dbgmsg("Looking up hash %s for %s(%u)%s(%u)\n", h, host, (unsigned)hlen, path, (unsigned)plen);
+	    if (prefix_matched) {
+		if (cli_bm_scanbuff(sha256_dig, 4, &virname, &rlist->hostkey_prefix,0,0,-1) == CL_VIRUS) {
+		    cli_dbgmsg("prefix matched\n");
+		    *prefix_matched = 1;
+		} else
+		    return CL_SUCCESS;
+	    }
+	    if (cli_bm_scanbuff(sha256_dig, 32, &virname, &rlist->sha256_hashes,0,0,-1) == CL_VIRUS) {
+		cli_dbgmsg("This hash matched: %s\n", h);
+		switch(*virname) {
+		    case 'W':
+			cli_dbgmsg("Hash is whitelisted, skipping\n");
+			break;
+		    case '1':
+			return CL_PHISH_HASH1;
+		    case '2':
+			return CL_PHISH_HASH2;
+		    default:
+			return CL_PHISH_HASH0;
 		}
+	    }
 	}
 	return CL_SUCCESS;
 }
 
 #define URL_MAX_LEN 1024
 #define COMPONENTS 4
-static int url_hash_match(const struct regex_matcher *rlist, const char *inurl, size_t len)
+int cli_url_canon(const char *inurl, size_t len, char *urlbuff, size_t dest_len, char **host, size_t *hostlen, const char **path, size_t *pathlen)
 {
-	char urlbuff[URL_MAX_LEN+3];/* htmlnorm truncates at 1024 bytes + terminating null + slash + host end null */
-	char *url, *p;
+	char *url, *p, *last;
+	char *host_begin, *path_begin;
 	const char *urlend = urlbuff + len;
-	char *host_begin;
 	size_t host_len, path_len;
-	char *path_begin;
-	const char *component;
-	const char *lp[COMPONENTS+1];
-	size_t pp[COMPONENTS+2];
-	size_t j, k, ji, ki;
-	int rc;
 
-	if(!rlist || !rlist->md5_hashes.bm_patterns) {
-		return CL_SUCCESS;
-	}
-	if(!inurl)
-		return CL_EMEM;
-	strncpy(urlbuff, inurl, URL_MAX_LEN);
-	urlbuff[URL_MAX_LEN] = urlbuff[URL_MAX_LEN+1] = urlbuff[URL_MAX_LEN+2] = '\0';
+	dest_len -= 3;
+	strncpy(urlbuff, inurl, dest_len);
+	urlbuff[dest_len] = urlbuff[dest_len+1] = urlbuff[dest_len+2] = '\0';
 	url = urlbuff;
-	str_hex_to_char(&url, &urlend);
-	len = urlend - url;
-	host_begin = strchr(url,':');
-	if(!host_begin) {
+
+	/* canonicalize only real URLs, with a protocol */
+	host_begin = strchr(url, ':');
+	if(!host_begin)
 		return CL_PHISH_CLEAN;
-	}
 	++host_begin;
+
+	/* ignore username in URL */
+	p = strchr(host_begin, '@');
+	if (p)
+	    host_begin = p+1;
+	url = host_begin;
+	/* repeatedly % unescape characters */
+	str_hex_to_char(&url, &urlend);
+	host_begin = url;
+	len = urlend - url;
+	/* skip to beginning of hostname */
 	while((host_begin < urlend) && *host_begin == '/') ++host_begin;
 	while(*host_begin == '.' && host_begin < urlend) ++host_begin;
-	p = strchr(host_begin, '@');
-	if(p)
-		host_begin = p+1;
 
+	last = strchr(host_begin, '/');
+	p = host_begin;
+	while (p < urlend) {
+	    if (p+2 < urlend && *p == '/' && p[1] == '.' ) {
+		if (p[2] == '/') {
+		    /* remove /./ */
+		    if (p + 3 < urlend)
+			memmove(p+1, p+3, urlend - p - 3);
+		    urlend -= 2;
+		}
+		else if (p[2] == '.' && (p[3] == '/' || p[3] == '\0') && last) {
+		    /* remove /component/../ */
+		    if (p+4 < urlend)
+			memmove(last+1, p+4, urlend - p - 4);
+		    urlend -= 3 + (p - last);
+		}
+	    }
+	    if (*p == '/')
+		last = p;
+	    p++;
+	}
+	p = &url[urlend - url];
+	*p = '\0';
+
+	p = host_begin;
+	while (p < urlend && p+2 < url + dest_len && urlend < urlbuff+dest_len) {
+	    unsigned char c = *p;
+	    if (c <= 32 || c >= 127 || c == '%' || c == '#') {
+		/* convert non-ascii characters back to % escaped */
+		const char hexchars[] = "0123456789ABCDEF";
+		memmove(p+3, p+1, urlend - p - 1);
+		*p++ = '%';
+		*p++ = hexchars[c>>4];
+		*p = hexchars[c&0xf];
+		urlend += 2;
+	    }
+	    p++;
+	}
+	*p = '\0';
+	urlend = p;
+	len = urlend - url;
+	/* determine end of hostname */
 	host_len = strcspn(host_begin, ":/?");
 	path_begin = host_begin + host_len;
-	if(host_len < len) {
+	if(host_len <= len) {
+		/* url without path, use a single / */
 		memmove(path_begin + 2, path_begin + 1, len - host_len);
 		*path_begin++ = '/';
 		*path_begin++ = '\0';
 	} else path_begin = url+len;
 	if(url + len >= path_begin) {
 		path_len = url + len - path_begin + 1;
-	} else
+		p = strchr(path_begin, '#');
+		if (p) {
+		    /* ignore anchor */
+		    *p = '\0';
+		    path_len = p - path_begin;
+		}
+		*path = path_begin;
+	} else {
 		path_len = 0;
+		*path = "";
+	}
+	/* lowercase entire URL */
 	str_make_lowercase(host_begin, host_len);
+	*host = host_begin;
+	*hostlen = host_len;
+	*pathlen = path_len;
+	return CL_PHISH_NODECISION;
+}
 
+static int url_hash_match(const struct regex_matcher *rlist, const char *inurl, size_t len)
+{
+	size_t j, k, ji, ki;
+	char *host_begin;
+	const char *path_begin;
+	const char *component;
+	size_t path_len;
+	size_t host_len;
+	char *p;
+	int rc, prefix_matched=0;
+	const char *lp[COMPONENTS+1];
+	size_t pp[COMPONENTS+2];
+	char urlbuff[URL_MAX_LEN+3];/* htmlnorm truncates at 1024 bytes + terminating null + slash + host end null */
+	unsigned count;
+
+	if(!rlist || !rlist->sha256_hashes.bm_patterns) {
+		/* no hashes loaded -> don't waste time canonicalizing and
+		 * looking up */
+		return CL_SUCCESS;
+	}
+	if(!inurl)
+		return CL_EMEM;
+
+	rc = cli_url_canon(inurl, len, urlbuff, sizeof(urlbuff), &host_begin, &host_len, &path_begin, &path_len);
+	if (rc == CL_PHISH_CLEAN)
+	    return rc;
+
+	/* get last 5 components of hostname */
 	j=COMPONENTS;
 	component = strrchr(host_begin, '.');
 	while(component && j > 0) {
@@ -1247,6 +1367,7 @@ static int url_hash_match(const struct regex_matcher *rlist, const char *inurl,
 	}
 	lp[j] = host_begin;
 
+	/* get first 5 components of path */
 	pp[0] = path_len;
 	if(path_len) {
 		pp[1] = strcspn(path_begin, "?");
@@ -1262,15 +1383,31 @@ static int url_hash_match(const struct regex_matcher *rlist, const char *inurl,
 		}
 	} else
 		k = 1;
-
-	for(ji=j;ji < COMPONENTS+1; ji++) {
-		for(ki=0;ki < k; ki++) {
-			assert(pp[ki] <= path_len);
-			rc = hash_match(rlist, lp[ji], host_begin + host_len - lp[ji] + 1, path_begin, pp[ki]);
-			if(rc) {
-				return rc;
-			}
+	count = 0;
+	for(ki=k;ki > 0;) {
+	    --ki;
+	    for(ji=COMPONENTS+1;ji > j;) {
+		/* lookup last 2 and 3 components of host, as hostkey prefix,
+		 * if not matched, shortcircuit lookups */
+		int need_prefixmatch = (count<2 && !prefix_matched) &&
+				       rlist->hostkey_prefix.bm_patterns;
+		--ji;
+		assert(pp[ki] <= path_len);
+		/* lookup prefix/suffix hashes of URL */
+		rc = hash_match(rlist, lp[ji], host_begin + host_len - lp[ji] + 1, path_begin, pp[ki], 
+				need_prefixmatch ? &prefix_matched : NULL);
+		if(rc) {
+		    return rc;
 		}
+		count++;
+		if (count == 2 && !prefix_matched && rlist->hostkey_prefix.bm_patterns) {
+		    /* if hostkey is not matched, don't bother calculating
+		     * hashes for other parts of the URL, they are not in the DB
+		     */
+		    cli_dbgmsg("hostkey prefix not matched, short-circuiting lookups\n");
+		    return CL_SUCCESS;
+		}
+	    }
 	}
 	return CL_SUCCESS;
 }
@@ -1283,7 +1420,7 @@ static enum phish_status phishingCheck(const struct cl_engine* engine,struct url
 	int phishy=0;
 	const struct phishcheck* pchk = (const struct phishcheck*) engine->phishcheck;
 
-	if(!urls->realLink.data || urls->displayLink.data[0]=='\0')
+	if(!urls->realLink.data)
 		return CL_PHISH_CLEAN;
 
 	cli_dbgmsg("Phishcheck:Checking url %s->%s\n", urls->realLink.data,
@@ -1292,14 +1429,23 @@ static enum phish_status phishingCheck(const struct cl_engine* engine,struct url
 	if(!strcmp(urls->realLink.data,urls->displayLink.data))
 		return CL_PHISH_CLEAN;/* displayed and real URL are identical -> clean */
 
-	if(!isURL(pchk, urls->realLink.data, 0)) {
+	if(!isURL(urls->realLink.data, 0)) {
 		cli_dbgmsg("Real 'url' is not url:%s\n",urls->realLink.data);
 		return CL_PHISH_CLEAN;
 	}
 
 	if(( rc = url_hash_match(engine->domainlist_matcher, urls->realLink.data, strlen(urls->realLink.data)) )) {
+	    if (rc == CL_PHISH_CLEAN) {
+		cli_dbgmsg("not analyzing, not a real url: %s\n", urls->realLink.data);
+		return CL_PHISH_CLEAN;
+	    } else {
 		cli_dbgmsg("Hash matched for: %s\n", urls->realLink.data);
 		return rc;
+	    }
+	}
+
+	if (urls->displayLink.data[0] == '\0') {
+	    return CL_PHISH_CLEAN;
 	}
 
 	if((rc = cleanupURLs(urls))) {
@@ -1311,7 +1457,7 @@ static enum phish_status phishingCheck(const struct cl_engine* engine,struct url
 	cli_dbgmsg("Phishcheck:URL after cleanup: %s->%s\n", urls->realLink.data,
 		urls->displayLink.data);
 
-	if((!isURL(pchk, urls->displayLink.data, 1) ) &&
+	if((!isURL(urls->displayLink.data, 1) ) &&
 			( (phishy&PHISHY_NUMERIC_IP && !isNumericURL(pchk, urls->displayLink.data)) ||
 			  !(phishy&PHISHY_NUMERIC_IP))) {
 		cli_dbgmsg("Displayed 'url' is not url:%s\n",urls->displayLink.data);
@@ -1323,12 +1469,12 @@ static enum phish_status phishingCheck(const struct cl_engine* engine,struct url
 
 	url_check_init(&host_url);
 
-	if((rc = url_get_host(pchk, urls, &host_url, DOMAIN_DISPLAY, &phishy))) {
+	if((rc = url_get_host(urls, &host_url, DOMAIN_DISPLAY, &phishy))) {
 		free_if_needed(&host_url);
 		return rc < 0 ? rc : CL_PHISH_CLEAN;
 	}
 
-	if (domainlist_match(engine,host_url.displayLink.data,host_url.realLink.data,&urls->pre_fixup,1,&urls->flags)) {
+	if (domainlist_match(engine, host_url.displayLink.data,host_url.realLink.data,&urls->pre_fixup,1)) {
 		phishy |= DOMAIN_LISTED;
 	} else {
 		urls->flags &= urls->always_check_flags;
@@ -1361,7 +1507,7 @@ static enum phish_status phishingCheck(const struct cl_engine* engine,struct url
 		return CL_PHISH_CLEAN;
 	}
 
-	if((rc = url_get_host(pchk, urls,&host_url,DOMAIN_REAL,&phishy)))
+	if((rc = url_get_host(urls,&host_url,DOMAIN_REAL,&phishy)))
 	{
 		free_if_needed(&host_url);
 		return rc < 0 ? rc : CL_PHISH_CLEAN;
@@ -1380,7 +1526,7 @@ static enum phish_status phishingCheck(const struct cl_engine* engine,struct url
 	{
 		struct url_check domain_url;
 		url_check_init(&domain_url);
-		url_get_domain(pchk, &host_url,&domain_url);
+		url_get_domain(&host_url,&domain_url);
 		if(!strcmp(domain_url.realLink.data,domain_url.displayLink.data)) {
 			free_if_needed(&host_url);
 			free_if_needed(&domain_url);
