@@ -91,15 +91,12 @@
 #include <zlib.h>
 #include "unzip.h"
 #include "dlp.h"
+#include "default.h"
 
 #ifdef HAVE_BZLIB_H
 #include <bzlib.h>
 #endif
 
-#ifdef ENABLE_UNRAR
-#include "libclamunrar_iface/unrar_iface.h"
-#endif
-
 #if defined(HAVE_READDIR_R_3) || defined(HAVE_READDIR_R_2)
 #include <limits.h>
 #include <stddef.h>
@@ -164,7 +161,8 @@ static int cli_scandir(const char *dirname, cli_ctx *ctx, cli_file_t container)
 				if(container == CL_TYPE_MAIL) {
 				    fd = open(fname, O_RDONLY|O_BINARY);
 				    if(fd == -1) {
-					    cli_warnmsg("Cannot open file %s: %s, mode: %x\n", fname, strerror(errno), statbuf.st_mode);
+					    char err[128];
+					    cli_warnmsg("Cannot open file %s: %s, mode: %x\n", fname, cli_strerror(errno, err, sizeof(err)), statbuf.st_mode);
 					    free(fname);
 					    continue;
 				    }
@@ -196,7 +194,6 @@ static int cli_scandir(const char *dirname, cli_ctx *ctx, cli_file_t container)
     return CL_CLEAN;
 }
 
-#ifdef ENABLE_UNRAR
 static int cli_unrar_scanmetadata(int desc, unrar_metadata_t *metadata, cli_ctx *ctx, unsigned int files, uint32_t* sfx_check)
 {
 	int ret = CL_SUCCESS;
@@ -276,10 +273,10 @@ static int cli_scanrar(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 
     if(sfx_offset)
 	if(lseek(desc, sfx_offset, SEEK_SET) == -1)
-	    return CL_EIO;
+	    return CL_ESEEK;
 
     /* generate the temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -288,8 +285,8 @@ static int cli_scanrar(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 	return CL_ETMPDIR;
     }
 
-    if((ret = unrar_open(desc, dir, &rar_state)) != UNRAR_OK) {
-	if(!cli_leavetemps_flag)
+    if((ret = cli_unrar_open(desc, dir, &rar_state)) != UNRAR_OK) {
+	if(!ctx->engine->keeptmp)
 	    cli_rmdirs(dir);
 	free(dir);
 	if(ret == UNRAR_PASSWD) {
@@ -305,43 +302,48 @@ static int cli_scanrar(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 	} if(ret == UNRAR_EMEM) {
 	    return CL_EMEM;
 	} else {
-	    return CL_ERAR;
+	    return CL_EUNPACK;
 	}
     }
 
     do {
 	int rc;
 	rar_state.ofd = -1;
-	ret = unrar_extract_next_prepare(&rar_state,dir);
+	ret = cli_unrar_extract_next_prepare(&rar_state,dir);
 	if(ret != UNRAR_OK) {
 	    if(ret == UNRAR_BREAK)
 		ret = CL_BREAK;
 	    else if(ret == UNRAR_EMEM)
 		ret = CL_EMEM;
 	    else
-		ret = CL_ERAR;
+		ret = CL_EUNPACK;
 	    break;
 	}
-	if((ret=cli_checklimits("RAR", ctx, rar_state.metadata_tail->unpack_size, rar_state.metadata_tail->pack_size, 0)!=CL_CLEAN)) {
+	if(ctx->engine->maxscansize && ctx->scansize >= ctx->engine->maxscansize) {
 	    free(rar_state.file_header->filename);
 	    free(rar_state.file_header);
 	    ret = CL_CLEAN;
-	    continue;
+	    break;
 	}
-	ret = unrar_extract_next(&rar_state,dir);
+	if(ctx->engine->maxscansize && ctx->scansize + ctx->engine->maxfilesize >= ctx->engine->maxscansize)
+	    rar_state.maxfilesize = ctx->engine->maxscansize - ctx->scansize;
+	else
+	    rar_state.maxfilesize = ctx->engine->maxfilesize;
+
+	ret = cli_unrar_extract_next(&rar_state,dir);
 	if(ret == UNRAR_OK)
 	    ret = CL_SUCCESS;
 	else if(ret == UNRAR_EMEM)
 	    ret = CL_EMEM;
 	else
-	    ret = CL_ERAR;
+	    ret = CL_EFORMAT;
 
 	if(rar_state.ofd > 0) {
 	    lseek(rar_state.ofd,0,SEEK_SET);
 	    rc = cli_magic_scandesc(rar_state.ofd,ctx);
 	    close(rar_state.ofd);
-	    if(!cli_leavetemps_flag) 
-		if (cli_unlink(rar_state.filename)) ret = CL_EIO;
+	    if(!ctx->engine->keeptmp) 
+		if (cli_unlink(rar_state.filename)) ret = CL_EUNLINK;
 	    if(rc == CL_VIRUS ) {
 		cli_dbgmsg("RAR: infected with %s\n",*ctx->virname);
 		ret = CL_VIRUS;
@@ -362,9 +364,9 @@ static int cli_scanrar(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
     if(cli_scandir(rar_state.comment_dir, ctx, 0) == CL_VIRUS)
 	ret = CL_VIRUS;
 
-    unrar_close(&rar_state);
+    cli_unrar_close(&rar_state);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
         cli_rmdirs(dir);
 
     free(dir);
@@ -380,7 +382,6 @@ static int cli_scanrar(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 
     return ret;
 }
-#endif /* ENABLE_UNRAR */
 
 static int cli_scanarj(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_check)
 {
@@ -391,7 +392,7 @@ static int cli_scanarj(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
     cli_dbgmsg("in cli_scanarj()\n");
 
      /* generate the temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -405,7 +406,7 @@ static int cli_scanarj(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 
     ret = cli_unarj_open(desc, dir);
     if (ret != CL_SUCCESS) {
-	if(!cli_leavetemps_flag)
+	if(!ctx->engine->keeptmp)
 	    cli_rmdirs(dir);
 	free(dir);
 	cli_dbgmsg("ARJ: Error: %s\n", cl_strerror(ret));
@@ -441,7 +442,7 @@ static int cli_scanarj(int desc, cli_ctx *ctx, off_t sfx_offset, uint32_t *sfx_c
 
     } while(ret == CL_SUCCESS);
     
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -469,10 +470,10 @@ static int cli_scangzip(int desc, cli_ctx *ctx)
 
     if((gd = gzdopen(dup(desc), "rb")) == NULL) {
 	cli_dbgmsg("GZip: Can't open descriptor %d\n", desc);
-	return CL_EGZIP;
+	return CL_EOPEN;
     }
 
-    if((ret = cli_gentempfd(NULL, &tmpname, &fd))) {
+    if((ret = cli_gentempfd(ctx->engine->tmpdir, &tmpname, &fd))) {
 	cli_dbgmsg("GZip: Can't generate temporary file.\n");
 	gzclose(gd);
 	return ret;
@@ -482,10 +483,10 @@ static int cli_scangzip(int desc, cli_ctx *ctx)
 	cli_dbgmsg("GZip: Unable to malloc %u bytes.\n", FILEBUFF);
 	gzclose(gd);
 	close(fd);
-	if(!cli_leavetemps_flag) {
+	if(!ctx->engine->keeptmp) {
 	    if(cli_unlink(tmpname)) {
 	    	free(tmpname);
-		return CL_EIO;
+		return CL_EUNLINK;
 	    }
 	}
 	return CL_EMEM;
@@ -500,18 +501,18 @@ static int cli_scangzip(int desc, cli_ctx *ctx)
 	if(cli_writen(fd, buff, bytes) != bytes) {
 	    cli_dbgmsg("GZip: Can't write to file.\n");
 	    close(fd);
-	    if(!cli_leavetemps_flag) {
+	    if(!ctx->engine->keeptmp) {
 	    	if (cli_unlink(tmpname)) {
 		    free(tmpname);
 		    gzclose(gd);
 		    free(buff);
-		    return CL_EIO;
+		    return CL_EUNLINK;
 		}
 	    }
 	    free(tmpname);	
 	    gzclose(gd);
 	    free(buff);
-	    return CL_EGZIP;
+	    return CL_EWRITE;
 	}
     }
 
@@ -520,8 +521,8 @@ static int cli_scangzip(int desc, cli_ctx *ctx)
 
     if(ret == CL_VIRUS) {
 	close(fd);
-	if(!cli_leavetemps_flag)
-	    if (cli_unlink(tmpname)) ret = CL_EIO;
+	if(!ctx->engine->keeptmp)
+	    if (cli_unlink(tmpname)) ret = CL_EUNLINK;
 	free(tmpname);	
 	return ret;
     }
@@ -530,18 +531,18 @@ static int cli_scangzip(int desc, cli_ctx *ctx)
     if((ret = cli_magic_scandesc(fd, ctx)) == CL_VIRUS ) {
 	cli_dbgmsg("GZip: Infected with %s\n", *ctx->virname);
 	close(fd);
-	if(!cli_leavetemps_flag) {
+	if(!ctx->engine->keeptmp) {
 	    if (cli_unlink(tmpname)) {
 	    	free(tmpname);
-		return CL_EIO;
+		return CL_EUNLINK;
 	    }
 	}
 	free(tmpname);	
 	return CL_VIRUS;
     }
     close(fd);
-    if(!cli_leavetemps_flag)
-	if (cli_unlink(tmpname)) ret = CL_EIO;
+    if(!ctx->engine->keeptmp)
+	if (cli_unlink(tmpname)) ret = CL_EUNLINK;
     free(tmpname);	
 
     return ret;
@@ -565,7 +566,6 @@ static int cli_scanbzip(int desc, cli_ctx *ctx) {
 static int cli_scanbzip(int desc, cli_ctx *ctx)
 {
 	int fd, bytes, ret = CL_CLEAN, bzerror = 0;
-	short memlim = 0;
 	unsigned long int size = 0;
 	char *buff;
 	FILE *fs;
@@ -575,20 +575,16 @@ static int cli_scanbzip(int desc, cli_ctx *ctx)
 
     if((fs = fdopen(dup(desc), "rb")) == NULL) {
 	cli_dbgmsg("Bzip: Can't open descriptor %d.\n", desc);
-	return CL_EBZIP;
+	return CL_EOPEN;
     }
 
-    if(ctx->limits)
-	if(ctx->limits->archivememlim)
-	    memlim = 1;
-
-    if((bfd = BZ2_bzReadOpen(&bzerror, fs, 0, memlim, NULL, 0)) == NULL) {
+    if((bfd = BZ2_bzReadOpen(&bzerror, fs, 0, 0, NULL, 0)) == NULL) {
 	cli_dbgmsg("Bzip: Can't initialize bzip2 library (descriptor: %d).\n", desc);
 	fclose(fs);
-	return CL_EBZIP;
+	return CL_EOPEN;
     }
 
-    if((ret = cli_gentempfd(NULL, &tmpname, &fd))) {
+    if((ret = cli_gentempfd(ctx->engine->tmpdir, &tmpname, &fd))) {
 	cli_dbgmsg("Bzip: Can't generate temporary file.\n");
 	BZ2_bzReadClose(&bzerror, bfd);
 	fclose(fs);
@@ -598,12 +594,12 @@ static int cli_scanbzip(int desc, cli_ctx *ctx)
     if(!(buff = (char *) cli_malloc(FILEBUFF))) {
 	cli_dbgmsg("Bzip: Unable to malloc %u bytes.\n", FILEBUFF);
 	close(fd);
-	if(!cli_leavetemps_flag) {
+	if(!ctx->engine->keeptmp) {
 	    if (cli_unlink(tmpname)) {
 	    	free(tmpname);
 		fclose(fs);
 		BZ2_bzReadClose(&bzerror, bfd);
-		return CL_EIO;
+		return CL_EUNLINK;
 	    }
 	}
 	free(tmpname);	
@@ -622,18 +618,18 @@ static int cli_scanbzip(int desc, cli_ctx *ctx)
 	    cli_dbgmsg("Bzip: Can't write to file.\n");
 	    BZ2_bzReadClose(&bzerror, bfd);
 	    close(fd);
-	    if(!cli_leavetemps_flag) {
+	    if(!ctx->engine->keeptmp) {
 		if (cli_unlink(tmpname)) {
 		    free(tmpname);
 		    free(buff);
 		    fclose(fs);
-		    return CL_EIO;
+		    return CL_EUNLINK;
 		}
 	    }
 	    free(tmpname);	
 	    free(buff);
 	    fclose(fs);
-	    return CL_EGZIP;
+	    return CL_EWRITE;
 	}
     }
 
@@ -642,8 +638,8 @@ static int cli_scanbzip(int desc, cli_ctx *ctx)
 
     if(ret == CL_VIRUS) {
 	close(fd);
-	if(!cli_leavetemps_flag)
-	    if (cli_unlink(tmpname)) ret = CL_EIO;
+	if(!ctx->engine->keeptmp)
+	    if (cli_unlink(tmpname)) ret = CL_EUNLINK;
 	free(tmpname);	
 	fclose(fs);
 	return ret;
@@ -654,8 +650,8 @@ static int cli_scanbzip(int desc, cli_ctx *ctx)
 	cli_dbgmsg("Bzip: Infected with %s\n", *ctx->virname);
     }
     close(fd);
-    if(!cli_leavetemps_flag)
-	if (cli_unlink(tmpname)) ret = CL_EIO;
+    if(!ctx->engine->keeptmp)
+	if (cli_unlink(tmpname)) ret = CL_EUNLINK;
     free(tmpname);	
     fclose(fs);
 
@@ -671,7 +667,7 @@ static int cli_scanszdd(int desc, cli_ctx *ctx)
 
     cli_dbgmsg("in cli_scanszdd()\n");
 
-    if((ret = cli_gentempfd(NULL, &tmpname, &ofd))) {
+    if((ret = cli_gentempfd(ctx->engine->tmpdir, &tmpname, &ofd))) {
 	cli_dbgmsg("MSEXPAND: Can't generate temporary file/descriptor\n");
 	return ret;
     }
@@ -681,8 +677,8 @@ static int cli_scanszdd(int desc, cli_ctx *ctx)
 
     if(ret != CL_SUCCESS) { /* CL_VIRUS or some error */
 	close(ofd);
-	if(!cli_leavetemps_flag)
-	    if (cli_unlink(tmpname)) ret = CL_EIO;
+	if(!ctx->engine->keeptmp)
+	    if (cli_unlink(tmpname)) ret = CL_EUNLINK;
 	free(tmpname);	
 	return ret;
     }
@@ -691,8 +687,8 @@ static int cli_scanszdd(int desc, cli_ctx *ctx)
     lseek(ofd, 0, SEEK_SET);
     ret = cli_magic_scandesc(ofd, ctx);
     close(ofd);
-    if(!cli_leavetemps_flag)
-	if (cli_unlink(tmpname)) ret = CL_EIO;
+    if(!ctx->engine->keeptmp)
+	if (cli_unlink(tmpname)) ret = CL_EUNLINK;
     free(tmpname);	
 
     return ret;
@@ -718,7 +714,7 @@ static int cli_scanmscab(int desc, cli_ctx *ctx, off_t sfx_offset)
 	if(cli_checklimits("CAB", ctx, file->length, 0, 0)!=CL_CLEAN)
 	    continue;
 
-	if(!(tempname = cli_gentemp(NULL))) {
+	if(!(tempname = cli_gentemp(ctx->engine->tmpdir))) {
 	    ret = CL_EMEM;
 	    break;
 	}
@@ -728,10 +724,10 @@ static int cli_scanmscab(int desc, cli_ctx *ctx, off_t sfx_offset)
 	else
 	    ret = cli_scanfile(tempname, ctx);
 
-	if(!cli_leavetemps_flag) {
+	if(!ctx->engine->keeptmp) {
 	    if (cli_unlink(tempname)) {
 	    	free(tempname);
-		ret = CL_EIO;
+		ret = CL_EUNLINK;
 		break;
 	    }
 	}
@@ -784,7 +780,7 @@ static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 		    /* cli_dbgmsg("Project content:\n%s", data); */
 		    if(ctx->scanned)
 			*ctx->scanned += data_len / CL_COUNT_PRECISION;
-		    if(cli_scanbuff(data, data_len, ctx, CL_TYPE_MSOLE2) == CL_VIRUS) {
+		    if(cli_scanbuff(data, data_len, 0, ctx, CL_TYPE_MSOLE2, NULL) == CL_VIRUS) {
 			free(data);
 			ret = CL_VIRUS;
 			break;
@@ -808,11 +804,11 @@ static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 	    vbaname[sizeof(vbaname)-1] = '\0';
 	    fd = open(vbaname, O_RDONLY|O_BINARY);
 	    if (fd == -1) continue;
-	    if ((fullname = cli_ppt_vba_read(fd))) {
+	    if ((fullname = cli_ppt_vba_read(fd, ctx))) {
 		if(cli_scandir(fullname, ctx, 0) == CL_VIRUS) {
 		    ret = CL_VIRUS;
 		}
-		if(!cli_leavetemps_flag)
+		if(!ctx->engine->keeptmp)
 		    cli_rmdirs(fullname);
 		free(fullname);
 	    }
@@ -842,7 +838,7 @@ static int cli_vba_scandir(const char *dirname, cli_ctx *ctx, struct uniq *U)
 			cli_dbgmsg("Project content:\n%s", data);
 			if(ctx->scanned)
 			    *ctx->scanned += vba_project->length[i] / CL_COUNT_PRECISION;
-			if(cli_scanbuff(data, vba_project->length[i], ctx, CL_TYPE_MSOLE2) == CL_VIRUS) {
+			if(cli_scanbuff(data, vba_project->length[i], 0, ctx, CL_TYPE_MSOLE2, NULL) == CL_VIRUS) {
 				free(data);
 				ret = CL_VIRUS;
 				break;
@@ -939,7 +935,7 @@ static int cli_scanhtml(int desc, cli_ctx *ctx)
 
     if(fstat(desc, &sb) == -1) {
         cli_errmsg("cli_scanhtml: fstat() failed for descriptor %d\n", desc);
-	return CL_EIO;
+	return CL_ESTAT;
     }
 
     /* Because HTML detection is FP-prone and html_normalise_fd() needs to
@@ -951,7 +947,7 @@ static int cli_scanhtml(int desc, cli_ctx *ctx)
 	return CL_CLEAN;
     }
 
-    if(!(tempname = cli_gentemp(NULL)))
+    if(!(tempname = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(tempname, 0700)) {
@@ -999,7 +995,7 @@ static int cli_scanhtml(int desc, cli_ctx *ctx)
 	ret = cli_scandir(fullname, ctx, 0);
     }
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
         cli_rmdirs(tempname);
 
     free(tempname);
@@ -1015,26 +1011,29 @@ static int cli_scanscript(int desc, cli_ctx *ctx)
 	char *tmpname = NULL;
 	int ofd = -1, ret;
 	ssize_t nread;
-	const struct cli_matcher *root = ctx->engine->root[7];
-	uint32_t maxpatlen = root ? root->maxpatlen : 0;
+	const struct cli_matcher *troot = ctx->engine->root[7];
+	uint32_t maxpatlen = troot ? troot->maxpatlen : 0, offset = 0;
+	struct cli_matcher *groot = ctx->engine->root[0];
+	struct cli_ac_data gmdata, tmdata;
+	struct cli_ac_data *mdata[2];
 
 	cli_dbgmsg("in cli_scanscript()\n");
 
 	if(fstat(desc, &sb) == -1) {
 		cli_errmsg("cli_scanscript: fstat() failed for descriptor %d\n", desc);
-		return CL_EIO;
+		return CL_ESTAT;
 	}
 
 	/* don't normalize files that are too large */
-	if(sb.st_size > 524288) {
-		cli_dbgmsg("cli_scanscript: exiting (file larger than 400 kB)\n");
+	if(sb.st_size > 5242880) {
+		cli_dbgmsg("cli_scanscript: exiting (file larger than 5 MB)\n");
 		return CL_CLEAN;
 	}
 
 	/* dump to disk only if explicitly asked to,
 	 * otherwise we can process just in-memory */
-	if(cli_leavetemps_flag) {
-		if((ret = cli_gentempfd(NULL, &tmpname, &ofd))) {
+	if(ctx->engine->keeptmp) {
+		if((ret = cli_gentempfd(ctx->engine->tmpdir, &tmpname, &ofd))) {
 			cli_dbgmsg("cli_scanscript: Can't generate temporary file/descriptor\n");
 			return ret;
 		}
@@ -1048,6 +1047,16 @@ static int cli_scanscript(int desc, cli_ctx *ctx)
 	text_normalize_init(&state, normalized, SCANBUFF + maxpatlen);
 	ret = CL_CLEAN;
 
+	if ((ret = cli_ac_initdata(&tmdata, troot->ac_partsigs, troot->ac_lsigs, CLI_DEFAULT_AC_TRACKLEN)))
+	    return ret;
+
+	if ((ret = cli_ac_initdata(&gmdata, groot->ac_partsigs, groot->ac_lsigs, CLI_DEFAULT_AC_TRACKLEN))) {
+	    cli_ac_freedata(&tmdata);
+	    return ret;
+	}
+	mdata[0] = &tmdata;
+	mdata[1] = &gmdata;
+
 	do {
 		nread = cli_readn(desc, buff, sizeof(buff));
 		if(nread <= 0 || state.out_pos + nread > state.out_len) {
@@ -1059,13 +1068,16 @@ static int cli_scanscript(int desc, cli_ctx *ctx)
 				/* we can continue to scan in memory */
 			}
 			/* when we flush the buffer also scan */
-			if(cli_scanbuff(state.out, state.out_pos, ctx, CL_TYPE_TEXT_ASCII) == CL_VIRUS) {
+			if(cli_scanbuff(state.out, state.out_pos, offset, ctx, CL_TYPE_TEXT_ASCII, mdata) == CL_VIRUS) {
 				ret = CL_VIRUS;
 				break;
 			}
+			if(ctx->scanned)
+			    *ctx->scanned += state.out_pos / CL_COUNT_PRECISION;
+			offset += state.out_pos;
 			/* carry over maxpatlen from previous buffer */
 			if (state.out_pos > maxpatlen)
-				memcpy(state.out, state.out + state.out_pos - maxpatlen, maxpatlen); 
+				memmove(state.out, state.out + state.out_pos - maxpatlen, maxpatlen); 
 			text_normalize_reset(&state);
 			state.out_pos = maxpatlen;
 		}
@@ -1076,7 +1088,9 @@ static int cli_scanscript(int desc, cli_ctx *ctx)
 		 * and using while(){} loop would mean code duplication */
 	} while (nread > 0);
 
-	if(cli_leavetemps_flag) {
+	cli_ac_freedata(&tmdata);
+	cli_ac_freedata(&gmdata);
+	if(ctx->engine->keeptmp) {
 		free(tmpname);
 		close(ofd);
 	}
@@ -1093,13 +1107,13 @@ static int cli_scanhtml_utf16(int desc, cli_ctx *ctx)
 
     cli_dbgmsg("in cli_scanhtml_utf16()\n");
 
-    if(!(tempname = cli_gentemp(NULL)))
+    if(!(tempname = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if((fd = open(tempname, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) {
 	cli_errmsg("cli_scanhtml_utf16: Can't create file %s\n", tempname);
 	free(tempname);
-	return CL_EIO;
+	return CL_EOPEN;
     }
 
     cli_dbgmsg("cli_scanhtml_utf16: using tempfile %s\n", tempname);
@@ -1113,7 +1127,7 @@ static int cli_scanhtml_utf16(int desc, cli_ctx *ctx)
 		cli_unlink(tempname);
 		free(tempname);
 		close(fd);
-		return CL_EIO;
+		return CL_EWRITE;
 	    }
 	    free(decoded);
 	}
@@ -1123,8 +1137,8 @@ static int cli_scanhtml_utf16(int desc, cli_ctx *ctx)
     ret = cli_scanhtml(fd, ctx);
     close(fd);
 
-    if(!cli_leavetemps_flag) {
-	if (cli_unlink(tempname)) ret = CL_EIO;
+    if(!ctx->engine->keeptmp) {
+	if (cli_unlink(tempname)) ret = CL_EUNLINK;
     } else
 	cli_dbgmsg("cli_scanhtml_utf16: Decoded HTML data saved in %s\n", tempname);
     free(tempname);
@@ -1140,11 +1154,11 @@ static int cli_scanole2(int desc, cli_ctx *ctx)
 
     cli_dbgmsg("in cli_scanole2()\n");
 
-    if(ctx->limits && ctx->limits->maxreclevel && ctx->recursion >= ctx->limits->maxreclevel)
+    if(ctx->engine->maxreclevel && ctx->recursion >= ctx->engine->maxreclevel)
         return CL_EMAXREC;
 
     /* generate the temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -1156,7 +1170,7 @@ static int cli_scanole2(int desc, cli_ctx *ctx)
     ret = cli_ole2_extract(desc, dir, ctx, &vba);
     if(ret!=CL_CLEAN && ret!=CL_VIRUS) {
 	cli_dbgmsg("OLE2: %s\n", cl_strerror(ret));
-	if(!cli_leavetemps_flag)
+	if(!ctx->engine->keeptmp)
 	    cli_rmdirs(dir);
 	free(dir);
 	return ret;
@@ -1173,7 +1187,7 @@ static int cli_scanole2(int desc, cli_ctx *ctx)
 	ctx->recursion--;
     }
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
     free(dir);
     return ret;
@@ -1188,7 +1202,7 @@ static int cli_scantar(int desc, cli_ctx *ctx, unsigned int posix)
     cli_dbgmsg("in cli_scantar()\n");
 
     /* generate temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -1199,7 +1213,7 @@ static int cli_scantar(int desc, cli_ctx *ctx, unsigned int posix)
 
     ret = cli_untar(dir, desc, posix, ctx);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1215,7 +1229,7 @@ static int cli_scanbinhex(int desc, cli_ctx *ctx)
     cli_dbgmsg("in cli_scanbinhex()\n");
 
     /* generate temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -1229,7 +1243,7 @@ static int cli_scanbinhex(int desc, cli_ctx *ctx)
     else
 	ret = cli_scandir(dir, ctx, 0);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1245,7 +1259,7 @@ static int cli_scanmschm(int desc, cli_ctx *ctx)
     cli_dbgmsg("in cli_scanmschm()\n");
 
      /* generate the temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -1256,7 +1270,7 @@ static int cli_scanmschm(int desc, cli_ctx *ctx)
 
     ret = cli_chm_open(desc, dir, &metadata);
     if (ret != CL_SUCCESS) {
-	if(!cli_leavetemps_flag)
+	if(!ctx->engine->keeptmp)
 	    cli_rmdirs(dir);
 	free(dir);
 	cli_dbgmsg("CHM: Error: %s\n", cl_strerror(ret));
@@ -1284,7 +1298,7 @@ static int cli_scanmschm(int desc, cli_ctx *ctx)
 
     cli_chm_close(&metadata);
    
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1303,7 +1317,7 @@ static int cli_scanscrenc(int desc, cli_ctx *ctx)
 
     cli_dbgmsg("in cli_scanscrenc()\n");
 
-    if(!(tempname = cli_gentemp(NULL)))
+    if(!(tempname = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(tempname, 0700)) {
@@ -1315,7 +1329,7 @@ static int cli_scanscrenc(int desc, cli_ctx *ctx)
     if (html_screnc_decode(desc, tempname))
 	ret = cli_scandir(tempname, ctx, 0);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(tempname);
 
     free(tempname);
@@ -1357,7 +1371,7 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 
     if(fstat(desc, &sb) == -1) {
 	cli_errmsg("CryptFF: Can't fstat descriptor %d\n", desc);
-	return CL_EIO;
+	return CL_ESTAT;
     }
 
     /* Skip the CryptFF file header */
@@ -1383,7 +1397,7 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 	cli_dbgmsg("CryptFF: Can't read from descriptor %d\n", desc);
 	free(dest);
 	free(src);
-	return CL_EIO;
+	return CL_EREAD;
     }
 
     for(i = 0; i < length; i++)
@@ -1391,7 +1405,7 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 
     free(src);
 
-    if(!(tempfile = cli_gentemp(NULL))) {
+    if(!(tempfile = cli_gentemp(ctx->engine->tmpdir))) {
 	free(dest);
 	return CL_EMEM;
     }
@@ -1400,7 +1414,7 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 	cli_errmsg("CryptFF: Can't create file %s\n", tempfile);
 	free(dest);
 	free(tempfile);
-	return CL_EIO;
+	return CL_ECREAT;
     }
 
     if(write(ndesc, dest, length) == -1) {
@@ -1408,7 +1422,7 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 	free(dest);
 	close(ndesc);
 	free(tempfile);
-	return CL_EIO;
+	return CL_EWRITE;
     }
 
     free(dest);
@@ -1422,10 +1436,10 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 
     close(ndesc);
 
-    if(cli_leavetemps_flag)
+    if(ctx->engine->keeptmp)
 	cli_dbgmsg("CryptFF: Decompressed data saved in %s\n", tempfile);
     else
-	if (cli_unlink(tempfile)) ret = CL_EIO;
+	if (cli_unlink(tempfile)) ret = CL_EUNLINK;
 
     free(tempfile);
     return ret;
@@ -1434,7 +1448,7 @@ static int cli_scancryptff(int desc, cli_ctx *ctx)
 static int cli_scanpdf(int desc, cli_ctx *ctx, off_t offset)
 {
 	int ret;
-	char *dir = cli_gentemp(NULL);
+	char *dir = cli_gentemp(ctx->engine->tmpdir);
 
     if(!dir)
 	return CL_EMEM;
@@ -1447,7 +1461,7 @@ static int cli_scanpdf(int desc, cli_ctx *ctx, off_t offset)
 
     ret = cli_pdf(dir, desc, ctx, offset);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1457,7 +1471,7 @@ static int cli_scanpdf(int desc, cli_ctx *ctx, off_t offset)
 static int cli_scantnef(int desc, cli_ctx *ctx)
 {
 	int ret;
-	char *dir = cli_gentemp(NULL);
+	char *dir = cli_gentemp(ctx->engine->tmpdir);
 
     if(!dir)
 	return CL_EMEM;
@@ -1468,12 +1482,12 @@ static int cli_scantnef(int desc, cli_ctx *ctx)
 	return CL_ETMPDIR;
     }
 
-    ret = cli_tnef(dir, desc);
+    ret = cli_tnef(dir, desc, ctx);
 
     if(ret == CL_CLEAN)
 	ret = cli_scandir(dir, ctx, 0);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1483,7 +1497,7 @@ static int cli_scantnef(int desc, cli_ctx *ctx)
 static int cli_scanuuencoded(int desc, cli_ctx *ctx)
 {
 	int ret;
-	char *dir = cli_gentemp(NULL);
+	char *dir = cli_gentemp(ctx->engine->tmpdir);
 
     if(!dir)
 	return CL_EMEM;
@@ -1499,7 +1513,7 @@ static int cli_scanuuencoded(int desc, cli_ctx *ctx)
     if(ret == CL_CLEAN)
 	ret = cli_scandir(dir, ctx, 0);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1515,7 +1529,7 @@ static int cli_scanmail(int desc, cli_ctx *ctx)
     cli_dbgmsg("Starting cli_scanmail(), recursion = %u\n", ctx->recursion);
 
     /* generate the temporary directory */
-    if(!(dir = cli_gentemp(NULL)))
+    if(!(dir = cli_gentemp(ctx->engine->tmpdir)))
 	return CL_EMEM;
 
     if(mkdir(dir, 0700)) {
@@ -1528,7 +1542,7 @@ static int cli_scanmail(int desc, cli_ctx *ctx)
      * Extract the attachments into the temporary directory
      */
     if((ret = cli_mbox(dir, desc, ctx))) {
-	if(!cli_leavetemps_flag)
+	if(!ctx->engine->keeptmp)
 	    cli_rmdirs(dir);
 	free(dir);
 	return ret;
@@ -1536,7 +1550,7 @@ static int cli_scanmail(int desc, cli_ctx *ctx)
 
     ret = cli_scandir(dir, ctx, CL_TYPE_MAIL);
 
-    if(!cli_leavetemps_flag)
+    if(!ctx->engine->keeptmp)
 	cli_rmdirs(dir);
 
     free(dir);
@@ -1550,17 +1564,14 @@ static int cli_scan_structured(int desc, cli_ctx *ctx)
 	unsigned int cc_count = 0;
 	unsigned int ssn_count = 0;
 	int done = 0;
-	const struct cl_limits *lim = NULL;
 	int (*ccfunc)(const unsigned char *buffer, int length);
 	int (*ssnfunc)(const unsigned char *buffer, int length);
 
 
-    if(ctx == NULL || ctx->limits == NULL)
+    if(ctx == NULL)
 	return CL_ENULLARG;
 
-    lim = ctx->limits;
-
-    if(lim->min_cc_count == 1)
+    if(ctx->engine->min_cc_count == 1)
 	ccfunc = dlp_has_cc;
     else
 	ccfunc = dlp_get_cc_count;
@@ -1568,21 +1579,21 @@ static int cli_scan_structured(int desc, cli_ctx *ctx)
     switch((ctx->options & CL_SCAN_STRUCTURED_SSN_NORMAL) | (ctx->options & CL_SCAN_STRUCTURED_SSN_STRIPPED)) {
 
 	case (CL_SCAN_STRUCTURED_SSN_NORMAL | CL_SCAN_STRUCTURED_SSN_STRIPPED):
-	    if(lim->min_ssn_count == 1)
+	    if(ctx->engine->min_ssn_count == 1)
 		ssnfunc = dlp_has_ssn;
 	    else
 		ssnfunc = dlp_get_ssn_count;
 	    break;
 
 	case CL_SCAN_STRUCTURED_SSN_NORMAL:
-	    if(lim->min_ssn_count == 1)
+	    if(ctx->engine->min_ssn_count == 1)
 		ssnfunc = dlp_has_normal_ssn;
 	    else
 		ssnfunc = dlp_get_normal_ssn_count;
 	    break;
 
 	case CL_SCAN_STRUCTURED_SSN_STRIPPED:
-	    if(lim->min_ssn_count == 1)
+	    if(ctx->engine->min_ssn_count == 1)
 		ssnfunc = dlp_has_stripped_ssn;
 	    else
 		ssnfunc = dlp_get_stripped_ssn_count;
@@ -1593,20 +1604,20 @@ static int cli_scan_structured(int desc, cli_ctx *ctx)
     }
 
     while(!done && ((result = cli_readn(desc, buf, 8191)) > 0)) {
-	if((cc_count += ccfunc((const unsigned char *)buf, result)) >= lim->min_cc_count)
+	if((cc_count += ccfunc((const unsigned char *)buf, result)) >= ctx->engine->min_cc_count)
 	    done = 1;
 
-	if(ssnfunc && ((ssn_count += ssnfunc((const unsigned char *)buf, result)) >= lim->min_ssn_count))
+	if(ssnfunc && ((ssn_count += ssnfunc((const unsigned char *)buf, result)) >= ctx->engine->min_ssn_count))
 	    done = 1;
     }
 
-    if(cc_count != 0 && cc_count >= lim->min_cc_count) {
+    if(cc_count != 0 && cc_count >= ctx->engine->min_cc_count) {
 	cli_dbgmsg("cli_scan_structured: %u credit card numbers detected\n", cc_count);
 	*ctx->virname = "Structured.CreditCardNumber";
 	return CL_VIRUS;
     }
 
-    if(ssn_count != 0 && ssn_count >= lim->min_ssn_count) {
+    if(ssn_count != 0 && ssn_count >= ctx->engine->min_ssn_count) {
 	cli_dbgmsg("cli_scan_structured: %u social security numbers detected\n", ssn_count);
 	*ctx->virname = "Structured.SSN";
 	return CL_VIRUS;
@@ -1623,14 +1634,14 @@ static int cli_scanembpe(int desc, cli_ctx *ctx)
 	char *tmpname;
 
 
-    tmpname = cli_gentemp(NULL);
+    tmpname = cli_gentemp(ctx->engine->tmpdir);
     if(!tmpname)
 	return CL_EMEM;
 
     if((fd = open(tmpname, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) {
 	cli_errmsg("cli_scanembpe: Can't create file %s\n", tmpname);
 	free(tmpname);
-	return CL_EIO;
+	return CL_ECREAT;
     }
 
     while((bytes = read(desc, buff, sizeof(buff))) > 0) {
@@ -1642,14 +1653,14 @@ static int cli_scanembpe(int desc, cli_ctx *ctx)
 	if(cli_writen(fd, buff, bytes) != bytes) {
 	    cli_dbgmsg("cli_scanembpe: Can't write to temporary file\n");
 	    close(fd);
-	    if(!cli_leavetemps_flag) {
+	    if(!ctx->engine->keeptmp) {
 		if (cli_unlink(tmpname)) {
 		    free(tmpname);
-		    return CL_EIO;
+		    return CL_EUNLINK;
 		}
 	    }
 	    free(tmpname);	
-	    return CL_EIO;
+	    return CL_EWRITE;
 	}
     }
 
@@ -1658,10 +1669,10 @@ static int cli_scanembpe(int desc, cli_ctx *ctx)
     if((ret = cli_magic_scandesc(fd, ctx)) == CL_VIRUS) {
 	cli_dbgmsg("cli_scanembpe: Infected with %s\n", *ctx->virname);
 	close(fd);
-	if(!cli_leavetemps_flag) {
+	if(!ctx->engine->keeptmp) {
 	    if (cli_unlink(tmpname)) {
 	    	free(tmpname);
-		return CL_EIO;
+		return CL_EUNLINK;
 	    }
 	}
 	free(tmpname);	
@@ -1670,10 +1681,10 @@ static int cli_scanembpe(int desc, cli_ctx *ctx)
     ctx->recursion--;
 
     close(fd);
-    if(!cli_leavetemps_flag) {
+    if(!ctx->engine->keeptmp) {
 	if (cli_unlink(tmpname)) {
 	    free(tmpname);
-	    return CL_EIO;
+	    return CL_EUNLINK;
 	}
     }
     free(tmpname);
@@ -1703,7 +1714,7 @@ static int cli_scanraw(int desc, cli_ctx *ctx, cli_file_t type, uint8_t typercg,
 
     if(lseek(desc, 0, SEEK_SET) < 0) {
 	cli_errmsg("cli_scanraw: lseek() failed\n");
-	return CL_EIO;
+	return CL_ESEEK;
     }
 
     ret = cli_scandesc(desc, ctx, type == CL_TYPE_TEXT_ASCII ? 0 : type, 0, &ftoffset, acmode);
@@ -1726,12 +1737,10 @@ static int cli_scanraw(int desc, cli_ctx *ctx, cli_file_t type, uint8_t typercg,
 	    while(fpt) {
 		switch(fpt->type) {
 		    case CL_TYPE_RARSFX:
-#ifdef ENABLE_UNRAR
-			if(SCAN_ARCHIVE && type == CL_TYPE_MSEXE && (DCONF_ARCH & ARCH_CONF_RAR)) {
+			if(have_rar && SCAN_ARCHIVE && type == CL_TYPE_MSEXE && (DCONF_ARCH & ARCH_CONF_RAR)) {
 			    cli_dbgmsg("RAR-SFX signature found at %u\n", (unsigned int) fpt->offset);
 			    nret = cli_scanrar(desc, ctx, fpt->offset, &lastrar);
 			}
-#endif
 			break;
 
 		    case CL_TYPE_ZIPSFX:
@@ -1852,7 +1861,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 
     if(fstat(desc, &sb) == -1) {
 	cli_errmsg("magic_scandesc: Can't fstat descriptor %d\n", desc);
-	return CL_EIO;
+	return CL_ESTAT;
     }
 
     if(sb.st_size <= 5) {
@@ -1862,6 +1871,11 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 
     if(!ctx->engine) {
 	cli_errmsg("CRITICAL: engine == NULL\n");
+	return CL_ENULLARG;
+    }
+
+    if(!(ctx->engine->dboptions & CL_DB_COMPILED)) {
+	cli_errmsg("CRITICAL: engine not compiled\n");
 	return CL_EMALFDB;
     }
 
@@ -1875,7 +1889,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
     if(cli_updatelimits(ctx, sb.st_size)!=CL_CLEAN)
         return CL_CLEAN;
 
-    if((SCAN_MAIL || SCAN_ARCHIVE) && ctx->limits && ctx->limits->maxreclevel && ctx->recursion > ctx->limits->maxreclevel) {
+    if((SCAN_MAIL || SCAN_ARCHIVE) && ctx->engine->maxreclevel && ctx->recursion > ctx->engine->maxreclevel) {
         cli_dbgmsg("Archive recursion limit exceeded (level = %u).\n", ctx->recursion);
 	return CL_CLEAN;
     }
@@ -1884,7 +1898,7 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
     type = cli_filetype2(desc, ctx->engine);
     if(type == CL_TYPE_ERROR) {
 	cli_dbgmsg("cli_magic_scandesc: cli_filetype2 returned CL_TYPE_ERROR\n");
-	return CL_EIO;
+	return CL_EREAD;
     }
     lseek(desc, 0, SEEK_SET);
 
@@ -1901,12 +1915,8 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
 	    break;
 
 	case CL_TYPE_RAR:
-#ifdef ENABLE_UNRAR
-	    if(SCAN_ARCHIVE && (DCONF_ARCH & ARCH_CONF_RAR))
+	    if(have_rar && SCAN_ARCHIVE && (DCONF_ARCH & ARCH_CONF_RAR))
 		ret = cli_scanrar(desc, ctx, 0, NULL);
-#else
-	    cli_warnmsg("RAR code not compiled-in\n");
-#endif
 	    break;
 
 	case CL_TYPE_ZIP:
@@ -2112,25 +2122,18 @@ int cli_magic_scandesc(int desc, cli_ctx *ctx)
     }
 }
 
-int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, const struct cl_limits *limits, unsigned int options)
+int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions)
 {
     cli_ctx ctx;
-    struct cl_limits l_limits;
     int rc;
 
-    if(!limits) {
-	cli_errmsg("cl_scandesc: limits == NULL\n");
-	return CL_ENULLARG;
-    }
     memset(&ctx, '\0', sizeof(cli_ctx));
     ctx.engine = engine;
     ctx.virname = virname;
     ctx.scanned = scanned;
-    ctx.options = options;
+    ctx.options = scanoptions;
     ctx.found_possibly_unwanted = 0;
     ctx.dconf = (struct cli_dconf *) engine->dconf;
-    ctx.limits = &l_limits;
-    memcpy(&l_limits, limits, sizeof(struct cl_limits));
 
     rc = cli_magic_scandesc(desc, &ctx);
     if(rc == CL_CLEAN && ctx.found_possibly_unwanted)
@@ -2173,7 +2176,7 @@ static int cli_scanfile(const char *filename, cli_ctx *ctx)
     return ret;
 }
 
-int cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, const struct cl_limits *limits, unsigned int options)
+int cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions)
 {
 	int fd, ret;
 
@@ -2181,7 +2184,7 @@ int cl_scanfile(const char *filename, const char **virname, unsigned long int *s
     if((fd = open(filename, O_RDONLY|O_BINARY)) == -1)
 	return CL_EOPEN;
 
-    ret = cl_scandesc(fd, virname, scanned, engine, limits, options);
+    ret = cl_scandesc(fd, virname, scanned, engine, scanoptions);
     close(fd);
 
     return ret;
