@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2007-2008 Sourcefire, Inc.
  *
- *  Authors: Trog
+ *  Authors: Trog, Török Edvin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -34,7 +34,7 @@
 #include <netinet/in.h>
 #endif
 #include <string.h>
-
+#include <ctype.h>
 #include "clamav.h"
 #include "others.h"
 #include "cltypes.h"
@@ -85,7 +85,7 @@ int cli_check_mydoom_log(int desc, const char **virname)
     return retval;
 }
 
-static int jpeg_check_photoshop_8bim(int fd)
+static int jpeg_check_photoshop_8bim(int fd, cli_ctx *ctx)
 {
 	unsigned char bim[5];
 	uint16_t id, ntmp;
@@ -140,7 +140,7 @@ static int jpeg_check_photoshop_8bim(int fd)
 	/* Jump past header */
 	lseek(fd, 28, SEEK_CUR);
 
-	retval = cli_check_jpeg_exploit(fd);
+	retval = cli_check_jpeg_exploit(fd, ctx);
 	if (retval == 1) {
 		cli_dbgmsg("Exploit found in thumbnail\n");
 	}
@@ -149,7 +149,7 @@ static int jpeg_check_photoshop_8bim(int fd)
 	return retval;
 }
 
-static int jpeg_check_photoshop(int fd)
+static int jpeg_check_photoshop(int fd, cli_ctx *ctx)
 {
 	int retval;
 	unsigned char buffer[14];
@@ -166,7 +166,7 @@ static int jpeg_check_photoshop(int fd)
 	cli_dbgmsg("Found Photoshop segment\n");
 	do {
 		old = lseek(fd, 0, SEEK_CUR);
-		retval = jpeg_check_photoshop_8bim(fd);
+		retval = jpeg_check_photoshop_8bim(fd, ctx);
 		new = lseek(fd, 0, SEEK_CUR);
 		if(new <= old)
 			break;
@@ -178,7 +178,7 @@ static int jpeg_check_photoshop(int fd)
 	return retval;
 }
 
-int cli_check_jpeg_exploit(int fd)
+int cli_check_jpeg_exploit(int fd, cli_ctx *ctx)
 {
 	unsigned char buffer[4];
 	off_t offset;
@@ -186,6 +186,8 @@ int cli_check_jpeg_exploit(int fd)
 
 
 	cli_dbgmsg("in cli_check_jpeg_exploit()\n");
+	if(ctx->recursion > ctx->engine->maxreclevel)
+	    return CL_EMAXREC;
 
 	if (cli_readn(fd, buffer, 2) != 2) {
 		return 0;
@@ -222,16 +224,18 @@ int cli_check_jpeg_exploit(int fd)
 
 		offset = ((unsigned int) buffer[2] << 8) + buffer[3];
 		if (offset < 2) {
-			return 1;
+			return -1;
 		}
 		offset -= 2;
 		offset += lseek(fd, 0, SEEK_CUR);
 
 		if (buffer[1] == 0xed) {
 			/* Possible Photoshop file */
-			if ((retval=jpeg_check_photoshop(fd)) != 0) {
+			ctx->recursion++;
+			retval=jpeg_check_photoshop(fd, ctx);
+			ctx->recursion--;
+			if (retval != 0)
 				return retval;
-			}
 		}
 
 		if (lseek(fd, offset, SEEK_SET) != offset) {
@@ -351,3 +355,150 @@ int cli_check_riff_exploit(int fd)
 	}
 	return retval;
 }
+
+static inline int swizz_j48(const uint16_t n[])
+{
+	cli_dbgmsg("swizz_j48: %u, %u, %u\n",n[0],n[1],n[2]);
+	/* rules based on J48 tree */
+	if (n[0] <= 945 || !n[1])
+		return 0;
+	if (n[0] <= 1006)
+		return (n[2] > 0 && n[2] <= 6);
+	else
+		return n[1] <= 10;
+}
+
+void cli_detect_swizz_str(const unsigned char *str, uint32_t len, struct swizz_stats *stats, int blob)
+{
+	unsigned char stri[4096];
+        uint32_t i, j = 0;
+	int bad = 0;
+	int lastalnum = 0;
+	uint8_t ngrams[17576];
+	uint16_t all=0;
+	uint16_t ngram_cnts[3];
+	uint16_t words = 0;
+	int ret;
+
+	for(i=0;i<len-1 && j < sizeof(stri)-2;i += 2) {
+		unsigned char c = str[i];
+		if (str[i+1] || !c) {
+			bad++;
+			continue;
+		}
+		if (!isalnum(c)) {
+			if (!lastalnum)
+				continue;
+			lastalnum = 0;
+			c = ' ';
+		} else {
+			lastalnum = 1;
+			if (isdigit(c))
+				continue;
+		}
+		stri[j++] = tolower(c);
+	}
+	stri[j++] = '\0';
+	if ((!blob && (bad >= 8)) || j < 4)
+		return;
+	memset(ngrams, 0, sizeof(ngrams));
+	memset(ngram_cnts, 0, sizeof(ngram_cnts));
+	for(i=0;i<j-2;i++) {
+		if (stri[i] != ' ' && stri[i+1] != ' ' && stri[i+2] != ' ') {
+			uint16_t idx = (stri[i] - 'a')*676 + (stri[i+1] - 'a')*26 + (stri[i+2] - 'a');
+			if (idx < sizeof(ngrams)) {
+				ngrams[idx]++;
+				stats->gngrams[idx]++;
+			}
+		} else if (stri[i] == ' ')
+			words++;
+	}
+	for(i=0;i<sizeof(ngrams);i++) {
+		uint8_t v = ngrams[i];
+		if (v > 3) v = 3;
+		if (v) {
+			ngram_cnts[v-1]++;
+			all++;
+		}
+	}
+	if (!all)
+		return;
+	cli_dbgmsg("cli_detect_swizz_str: %u, %u, %u\n",ngram_cnts[0],ngram_cnts[1],ngram_cnts[2]);
+	/* normalize */
+	for(i=0;i<sizeof(ngram_cnts)/sizeof(ngram_cnts[0]);i++) {
+		uint32_t v = ngram_cnts[i];
+		ngram_cnts[i] = (v<<10)/all;
+	}
+	ret = swizz_j48(ngram_cnts) ? CL_VIRUS : CL_CLEAN;
+	cli_dbgmsg("cli_detect_swizz_str: %s, %u words\n", ret == CL_VIRUS ? "suspicious" : "ok", words);
+	if (ret == CL_VIRUS) {
+		stats->suspicious += j;
+		cli_dbgmsg("cli_detect_swizz_str: %s\n", stri);
+	}
+	stats->total += j;
+}
+
+static inline int swizz_j48_global(const uint32_t gn[])
+{
+	if (gn[0] <= 24185) {
+		return gn[0] > 22980 && gn[8] > 0 && gn[8] <= 97;
+	}
+	if (!gn[8]) {
+		if (gn[4] <= 311) {
+			if (!gn[4]) {
+				return gn[1] > 0 &&
+					((gn[0] <= 26579 && gn[3] > 0) ||
+					 (gn[0] > 28672 && gn[0] <= 30506));
+			}
+			if (gn[5] <= 616) {
+				if (gn[6] <= 104) {
+					return gn[9] <= 167;
+				}
+				return gn[6] <= 286;
+			}
+		}
+		return 0;
+	}
+	return 1;
+}
+
+int cli_detect_swizz(struct swizz_stats *stats)
+{
+	uint32_t gn[10];
+	uint32_t all = 0;
+	unsigned i;
+	int global_swizz = CL_CLEAN;
+
+	cli_dbgmsg("cli_detect_swizz: %lu/%lu, version:%d, manifest: %d \n",
+			(unsigned long)stats->suspicious, (unsigned long)stats->total,
+			stats->has_version, stats->has_manifest);
+	memset(gn, 0, sizeof(gn));
+	for(i=0;i<17576;i++) {
+		uint8_t v = stats->gngrams[i];
+		if (v > 10) v = 10;
+		if (v) {
+			gn[v-1]++;
+			all++;
+		}
+	}
+	if (all) {
+		/* normalize */
+		cli_dbgmsg("cli_detect_swizz: gn: ");
+		for(i=0;i<sizeof(gn)/sizeof(gn[0]);i++) {
+			uint32_t v = gn[i];
+			gn[i] = (v<<15)/all;
+			if (cli_debug_flag)
+			cli_dbgmsg("%lu, ", (unsigned long)gn[i]);
+		}
+		global_swizz = swizz_j48_global(gn) ? CL_VIRUS : CL_CLEAN;
+		cli_dbgmsg("\ncli_detect_swizz: global: %s\n", global_swizz ? "suspicious" : "clean");
+	}
+
+	if (stats->total <= 337)
+		return CL_CLEAN;
+	if (stats->suspicious<<10 > 20*stats->total)
+		return CL_VIRUS;
+  if (!stats->suspicious)
+    return CL_CLEAN;
+	return global_swizz;
+}
