@@ -86,12 +86,12 @@ static int unz(uint8_t *src, uint32_t csize, uint32_t usize, uint16_t method, ui
     snprintf(name, sizeof(name), "%s/zip.%03u", tmpd, *fu);
     name[sizeof(name)-1]='\0';
   } else {
-    if(!(tempfile = cli_gentemp(NULL))) return CL_EMEM;
+    if(!(tempfile = cli_gentemp(ctx->engine->tmpdir))) return CL_EMEM;
   }
   if((of = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRUSR|S_IWUSR))==-1) {
     cli_warnmsg("cli_unzip: failed to create temporary file %s\n", tempfile);
     if(!tmpd) free(tempfile);
-    return CL_EIO;
+    return CL_ECREAT;
   }
   switch (method) {
   case ALG_STORED:
@@ -105,11 +105,11 @@ static int unz(uint8_t *src, uint32_t csize, uint32_t usize, uint16_t method, ui
       else break;
     }
     if(res==1) {
-      if(ctx->limits && ctx->limits->maxfilesize && csize > ctx->limits->maxfilesize) {
-	cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->limits->maxfilesize);
-	csize = ctx->limits->maxfilesize;
+      if(ctx->engine->maxfilesize && csize > ctx->engine->maxfilesize) {
+	cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->engine->maxfilesize);
+	csize = ctx->engine->maxfilesize;
       }
-      if(cli_writen(of, src, csize)!=(int)csize) ret = CL_EIO;
+      if(cli_writen(of, src, csize)!=(int)csize) ret = CL_EWRITE;
       else res=0;
     }
     break;
@@ -166,15 +166,16 @@ static int unz(uint8_t *src, uint32_t csize, uint32_t usize, uint16_t method, ui
       while((res = unz_unz(&strm, Z_NO_FLUSH))==Z_OK) {};
       if(*avail_out!=sizeof(obuf)) {
 	written+=sizeof(obuf)-(*avail_out);
-	if(ctx->limits && ctx->limits->maxfilesize && written > ctx->limits->maxfilesize) {
-	  cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->limits->maxfilesize);
+	if(ctx->engine->maxfilesize && written > ctx->engine->maxfilesize) {
+	  cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->engine->maxfilesize);
 	  res = Z_STREAM_END;
 	  break;
 	}
 	if(cli_writen(of, obuf, sizeof(obuf)-(*avail_out)) != (int)(sizeof(obuf)-(*avail_out))) {
 	  cli_warnmsg("cli_unzip: falied to write %lu inflated bytes\n", sizeof(obuf)-(*avail_out));
-	  ret = CL_EIO;
-	  res=1;
+	  ret = CL_EWRITE;
+	  res = 100;
+	  break;
 	}
 	*next_out = obuf;
 	*avail_out = sizeof(obuf);
@@ -209,26 +210,25 @@ static int unz(uint8_t *src, uint32_t csize, uint32_t usize, uint16_t method, ui
     while((res = BZ2_bzDecompress(&strm))==BZ_OK || res==BZ_STREAM_END) {
       if(strm.avail_out!=sizeof(obuf)) {
 	written+=sizeof(obuf)-strm.avail_out;
-	if(ctx->limits && ctx->limits->maxfilesize && written > ctx->limits->maxfilesize) {
-	  cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->limits->maxfilesize);
+	if(ctx->engine->maxfilesize && written > ctx->engine->maxfilesize) {
+	  cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->engine->maxfilesize);
 	  res = BZ_STREAM_END;
 	  break;
 	}
 	if(cli_writen(of, obuf, sizeof(obuf)-strm.avail_out) != (int)(sizeof(obuf)-strm.avail_out)) {
 	  cli_warnmsg("cli_unzip: falied to write %lu bunzipped bytes\n", sizeof(obuf)-strm.avail_out);
-	  ret = CL_EIO;
-	  res=1;
+	  ret = CL_EWRITE;
+	  res = 100;
+	  break;
 	}
 	strm.next_out = obuf;
 	strm.avail_out = sizeof(obuf);
-	continue;
+	if (res == BZ_OK) continue; /* after returning BZ_STREAM_END once, decompress returns an error */
       }
       break;
     }
     BZ2_bzDecompressEnd(&strm);
-    if (res == BZ_STREAM_END) {
-      res=0;
-    }
+    if (res == BZ_STREAM_END) res=0;
     break;
   }
 #endif /* HAVE_BZLIB_H */
@@ -247,15 +247,16 @@ static int unz(uint8_t *src, uint32_t csize, uint32_t usize, uint16_t method, ui
     while((res = explode(&strm))==EXPLODE_OK) {
       if(strm.avail_out!=sizeof(obuf)) {
 	written+=sizeof(obuf)-strm.avail_out;
-	if(ctx->limits && ctx->limits->maxfilesize && written > ctx->limits->maxfilesize) {
-	  cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->limits->maxfilesize);
+	if(ctx->engine->maxfilesize && written > ctx->engine->maxfilesize) {
+	  cli_dbgmsg("cli_unzip: trimming output size to maxfilesize (%lu)\n", ctx->engine->maxfilesize);
 	  res = 0;
 	  break;
 	}
 	if(cli_writen(of, obuf, sizeof(obuf)-strm.avail_out) != (int)(sizeof(obuf)-strm.avail_out)) {
 	  cli_warnmsg("cli_unzip: falied to write %lu exploded bytes\n", sizeof(obuf)-strm.avail_out);
-	  ret = CL_EIO;
-	  res=1;
+	  ret = CL_EWRITE;
+	  res = 100;
+	  break;
 	}
 	strm.next_out = (uint8_t *)obuf;
 	strm.avail_out = sizeof(obuf);
@@ -302,15 +303,15 @@ static int unz(uint8_t *src, uint32_t csize, uint32_t usize, uint16_t method, ui
     lseek(of, 0, SEEK_SET);
     ret = cli_magic_scandesc(of, ctx);
     close(of);
-    if(!cli_leavetemps_flag)
-      if(cli_unlink(tempfile)) ret = CL_EIO;
+    if(!ctx->engine->keeptmp)
+      if(cli_unlink(tempfile)) ret = CL_EUNLINK;
     if(!tmpd) free(tempfile);
     return ret;
   }
 
   close(of);
-  if(!cli_leavetemps_flag)
-    if(cli_unlink(tempfile)) ret = CL_EIO;
+  if(!ctx->engine->keeptmp)
+    if(cli_unlink(tempfile)) ret = CL_EUNLINK;
   if(!tmpd) free(tempfile);
   cli_dbgmsg("cli_unzip: extraction failed\n");
   return ret;
@@ -481,7 +482,7 @@ int cli_unzip(int f, cli_ctx *ctx) {
   cli_dbgmsg("in cli_unzip\n");
   if (fstat(f, &st)==-1) {
     cli_warnmsg("cli_unzip: fstat() failed\n");
-    return CL_EIO;
+    return CL_ESTAT;
   }
   fsize = (uint32_t)st.st_size;
   if(sizeof(off_t)!=sizeof(uint32_t) && (off_t)fsize!=st.st_size) {
@@ -496,11 +497,11 @@ int cli_unzip(int f, cli_ctx *ctx) {
 #if HAVE_MMAP
   if ((map = mmap(NULL, fsize, PROT_READ, MAP_PRIVATE, f, 0))==MAP_FAILED) {
     cli_dbgmsg("cli_unzip: mmap failed\n");
-    return CL_EMEM;
+    return CL_EMAP;
   }
 #else
   if(fsize > CLI_MAX_ALLOCATION) {
-    cli_warnmsg("cli_unzip: unzip support not compiled in and file is too big\n");
+    cli_warnmsg("cli_unzip: mmap not available and file is too big\n");
     return CL_CLEAN;
   }
   lseek(f, 0, SEEK_SET);
@@ -508,11 +509,11 @@ int cli_unzip(int f, cli_ctx *ctx) {
     return CL_EMEM;
   if(cli_readn(f, map, fsize)!=fsize) {
     free(map);
-    return CL_EIO;
+    return CL_EREAD;
   }
 #endif
 
-  if (!(tmpd = cli_gentemp(NULL))) {
+  if (!(tmpd = cli_gentemp(ctx->engine->tmpdir))) {
     destroy_map(map, fsize);
     return CL_ETMPDIR;
   }
@@ -536,8 +537,8 @@ int cli_unzip(int f, cli_ctx *ctx) {
     cli_dbgmsg("cli_unzip: central @%x\n", coff);
     while(ret==CL_CLEAN && (coff=chdr(map, coff, fsize, &fu, fc+1, &ret, ctx, tmpd))) {
       fc++;
-      if (ctx->limits && ctx->limits->maxfiles && fu>=ctx->limits->maxfiles) {
-	cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->limits->maxfiles);
+      if (ctx->engine->maxfiles && fu>=ctx->engine->maxfiles) {
+	cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
 	ret=CL_EMAXFILES;
       }
     }
@@ -547,15 +548,15 @@ int cli_unzip(int f, cli_ctx *ctx) {
     while (ret==CL_CLEAN && lhoff<fsize && (coff=lhdr(&map[lhoff], fsize-lhoff, &fu, fc+1, NULL, &ret, ctx, tmpd))) {
       fc++;
       lhoff+=coff;
-      if (ctx->limits && ctx->limits->maxfiles && fu>=ctx->limits->maxfiles) {
-	cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->limits->maxfiles);
+      if (ctx->engine->maxfiles && fu>=ctx->engine->maxfiles) {
+	cli_dbgmsg("cli_unzip: Files limit reached (max: %u)\n", ctx->engine->maxfiles);
 	ret=CL_EMAXFILES;
       }
     }
   }
 
   destroy_map(map, fsize);
-  if (!cli_leavetemps_flag) cli_rmdirs(tmpd);
+  if (!ctx->engine->keeptmp) cli_rmdirs(tmpd);
   free(tmpd);
 
   return ret;
@@ -571,7 +572,7 @@ int cli_unzip_single(int f, cli_ctx *ctx, off_t lhoffl) {
   cli_dbgmsg("in cli_unzip_single\n");
   if (fstat(f, &st)==-1) {
     cli_warnmsg("cli_unzip: fstat() failed\n");
-    return CL_EIO;
+    return CL_ESTAT;
   }
   fsize = (uint32_t)(st.st_size - lhoffl);
   if (lhoffl<0 || lhoffl>st.st_size || (sizeof(off_t)!=sizeof(uint32_t) && (off_t)fsize!=st.st_size - lhoffl)) {
@@ -586,11 +587,11 @@ int cli_unzip_single(int f, cli_ctx *ctx, off_t lhoffl) {
 #if HAVE_MMAP
   if ((map = mmap(NULL, st.st_size, PROT_READ, MAP_PRIVATE, f, 0))==MAP_FAILED) {
     cli_dbgmsg("cli_unzip: mmap() failed\n");
-    return CL_EMEM;
+    return CL_EMAP;
   }
 #else
   if(st.st_size > CLI_MAX_ALLOCATION) {
-    cli_warnmsg("cli_unzip: unzip support not compiled in and file is too big\n");
+    cli_warnmsg("cli_unzip: mmap not available and file is too big\n");
     return CL_CLEAN;
   }
   lseek(f, 0, SEEK_SET);
@@ -598,7 +599,7 @@ int cli_unzip_single(int f, cli_ctx *ctx, off_t lhoffl) {
     return CL_EMEM;
   if(cli_readn(f, map, st.st_size)!=st.st_size) {
     free(map);
-    return CL_EIO;
+    return CL_EREAD;
   }
 #endif
   lhdr(&map[lhoffl], fsize, &fu, 0, NULL, &ret, ctx, NULL);
