@@ -107,7 +107,7 @@ get_unicode_name(const char *name, int size, int big_endian)
 	if((name == NULL) || (*name == '\0') || (size <= 0))
 		return NULL;
 
-	newname = (char *)cli_malloc(size * 7);
+	newname = (char *)cli_malloc(size * 7 + 1);
 	if(newname == NULL)
 		return NULL;
 
@@ -561,7 +561,7 @@ cli_scan_ole10(int fd, cli_ctx *ctx)
 		return CL_CLEAN;
 
 	if(fstat(fd, &statbuf) == -1)
-		return CL_EIO;
+		return CL_ESTAT;
 
 	if ((statbuf.st_size - object_size) >= 4) {
 		/* Probably the OLE type id */
@@ -588,7 +588,7 @@ cli_scan_ole10(int fd, cli_ctx *ctx)
 		if(!read_uint32(fd, &object_size, FALSE))
 			return CL_CLEAN;
 	}
-	if(!(fullname = cli_gentemp(NULL))) {
+	if(!(fullname = cli_gentemp(ctx ? ctx->engine->tmpdir : NULL))) {
 		return CL_EMEM;
 	}
 	ofd = open(fullname, O_RDWR|O_CREAT|O_TRUNC|O_BINARY|O_EXCL,
@@ -596,16 +596,16 @@ cli_scan_ole10(int fd, cli_ctx *ctx)
 	if (ofd < 0) {
 		cli_warnmsg("cli_decode_ole_object: can't create %s\n",	fullname);
 		free(fullname);
-		return CL_EIO;
+		return CL_ECREAT;
 	}
 	cli_dbgmsg("cli_decode_ole_object: decoding to %s\n", fullname);
 	ole_copy_file_data(fd, ofd, object_size);
 	lseek(ofd, 0, SEEK_SET);
 	ret = cli_magic_scandesc(ofd, ctx);
 	close(ofd);
-	if(!cli_leavetemps_flag)
+	if(ctx && !ctx->engine->keeptmp)
 	  if (cli_unlink(fullname))
-	    ret = CL_EIO;
+	    ret = CL_EUNLINK;
 	free(fullname);
 	return ret;
 }
@@ -760,13 +760,13 @@ ppt_stream_iter(int fd, const char *dir)
 }
 
 char *
-cli_ppt_vba_read(int ifd)
+cli_ppt_vba_read(int ifd, cli_ctx *ctx)
 {
 	char *dir;
 	const char *ret;
 
 	/* Create a directory to store the extracted OLE2 objects */
-	dir = cli_gentemp(NULL);
+	dir = cli_gentemp(ctx ? ctx->engine->tmpdir : NULL);
 	if(dir == NULL)
 		return NULL;
 	if(mkdir(dir, 0700)) {
