@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2004 - 2007 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -29,6 +31,7 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
 #ifndef	C_WINDOWS
 #include <dirent.h>
 #endif
@@ -36,7 +39,7 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include "shared/cfgparser.h"
+#include "shared/optparser.h"
 #include "shared/output.h"
 
 #include "libclamav/clamav.h"
@@ -54,40 +57,34 @@
 #define REPO_VERSION "exported"
 #endif
 
-#ifdef CL_EXPERIMENTAL
-#define EXP_VER "-exp"
-#else
-#define EXP_VER
-#endif
-
 const char *get_version(void)
 {
 	if(!strncmp("devel-",VERSION,6) && strcmp("exported",REPO_VERSION)) {
-		return REPO_VERSION""EXP_VER;
+		return REPO_VERSION""VERSION_SUFFIX;
 	}
 	/* it is a release, or we have nothing better */
-	return VERSION""EXP_VER;
+	return VERSION""VERSION_SUFFIX;
 }
 /* CL_NOLIBCLAMAV means to omit functions that depends on libclamav */
 #ifndef CL_NOLIBCLAMAV
 char *freshdbdir(void)
 {
 	struct cl_cvd *d1, *d2;
-	struct cfgstruct *copt;
-	const struct cfgstruct *cpt;
+	struct optstruct *opts;
+	const struct optstruct *opt;
 	const char *dbdir;
 	char *retdir;
 
 
-    /* try to find fresh directory */
+    /* try to find the most up-to-date db directory */
     dbdir = cl_retdbdir();
-    if((copt = getcfg(CONFDIR"/freshclam.conf", 0))) {
-	if((cpt = cfgopt(copt, "DatabaseDirectory"))->enabled || (cpt = cfgopt(copt, "DataDirectory"))->enabled) {
-	    if(strcmp(dbdir, cpt->strarg)) {
-		    char *daily = (char *) malloc(strlen(cpt->strarg) + strlen(dbdir) + 30);
-		sprintf(daily, "%s/daily.cvd", cpt->strarg);
+    if((opts = optparse(CONFDIR"/freshclam.conf", 0, NULL, 0, OPT_FRESHCLAM, 0, NULL))) {
+	if((opt = optget(opts, "DatabaseDirectory"))->enabled) {
+	    if(strcmp(dbdir, opt->strarg)) {
+		    char *daily = (char *) malloc(strlen(opt->strarg) + strlen(dbdir) + 30);
+		sprintf(daily, "%s/daily.cvd", opt->strarg);
 		if(access(daily, R_OK))
-		    sprintf(daily, "%s/daily.cld", cpt->strarg);
+		    sprintf(daily, "%s/daily.cld", opt->strarg);
 
 		if(!access(daily, R_OK) && (d1 = cl_cvdhead(daily))) {
 		    sprintf(daily, "%s/daily.cvd", dbdir);
@@ -97,11 +94,11 @@ char *freshdbdir(void)
 		    if(!access(daily, R_OK) && (d2 = cl_cvdhead(daily))) {
 			free(daily);
 			if(d1->version > d2->version)
-			    dbdir = cpt->strarg;
+			    dbdir = opt->strarg;
 			cl_cvdfree(d2);
 		    } else {
 			free(daily);
-			dbdir = cpt->strarg;
+			dbdir = opt->strarg;
 		    }
 		    cl_cvdfree(d1);
 		} else {
@@ -113,13 +110,12 @@ char *freshdbdir(void)
 
     retdir = strdup(dbdir);
 
-    if(copt)
-	freecfg(copt);
+    if(opts)
+	optfree(opts);
 
     return retdir;
 }
 
-
 void print_version(const char *dbdir)
 {
 	char *fdbdir, *path;
@@ -162,9 +158,9 @@ void print_version(const char *dbdir)
     free(path);
 }
 #endif
+
 int filecopy(const char *src, const char *dest)
 {
-
 #ifdef C_DARWIN
 	pid_t pid;
 
@@ -183,110 +179,11 @@ int filecopy(const char *src, const char *dest)
 
     return -1;
 
-#else
-	char buffer[FILEBUFF];
-	int s, d, bytes, ret;
-	struct stat sb;
-
-
-    if((s = open(src, O_RDONLY|O_BINARY)) == -1)
-	return -1;
-
-    if((d = open(dest, O_CREAT|O_WRONLY|O_TRUNC|O_BINARY, 0644)) == -1) {
-	close(s);
-	return -1;
-    }
-
-    while((bytes = read(s, buffer, FILEBUFF)) > 0)
-	if(write(d, buffer, bytes) < bytes) {
-	    close(s);
-	    close(d);
-	    return -1;
-	}
-
-    close(s);
-    /* njh@bandsman.co.uk: check result of close for NFS file */
-    ret = close(d);
-
-    stat(src, &sb);
-    chmod(dest, sb.st_mode);
-
-    return ret;
-
+#else /* C_DARWIN */
+    return cli_filecopy(src, dest);
 #endif
-
 }
 
-#ifndef CL_NOLIBCLAMAV
-int dircopy(const char *src, const char *dest)
-{
-	DIR *dd;
-	struct dirent *dent;
-	struct stat sb;
-	char spath[512], dpath[512];
-
-
-    if(stat(dest, &sb) == -1) {
-	if(mkdir(dest, 0755)) {
-	    /* mprintf("!dircopy: Can't create temporary directory %s\n", dest); */
-	    return -1;
-	}
-    }
-
-    if((dd = opendir(src)) == NULL) {
-        /* mprintf("!dircopy: Can't open directory %s\n", src); */
-        return -1;
-    }
-
-    while((dent = readdir(dd))) {
-#if (!defined(C_INTERIX)) && (!defined(C_WINDOWS))
-	if(dent->d_ino)
-#endif
-	{
-	    if(!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
-		continue;
-
-	    snprintf(spath, sizeof(spath), "%s/%s", src, dent->d_name);
-	    snprintf(dpath, sizeof(dpath), "%s/%s", dest, dent->d_name);
-
-	    if(filecopy(spath, dpath) == -1) {
-		/* mprintf("!dircopy: Can't copy %s to %s\n", spath, dpath); */
-		cli_rmdirs(dest);
-		closedir(dd);
-		return -1;
-	    }
-	}
-    }
-
-    closedir(dd);
-    return 0;
-}
-#endif
-
-#ifndef CL_NOLIBCLAMAV
-int cvd_unpack(const char *cvd, const char *destdir)
-{
-	int fd;
-
-
-    if((fd = open(cvd, O_RDONLY|O_BINARY)) == -1)
-	return -1;
-
-    if(lseek(fd, 512, SEEK_SET) == -1) {
-	close(fd);
-	return -1;
-    }
-
-    if(cli_untgz(fd, destdir) == -1) {
-	close(fd);
-	return -1;
-    }
-    close(fd);
-
-    return 0;
-}
-#endif
-
 int daemonize(void)
 {
 #if defined(C_OS2) || defined(C_WINDOWS)
@@ -373,3 +270,25 @@ int match_regex(const char *filename, const char *pattern)
 	return match;
 }
 #endif
+
+int cfg_tcpsock(const struct optstruct *opts, struct sockaddr_in *tcpsock, in_addr_t defaultbind)
+{
+    struct hostent *he;
+    const struct optstruct *opt = optget(opts, "TCPSocket");
+
+    memset(tcpsock, 0, sizeof(*tcpsock));
+    tcpsock->sin_family = AF_INET;
+    tcpsock->sin_port = htons(opt->numarg);
+
+    if(!(opt = optget(opts, "TCPAddr"))->enabled) {
+	tcpsock->sin_addr.s_addr = htonl(defaultbind);
+	return 0;
+    }
+    he = gethostbyname(opt->strarg);
+    if(!he)
+	return -1;
+
+    tcpsock->sin_addr = *(struct in_addr *) he->h_addr_list[0];
+    return 0;
+}
+
