@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2004 - 2005 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -18,7 +20,9 @@
 
 #ifndef __MISC_H
 #define __MISC_H
-
+#include <netdb.h>
+#include <netinet/in.h>
+#include "optparser.h"
 /* Maximum filenames under various systems - njh */
 #ifndef	NAME_MAX	/* e.g. Linux */
 # ifdef	MAXNAMELEN	/* e.g. Solaris */
@@ -30,13 +34,21 @@
 # endif
 #endif
 
-#include "cfgparser.h"
+#ifndef HAVE_IN_ADDR_T
+typedef unsigned        int     in_addr_t;
+#endif
+
+#include <limits.h>
+
+#ifndef PATH_MAX
+#define PATH_MAX 1024
+#endif
 
 char *freshdbdir(void);
 void print_version(const char *dbdir);
 int filecopy(const char *src, const char *dest);
-int dircopy(const char *src, const char *dest);
-int cvd_unpack(const char *cvd, const char *destdir);
 int daemonize(void);
-
+const char *get_version(void);
+int match_regex(const char *filename, const char *pattern);
+int cfg_tcpsock(const struct optstruct *opts, struct sockaddr_in *server, in_addr_t defaultbind);
 #endif
