@@ -47,12 +47,11 @@
 
 #include "vba.h"
 
-#include "shared/options.h"
 #include "shared/output.h"
-#include "shared/cfgparser.h"
+#include "shared/optparser.h"
 #include "shared/misc.h"
 #include "shared/cdiff.h"
-#include "shared/sha256.h"
+#include "libclamav/sha256.h"
 #include "shared/tar.h"
 
 #include "libclamav/clamav.h"
@@ -61,9 +60,15 @@
 #include "libclamav/str.h"
 #include "libclamav/ole2_extract.h"
 #include "libclamav/htmlnorm.h"
+#include "libclamav/default.h"
 
 #define MAX_DEL_LOOKAHEAD   200
 
+/*
+ * Force backward compatibility with the cdiff interpreter of clamav < 0.95
+ */
+#define COMPATIBILITY_LIMIT 980
+
 static const struct dblist_s {
     const char *name;
     unsigned int count;
@@ -74,7 +79,7 @@ static const struct dblist_s {
     { "daily.cfg",  0 },
     { "daily.ign",  0 },
     { "daily.ftm",  0 },
-    { "main.info",  0 },    { "daily.info", 0 },
+    { "main.info",  0 },    { "daily.info", 0 },    { "safebrowsing.info", 0 },
 
     /* databases */
     { "main.db",    1 },    { "daily.db",   1 },
@@ -90,12 +95,26 @@ static const struct dblist_s {
     { "main.zmd",   1 },    { "daily.zmd",  1 },
     { "main.rmd",   1 },    { "daily.rmd",  1 },
     { "main.fp",    0 },    { "daily.fp",   0 },
-    { "main.pdb",   0 },    { "daily.pdb",  0 },
-    { "main.wdb",   0 },    { "daily.wdb",  0 },
+    { "main.pdb",   1 },    { "daily.pdb",  1 },    { "safebrowsing.gdb", 1 },
+    { "main.wdb",   0 },    { "daily.wdb",  0 },    { "safebrowsing.wdb", 0 },
 
     { NULL,	    0 }
 };
 
+static const char *getdbname(const char *str)
+{
+    if(strstr(str, "main"))
+	return "main";
+    else if(strstr(str, "daily"))
+	return "daily";
+    else if(strstr(str, "safebrowsing"))
+	return "safebrowsing";
+    else {
+	mprintf("!getdbname: Can't extract db name\n");
+	return "UNKNOWN";
+    }
+}
+
 static int hexdump(void)
 {
 	char buffer[FILEBUFF], *pt;
@@ -118,36 +137,33 @@ static int hexdump(void)
     return 0;
 }
 
-static int md5sig(struct optstruct *opt, unsigned int mdb)
+static int md5sig(const struct optstruct *opts, unsigned int mdb)
 {
-	char *md5, *filename;
+	char *md5;
 	unsigned int i;
 	struct stat sb;
 
 
-    if(opt->filename) {
-	for(i = 0; (filename = cli_strtok(opt->filename, i, "\t")); i++) {
-	    if(stat(filename, &sb) == -1) {
-		mprintf("!md5sig: Can't access file %s\n", filename);
+    if(opts->filename) {
+	for(i = 0; opts->filename[i]; i++) {
+	    if(stat(opts->filename[i], &sb) == -1) {
+		mprintf("!md5sig: Can't access file %s\n", opts->filename[i]);
 		perror("md5sig");
-		free(filename);
 		return -1;
 	    } else {
 		if((sb.st_mode & S_IFMT) == S_IFREG) {
-		    if((md5 = cli_md5file(filename))) {
+		    if((md5 = cli_md5file(opts->filename[i]))) {
 			if(mdb)
-			    mprintf("%u:%s:%s\n", (unsigned int) sb.st_size, md5, filename);
+			    mprintf("%u:%s:%s\n", (unsigned int) sb.st_size, md5, opts->filename[i]);
 			else
-			    mprintf("%s:%u:%s\n", md5, (unsigned int) sb.st_size, filename);
+			    mprintf("%s:%u:%s\n", md5, (unsigned int) sb.st_size, opts->filename[i]);
 			free(md5);
 		    } else {
-			mprintf("!md5sig: Can't generate MD5 checksum for %s\n", filename);
-			free(filename);
+			mprintf("!md5sig: Can't generate MD5 checksum for %s\n", opts->filename[i]);
 			return -1;
 		    }
 		}
 	    }
-	    free(filename);
 	}
 
     } else { /* stream */
@@ -163,13 +179,13 @@ static int md5sig(struct optstruct *opt, unsigned int mdb)
     return 0;
 }
 
-static int htmlnorm(struct optstruct *opt)
+static int htmlnorm(const struct optstruct *opts)
 {
 	int fd;
 
 
-    if((fd = open(opt_arg(opt, "html-normalise"), O_RDONLY)) == -1) {
-	mprintf("!htmlnorm: Can't open file %s\n", opt_arg(opt, "html-normalise"));
+    if((fd = open(optget(opts, "html-normalise")->strarg, O_RDONLY)) == -1) {
+	mprintf("!htmlnorm: Can't open file %s\n", optget(opts, "html-normalise")->strarg);
 	return -1;
     }
 
@@ -179,14 +195,14 @@ static int htmlnorm(struct optstruct *opt)
     return 0;
 }
 
-static int utf16decode(struct optstruct *opt)
+static int utf16decode(const struct optstruct *opts)
 {
 	const char *fname;
 	char *newname, buff[512], *decoded;
 	int fd1, fd2, bytes;
 
 
-    fname = opt_arg(opt, "utf16-decode");
+    fname = optget(opts, "utf16-decode")->strarg;
     if((fd1 = open(fname, O_RDONLY)) == -1) {
 	mprintf("!utf16decode: Can't open file %s\n", fname);
 	return -1;
@@ -297,11 +313,7 @@ static char *getdsig(const char *host, const char *user, const unsigned char *da
 	mprintf("\n");
     }
 
-#ifdef PF_INET
-    if((sockd = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
-#else
     if((sockd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-#endif
 	perror("socket()");
 	mprintf("!getdsig: Can't create socket\n");
 	memset(pass, 0, sizeof(pass));
@@ -415,7 +427,7 @@ static int writeinfo(const char *dbname, const char *header)
 static int diffdirs(const char *old, const char *new, const char *patch);
 static int verifydiff(const char *diff, const char *cvd, const char *incdir);
 
-static int script2cdiff(const char *script, const char *builder, struct optstruct *opt)
+static int script2cdiff(const char *script, const char *builder, const struct optstruct *opts)
 {
 	char *cdiff, *pt, buffer[FILEBUFF];
 	unsigned char digest[32];
@@ -509,10 +521,9 @@ static int script2cdiff(const char *script, const char *builder, struct optstruc
 	sha256_update(&ctx, (unsigned char *) buffer, bytes);
 
     fclose(cdiffh);
-    sha256_final(&ctx);
-    sha256_digest(&ctx, digest);
+    sha256_final(&ctx, digest);
 
-    if(!(pt = getdsig(opt_arg(opt, "server"), builder, digest, 32, 1))) {
+    if(!(pt = getdsig(optget(opts, "server")->strarg, builder, digest, 32, 1))) {
 	mprintf("!script2cdiff: Can't get digital signature from remote server\n");
 	unlink(cdiff);
 	free(cdiff);
@@ -535,7 +546,7 @@ static int script2cdiff(const char *script, const char *builder, struct optstruc
     return 0;
 }
 
-static int build(struct optstruct *opt)
+static int build(const struct optstruct *opts)
 {
 	int ret;
 	size_t bytes;
@@ -544,7 +555,7 @@ static int build(struct optstruct *opt)
 	unsigned char buffer[FILEBUFF];
 	char *tarfile, header[513], smbuff[32], builder[32], *pt, olddb[512], patch[32], broken[32];
 	const char *dbname, *newcvd;
-        struct cl_engine *engine = NULL;
+        struct cl_engine *engine;
 	FILE *cvd, *fh;
 	gzFile *tar;
 	time_t timet;
@@ -552,7 +563,7 @@ static int build(struct optstruct *opt)
 	struct cl_cvd *oldcvd;
 
 
-    if(!opt_check(opt, "server")) {
+    if(!optget(opts, "server")->enabled) {
 	mprintf("!build: --server is required for --build\n");
 	return -1;
     }
@@ -562,15 +573,20 @@ static int build(struct optstruct *opt)
 	return -1;
     }
 
-    dbname = strstr(opt_arg(opt, "build"), "main") ? "main" : "daily";
+    dbname = getdbname(optget(opts, "build")->strarg);
 
-    if((ret = cl_load(".", &engine, &sigs, CL_DB_STDOPT | CL_DB_PUA))) {
-	mprintf("!build: Can't load database: %s\n", cl_strerror(ret));
-	return -1;
-    } else {
-	cl_free(engine);
+    if(!(engine = cl_engine_new())) {
+	mprintf("!build: Can't initialize antivirus engine\n");
+	return 50;
     }
 
+    if((ret = cl_load(".", engine, &sigs, CL_DB_STDOPT | CL_DB_PUA))) {
+	mprintf("!build: Can't load database: %s\n", cl_strerror(ret));
+	cl_engine_free(engine);
+	return -1;
+    }
+    cl_engine_free(engine);
+
     if(!sigs) {
 	mprintf("!build: There are no signatures in database files\n");
     } else {
@@ -588,9 +604,9 @@ static int build(struct optstruct *opt)
     }
 
     /* try to read cvd header of current database */
-    if(opt->filename) {
-	if(cli_strbcasestr(opt->filename, ".cvd") || cli_strbcasestr(opt->filename, ".cld")) {
-	    strncpy(olddb, opt->filename, sizeof(olddb));
+    if(opts->filename) {
+	if(cli_strbcasestr(opts->filename[0], ".cvd") || cli_strbcasestr(opts->filename[0], ".cld")) {
+	    strncpy(olddb, opts->filename[0], sizeof(olddb));
 	    olddb[sizeof(olddb)-1]='\0';
 	} else {
 	    mprintf("!build: Not a CVD/CLD file\n");
@@ -735,7 +751,7 @@ static int build(struct optstruct *opt)
     sprintf(header + strlen(header), "%s:", pt);
     free(pt);
 
-    if(!(pt = getdsig(opt_arg(opt, "server"), builder, buffer, 16, 0))) {
+    if(!(pt = getdsig(optget(opts, "server")->strarg, builder, buffer, 16, 0))) {
 	mprintf("!build: Can't get digital signature from remote server\n");
 	fclose(fh);
 	unlink(tarfile);
@@ -756,7 +772,7 @@ static int build(struct optstruct *opt)
 	strcat(header, " ");
 
     /* build the final database */
-    newcvd = opt_arg(opt, "build");
+    newcvd = optget(opts, "build")->strarg;
     if(!(cvd = fopen(newcvd, "wb"))) {
 	mprintf("!build: Can't create final database %s\n", newcvd);
 	fclose(fh);
@@ -801,6 +817,11 @@ static int build(struct optstruct *opt)
 
     mprintf("Created %s\n", newcvd);
 
+    if(optget(opts, "no-cdiff")->enabled) {
+	mprintf("Skipping .cdiff creation\n");
+	return 0;
+    }
+
     /* generate patch */
     if(!(pt = cli_gentemp(NULL))) {
 	mprintf("!build: Can't generate temporary name\n");
@@ -815,7 +836,7 @@ static int build(struct optstruct *opt)
 	return -1;
     }
 
-    if(cvd_unpack(olddb, pt) == -1) {
+    if(cli_cvdunpack(olddb, pt) == -1) {
 	mprintf("!build: Can't unpack CVD file %s\n", olddb);
 	cli_rmdirs(pt);
 	free(pt);
@@ -841,7 +862,7 @@ static int build(struct optstruct *opt)
 	return -1;
     }
 
-    if(cvd_unpack(newcvd, pt) == -1) {
+    if(cli_cvdunpack(newcvd, pt) == -1) {
 	mprintf("!build: Can't unpack CVD file %s\n", newcvd);
 	cli_rmdirs(pt);
 	free(pt);
@@ -850,11 +871,7 @@ static int build(struct optstruct *opt)
 	return -1;
     }
 
-    if(!strcmp(dbname, "main"))
-	snprintf(patch, sizeof(patch), "main-%u.script", version);
-    else
-	snprintf(patch, sizeof(patch), "daily-%u.script", version);
-
+    snprintf(patch, sizeof(patch), "%s-%u.script", dbname, version);
     ret = diffdirs(olddb, pt, patch);
 
     cli_rmdirs(pt);
@@ -878,24 +895,24 @@ static int build(struct optstruct *opt)
 	    mprintf("!Generated file is incorrect, renamed to %s\n", broken);
 	}
     } else {
-	ret = script2cdiff(patch, builder, opt);
+	ret = script2cdiff(patch, builder, opts);
     }
 
     return ret;
 }
 
-static int unpack(struct optstruct *opt)
+static int unpack(const struct optstruct *opts)
 {
 	char name[512], *dbdir;
 
 
-    if(opt_check(opt, "unpack-current")) {
+    if(optget(opts, "unpack-current")->enabled) {
 	dbdir = freshdbdir();
-	snprintf(name, sizeof(name), "%s/%s.cvd", dbdir, opt_arg(opt, "unpack-current"));
+	snprintf(name, sizeof(name), "%s/%s.cvd", dbdir, optget(opts, "unpack-current")->strarg);
 	if(access(name, R_OK)) {
-	    snprintf(name, sizeof(name), "%s/%s.cld", dbdir, opt_arg(opt, "unpack-current"));
+	    snprintf(name, sizeof(name), "%s/%s.cld", dbdir, optget(opts, "unpack-current")->strarg);
 	    if(access(name, R_OK)) {
-		mprintf("!unpack: Couldn't find %s CLD/CVD database\n", opt_arg(opt, "unpack-current"));
+		mprintf("!unpack: Couldn't find %s CLD/CVD database\n", optget(opts, "unpack-current")->strarg);
 		free(dbdir);
 		return -1;
 	    }
@@ -903,11 +920,11 @@ static int unpack(struct optstruct *opt)
 	free(dbdir);
 
     } else {
-	strncpy(name, opt_arg(opt, "unpack"), sizeof(name));
+	strncpy(name, optget(opts, "unpack")->strarg, sizeof(name));
 	name[sizeof(name)-1]='\0';
     }
 
-    if(cvd_unpack(name, ".") == -1) {
+    if(cli_cvdunpack(name, ".") == -1) {
 	mprintf("!unpack: Can't unpack file %s\n", name);
 	return -1;
     }
@@ -915,14 +932,14 @@ static int unpack(struct optstruct *opt)
     return 0;
 }
 
-static int cvdinfo(struct optstruct *opt)
+static int cvdinfo(const struct optstruct *opts)
 {
 	struct cl_cvd *cvd;
 	char *pt;
 	int ret;
 
 
-    pt = opt_arg(opt, "info");
+    pt = optget(opts, "info")->strarg;
     if((cvd = cl_cvdhead(pt)) == NULL) {
 	mprintf("!cvdinfo: Can't read/parse CVD header of %s\n", pt);
 	return -1;
@@ -937,16 +954,11 @@ static int cvdinfo(struct optstruct *opt)
     mprintf("Functionality level: %u\n", cvd->fl);
     mprintf("Builder: %s\n", cvd->builder);
 
-    pt = opt_arg(opt, "info");
+    pt = optget(opts, "info")->strarg;
     if(cli_strbcasestr(pt, ".cvd")) {
 	mprintf("MD5: %s\n", cvd->md5);
 	mprintf("Digital signature: %s\n", cvd->dsig);
 	cl_cvdfree(cvd);
-
-#ifndef HAVE_LIBGMP
-	mprintf("^Digital signature support not compiled in.\n");
-#endif
-
 	if((ret = cl_cvdverify(pt))) {
 	    mprintf("!cvdinfo: Verification: %s\n", cl_strerror(ret));
 	    return -1;
@@ -1068,7 +1080,7 @@ static int listdb(const char *filename)
 	    return -1;
 	}
 
-	if(cvd_unpack(filename, dir) == -1) {
+	if(cli_cvdunpack(filename, dir) == -1) {
 	    mprintf("!listdb: Can't unpack CVD file %s\n", filename);
 	    cli_rmdirs(dir);
 	    free(dir);
@@ -1161,27 +1173,37 @@ static int listdb(const char *filename)
     return 0;
 }
 
-static int listsigs(struct optstruct *opt)
+static int listsigs(const struct optstruct *opts)
 {
 	int ret;
 	const char *name;
 	char *dbdir;
+	struct stat sb;
 
 
+    name = optget(opts, "list-sigs")->strarg;
+    if(stat(name, &sb) == -1) {
+	mprintf("--list-sigs: Can't get status of %s\n", name);
+	return -1;
+    }
+
     mprintf_stdout = 1;
-
-    if((name = opt_arg(opt, "list-sigs"))) {
-	ret = listdb(name);
+    if(S_ISDIR(sb.st_mode)) {
+	if(!strcmp(name, DATADIR)) {
+	    dbdir = freshdbdir();
+	    ret = listdir(dbdir);
+	    free(dbdir);
+	} else {
+	    ret = listdir(name);
+	}
     } else {
-	dbdir = freshdbdir();
-	ret = listdir(dbdir);
-	free(dbdir);
+	ret = listdb(name);
     }
 
     return ret;
 }
 
-static int vbadump(struct optstruct *opt)
+static int vbadump(const struct optstruct *opts)
 {
 	int fd, hex_output;
 	char *dir;
@@ -1189,12 +1211,12 @@ static int vbadump(struct optstruct *opt)
 	struct uniq *vba = NULL;
 
 
-    if(opt_check(opt, "vba-hex")) {
+    if(optget(opts, "vba-hex")->enabled) {
 	hex_output = 1;
-	pt = opt_arg(opt, "vba-hex");
+	pt = optget(opts, "vba-hex")->strarg;
     } else {
 	hex_output = 0;
-	pt = opt_arg(opt, "vba");
+	pt = optget(opts, "vba")->strarg;
     }
  
     if((fd = open(pt, O_RDONLY)) == -1) {
@@ -1232,15 +1254,12 @@ static int vbadump(struct optstruct *opt)
 
 static int comparemd5(const char *dbname)
 {
-	char info[16], buff[256], *md5, *pt;
+	char info[32], buff[256], *md5, *pt;
 	FILE *fh;
 	int ret = 0;
 
 
-    if(strstr(dbname, "main"))
-	strcpy(info, "main.info");
-    else
-	strcpy(info, "daily.info");
+    snprintf(info, sizeof(info), "%s.info", getdbname(dbname));
 
     if(!(fh = fopen(info, "r"))) {
 	mprintf("!verifydiff: Can't open %s\n", info);
@@ -1280,14 +1299,14 @@ static int comparemd5(const char *dbname)
 }
 
 
-static int rundiff(struct optstruct *opt)
+static int rundiff(const struct optstruct *opts)
 {
 	int fd, ret;
 	unsigned short mode;
 	const char *diff;
 
 
-    diff = opt_arg(opt, "run-cdiff");
+    diff = optget(opts, "run-cdiff")->strarg;
     if(strstr(diff, ".cdiff")) {
 	mode = 1;
     } else if(strstr(diff, ".script")) {
@@ -1314,7 +1333,7 @@ static int rundiff(struct optstruct *opt)
 static int compare(const char *oldpath, const char *newpath, FILE *diff)
 {
 	FILE *old, *new;
-	char obuff[1024], nbuff[1024], tbuff[1024], *pt, *omd5, *nmd5;
+	char obuff[CLI_DEFAULT_LSIG_BUFSIZE + 1], nbuff[CLI_DEFAULT_LSIG_BUFSIZE + 1], tbuff[CLI_DEFAULT_LSIG_BUFSIZE + 1], *pt, *omd5, *nmd5;
 	unsigned int oline = 0, tline, found, i;
 	long opos;
 
@@ -1344,8 +1363,14 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
     old = fopen(oldpath, "r");
 
     while(fgets(nbuff, sizeof(nbuff), new)) {
+	i = strlen(nbuff);
+	if(i >= 2 && (nbuff[i - 1] == '\r' || (nbuff[i - 1] == '\n' && nbuff[i - 2] == '\r'))) {
+	    mprintf("!compare: New %s file contains lines terminated with CRLF or CR\n", newpath);
+	    if(old)
+		fclose(old);
+	    return -1;
+	}
 	cli_chomp(nbuff);
-
 	if(!old) {
 	    fprintf(diff, "ADD %s\n", nbuff);
 	} else {
@@ -1398,6 +1423,14 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
 		fprintf(diff, "ADD %s\n", nbuff);
 	    }
 	}
+#ifdef COMPATIBILITY_LIMIT
+       if(strlen(nbuff) > COMPATIBILITY_LIMIT) {
+           mprintf("!compare: COMPATIBILITY_LIMIT: Found too long line in new %s\n", newpath);
+           if(old)
+               fclose(old);
+           return -1;
+       }
+#endif
     }
 
     if(old) {
@@ -1415,6 +1448,50 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
     return 0;
 }
 
+static int dircopy(const char *src, const char *dest)
+{
+	DIR *dd;
+	struct dirent *dent;
+	struct stat sb;
+	char spath[512], dpath[512];
+
+
+    if(stat(dest, &sb) == -1) {
+	if(mkdir(dest, 0755)) {
+	    /* mprintf("!dircopy: Can't create temporary directory %s\n", dest); */
+	    return -1;
+	}
+    }
+
+    if((dd = opendir(src)) == NULL) {
+        /* mprintf("!dircopy: Can't open directory %s\n", src); */
+        return -1;
+    }
+
+    while((dent = readdir(dd))) {
+#if (!defined(C_INTERIX)) && (!defined(C_WINDOWS))
+	if(dent->d_ino)
+#endif
+	{
+	    if(!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
+		continue;
+
+	    snprintf(spath, sizeof(spath), "%s/%s", src, dent->d_name);
+	    snprintf(dpath, sizeof(dpath), "%s/%s", dest, dent->d_name);
+
+	    if(filecopy(spath, dpath) == -1) {
+		/* mprintf("!dircopy: Can't copy %s to %s\n", spath, dpath); */
+		cli_rmdirs(dest);
+		closedir(dd);
+		return -1;
+	    }
+	}
+    }
+
+    closedir(dd);
+    return 0;
+}
+
 static int verifydiff(const char *diff, const char *cvd, const char *incdir)
 {
 	char *tempdir, cwd[512];
@@ -1444,7 +1521,7 @@ static int verifydiff(const char *diff, const char *cvd, const char *incdir)
     }
 
     if(cvd) {
-	if(cvd_unpack(cvd, tempdir) == -1) {
+	if(cli_cvdunpack(cvd, tempdir) == -1) {
 	    mprintf("!verifydiff: Can't unpack CVD file %s\n", cvd);
 	    cli_rmdirs(tempdir);
 	    free(tempdir);
@@ -1549,6 +1626,8 @@ static int diffdirs(const char *old, const char *new, const char *patch)
 
 	    snprintf(path, sizeof(path), "%s/%s", old, dent->d_name);
 	    if(compare(path, dent->d_name, diff) == -1) {
+		if(chdir(cwd) == -1)
+		    mprintf("^diffdirs: Can't chdir to %s\n", cwd);
 		fclose(diff);
 		unlink(patch);
 		closedir(dd);
@@ -1588,7 +1667,7 @@ static int diffdirs(const char *old, const char *new, const char *patch)
     return 0;
 }
 
-static int makediff(struct optstruct *opt)
+static int makediff(const struct optstruct *opts)
 {
 	char *odir, *ndir, name[32], broken[32];
 	struct cl_cvd *cvd;
@@ -1596,20 +1675,20 @@ static int makediff(struct optstruct *opt)
 	int ret;
 
 
-    if(!opt->filename) {
+    if(!opts->filename) {
 	mprintf("!makediff: --diff requires two arguments\n");
 	return -1;
     }
 
-    if(!(cvd = cl_cvdhead(opt->filename))) {
-	mprintf("!makediff: Can't read CVD header from %s\n", opt->filename);
+    if(!(cvd = cl_cvdhead(opts->filename[0]))) {
+	mprintf("!makediff: Can't read CVD header from %s\n", opts->filename[0]);
 	return -1;
     }
     newver = cvd->version;
     free(cvd);
 
-    if(!(cvd = cl_cvdhead(opt_arg(opt, "diff")))) {
-	mprintf("!makediff: Can't read CVD header from %s\n", opt_arg(opt, "diff"));
+    if(!(cvd = cl_cvdhead(optget(opts, "diff")->strarg))) {
+	mprintf("!makediff: Can't read CVD header from %s\n", optget(opts, "diff")->strarg);
 	return -1;
     }
     oldver = cvd->version;
@@ -1632,8 +1711,8 @@ static int makediff(struct optstruct *opt)
 	return -1;
     }
 
-    if(cvd_unpack(opt_arg(opt, "diff"), odir) == -1) {
-	mprintf("!makediff: Can't unpack CVD file %s\n", opt_arg(opt, "diff"));
+    if(cli_cvdunpack(optget(opts, "diff")->strarg, odir) == -1) {
+	mprintf("!makediff: Can't unpack CVD file %s\n", optget(opts, "diff")->strarg);
 	cli_rmdirs(odir);
 	free(odir);
 	return -1;
@@ -1655,8 +1734,8 @@ static int makediff(struct optstruct *opt)
 	return -1;
     }
 
-    if(cvd_unpack(opt->filename, ndir) == -1) {
-	mprintf("!makediff: Can't unpack CVD file %s\n", opt->filename);
+    if(cli_cvdunpack(opts->filename[0], ndir) == -1) {
+	mprintf("!makediff: Can't unpack CVD file %s\n", opts->filename[0]);
 	cli_rmdirs(odir);
 	cli_rmdirs(ndir);
 	free(odir);
@@ -1664,11 +1743,7 @@ static int makediff(struct optstruct *opt)
 	return -1;
     }
 
-    if(strstr(opt->filename, "main"))
-	snprintf(name, sizeof(name), "main-%u.script", newver);
-    else
-	snprintf(name, sizeof(name), "daily-%u.script", newver);
-
+    snprintf(name, sizeof(name), "%s-%u.script", getdbname(opts->filename[0]), newver);
     ret = diffdirs(odir, ndir, name);
 
     cli_rmdirs(odir);
@@ -1679,7 +1754,7 @@ static int makediff(struct optstruct *opt)
     if(ret == -1)
 	return -1;
 
-    if(verifydiff(name, opt_arg(opt, "diff"), NULL) == -1) {
+    if(verifydiff(name, optget(opts, "diff")->strarg, NULL) == -1) {
 	snprintf(broken, sizeof(broken), "%s.broken", name);
 	if(rename(name, broken)) {
 	    unlink(name);
@@ -1697,7 +1772,8 @@ static void help(void)
 {
     mprintf("\n");
     mprintf("             Clam AntiVirus: Signature Tool (sigtool)  %s\n", get_version());
-    mprintf("    (C) 2002 - 2007 ClamAV Team - http://www.clamav.net/team\n\n");
+    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
+    printf("           (C) 2007-2009 Sourcefire, Inc. et al.\n\n");
 
     mprintf("    --help                 -h              show help\n");
     mprintf("    --version              -V              print version number and exit\n");
@@ -1713,6 +1789,7 @@ static void help(void)
     mprintf("    --utf16-decode=FILE                    decode UTF16 encoded files\n");
     mprintf("    --info=FILE            -i FILE         print database information\n");
     mprintf("    --build=NAME [cvd] -b NAME             build a CVD file\n");
+    mprintf("    --no-cdiff                             Don't generate .cdiff file\n");
     mprintf("    --server=ADDR                          ClamAV Signing Service address\n");
     mprintf("    --unpack=FILE          -u FILE         Unpack a CVD/CLD file\n");
     mprintf("    --unpack-current=SHORTNAME             Unpack local CVD/CLD into cwd\n");
@@ -1730,107 +1807,80 @@ static void help(void)
 int main(int argc, char **argv)
 {
 	int ret = 1;
-        struct optstruct *opt;
+        struct optstruct *opts;
 	struct stat sb;
-	const char *short_options = "hvVb:i:u:l::r:d:";
-	static struct option long_options[] = {
-	    {"help", 0, 0, 'h'},
-	    {"quiet", 0, 0, 0},
-	    {"debug", 0, 0, 0},
-	    {"verbose", 0, 0, 'v'},
-	    {"stdout", 0, 0, 0},
-	    {"version", 0, 0, 'V'},
-	    {"tempdir", 1, 0, 0},
-	    {"hex-dump", 0, 0, 0},
-	    {"md5", 0, 0, 0},
-	    {"mdb", 0, 0, 0},
-	    {"html-normalise", 1, 0, 0},
-	    {"utf16-decode", 1, 0, 0},
-	    {"build", 1, 0, 'b'},
-	    {"server", 1, 0, 0},
-	    {"unpack", 1, 0, 'u'},
-	    {"unpack-current", 1, 0, 0},
-	    {"info", 1, 0, 'i'},
-	    {"list-sigs", 2, 0, 'l'},
-	    {"vba", 1, 0 ,0},
-	    {"vba-hex", 1, 0, 0},
-	    {"diff", 1, 0, 'd'},
-	    {"run-cdiff", 1, 0, 'r'},
-	    {"verify-cdiff", 1, 0, 0},
-	    {0, 0, 0, 0}
-    	};
 
-    opt = opt_parse(argc, argv, short_options, long_options, NULL);
-    if(!opt) {
-	mprintf("!Can't parse the command line\n");
+    opts = optparse(NULL, argc, argv, 1, OPT_SIGTOOL, 0, NULL);
+    if(!opts) {
+	mprintf("!Can't parse command line options\n");
 	return 1;
     }
 
-    if(opt_check(opt, "quiet"))
+    if(optget(opts, "quiet")->enabled)
 	mprintf_quiet = 1;
 
-    if(opt_check(opt, "stdout"))
+    if(optget(opts, "stdout")->enabled)
 	mprintf_stdout = 1;
 
-    if(opt_check(opt, "debug"))
+    if(optget(opts, "debug")->enabled)
 	cl_debug();
 
-    if(opt_check(opt, "version")) {
+    if(optget(opts, "version")->enabled) {
 	print_version(NULL);
-	opt_free(opt);
+	optfree(opts);
 	return 0;
     }
 
-    if(opt_check(opt, "help")) {
-	opt_free(opt);
+    if(optget(opts, "help")->enabled) {
+	optfree(opts);
     	help();
 	return 0;
     }
 
-    if(opt_check(opt, "hex-dump"))
+    if(optget(opts, "hex-dump")->enabled)
 	ret = hexdump();
-    else if(opt_check(opt, "md5"))
-	ret = md5sig(opt, 0);
-    else if(opt_check(opt, "mdb"))
-	ret = md5sig(opt, 1);
-    else if(opt_check(opt, "html-normalise"))
-	ret = htmlnorm(opt);
-    else if(opt_check(opt, "utf16-decode"))
-	ret = utf16decode(opt);
-    else if(opt_check(opt, "build"))
-	ret = build(opt);
-    else if(opt_check(opt, "unpack"))
-	ret = unpack(opt);
-    else if(opt_check(opt, "unpack-current"))
-	ret = unpack(opt);
-    else if(opt_check(opt, "info"))
-	ret = cvdinfo(opt);
-    else if(opt_check(opt, "list-sigs"))
-	ret = listsigs(opt);
-    else if(opt_check(opt, "vba") || opt_check(opt, "vba-hex"))
-	ret = vbadump(opt);
-    else if(opt_check(opt, "diff"))
-	ret = makediff(opt);
-    else if(opt_check(opt, "run-cdiff"))
-	ret = rundiff(opt);
-    else if(opt_check(opt, "verify-cdiff")) {
-	if(!opt->filename) {
+    else if(optget(opts, "md5")->enabled)
+	ret = md5sig(opts, 0);
+    else if(optget(opts, "mdb")->enabled)
+	ret = md5sig(opts, 1);
+    else if(optget(opts, "html-normalise")->enabled)
+	ret = htmlnorm(opts);
+    else if(optget(opts, "utf16-decode")->enabled)
+	ret = utf16decode(opts);
+    else if(optget(opts, "build")->enabled)
+	ret = build(opts);
+    else if(optget(opts, "unpack")->enabled)
+	ret = unpack(opts);
+    else if(optget(opts, "unpack-current")->enabled)
+	ret = unpack(opts);
+    else if(optget(opts, "info")->enabled)
+	ret = cvdinfo(opts);
+    else if(optget(opts, "list-sigs")->active)
+	ret = listsigs(opts);
+    else if(optget(opts, "vba")->enabled || optget(opts, "vba-hex")->enabled)
+	ret = vbadump(opts);
+    else if(optget(opts, "diff")->enabled)
+	ret = makediff(opts);
+    else if(optget(opts, "run-cdiff")->enabled)
+	ret = rundiff(opts);
+    else if(optget(opts, "verify-cdiff")->enabled) {
+	if(!opts->filename) {
 	    mprintf("!--verify-cdiff requires two arguments\n");
 	    ret = -1;
 	} else {
-	    if(stat(opt->filename, &sb) == -1) {
-		mprintf("--verify-cdiff: Can't get status of %s\n", opt->filename);
+	    if(stat(opts->filename[0], &sb) == -1) {
+		mprintf("--verify-cdiff: Can't get status of %s\n", opts->filename[0]);
 		ret = -1;
 	    } else {
 		if(S_ISDIR(sb.st_mode))
-		    ret = verifydiff(opt_arg(opt, "verify-cdiff"), NULL, opt->filename);
+		    ret = verifydiff(optget(opts, "verify-cdiff")->strarg, NULL, opts->filename[0]);
 		else
-		    ret = verifydiff(opt_arg(opt, "verify-cdiff"), opt->filename, NULL);
+		    ret = verifydiff(optget(opts, "verify-cdiff")->strarg, opts->filename[0], NULL);
 	    }
 	}
     } else
 	help();
 
-    opt_free(opt);
+    optfree(opts);
     return ret ? 1 : 0;
 }
