@@ -1,5 +1,6 @@
 #!/bin/sh
 # Run under duma
+test "x$RUNDUMA" = "x1" || { echo "*** duma tests skipped by default, use 'make check RUNDUMA=1' to activate (but don't report bugs about timeouts!)"; exit 77; }
 LIBDIRS=`../libtool --config | grep sys_lib_search_path_spec | sed -e 's/.*"\(.*\)"/\1/'`
 if test -z "$LIBDUMA"; then
 	for i in $LIBDIRS; do
@@ -16,34 +17,27 @@ DUMA_OUTPUT_FILE=duma.log
 DUMA_DISABLE_BANNER=1
 LIBPRELOAD="$LIBDUMA"
 rm -f duma.log
+export CK_DEFAULT_TIMEOUT=40
 export DUMA_FILL DUMA_MALLOC_0_STRATEGY DUMA_OUTPUT_FILE DUMA_DISABLE_BANNER LIBPRELOAD
 echo "--- starting clamd under duma to detect overruns"
-CLAMD_WRAPPER=$abs_srcdir/preload_run.sh $abs_srcdir/check_clamd.sh &
-pid1=$!
+CLAMD_WRAPPER=$abs_srcdir/preload_run.sh $abs_srcdir/check_clamd.sh
+if test $? -ne 0; then
+	echo "*** DUMA has detected errors"
+	cat duma.log
+	rm -f duma.log duma2.log
+	exit 1
+fi
 
 echo "--- starting clamd under duma to detect underruns"
 DUMA_OUTPUT_FILE=duma2.log
 DUMA_PROTECT_BELOW=1
 export DUMA_PROTECT_BELOW
 rm -f duma2.log
-CLAMD_TEST_UNIQ1=3 CLAMD_TEST_UNIQ2=4 CLAMD_WRAPPER=$abs_srcdir/preload_run.sh $abs_srcdir/check_clamd.sh&
-pid2=$!
-
-wait $pid1
-exitcode1=$?
-wait $pid2
-exitcode2=$?
-if test $exitcode1 -ne 0; then
-	echo "*** DUMA has detected errors"
-	cat duma.log
-	rm -f duma.log duma2.log
-	exit 1
-fi
-if test $exitcode2 -ne 0; then
+CLAMD_TEST_UNIQ1=3 CLAMD_TEST_UNIQ2=4 CLAMD_WRAPPER=$abs_srcdir/preload_run.sh $abs_srcdir/check_clamd.sh
+if test $? -ne 0; then
 	echo "*** DUMA has detected errors"
 	cat duma2.log
 	rm -f duma.log duma2.log
 	exit 1
 fi
 exit 0
-
